@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * bftextview2.h
  *
- * Copyright (C) 2008,2009,2010,2011 Olivier Sessink
+ * Copyright (C) 2008,2009,2010,2011,2012 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -261,107 +261,7 @@ The slave widget should always be destroyed before the master.
 #include <gtk/gtk.h>
 #include "config.h"
 
-#if defined(__GNUC__) || (defined(__SUNPRO_C) && __SUNPRO_C > 0x580)
-#define DBG_NONE(args...)
- /**/
-#else							/* notdef __GNUC__ || __SUNPRO_C */
-extern void g_none(char *first, ...);
-#define DBG_NONE g_none
-#endif
- /**/
 #define IDENTSTORING
-#define BF2_OFFSET_UNDEFINED G_MAXUINT32
-
-/*#define DUMP_SCANCACHE*/
-/*#define CHECK_CONSISTENCY*/
-
-#define DBG_MSG DBG_NONE
-#define DBG_SCANCACHE DBG_NONE
-#define DBG_SCANNING DBG_NONE
-#define DBG_BLOCKMATCH DBG_NONE
-#define DBG_PATCOMPILE DBG_NONE
-#define DBG_SIGNALS DBG_NONE
-#define DBG_AUTOCOMP DBG_NONE
-#define DBG_DELAYSCANNING DBG_NONE
-#define DBG_FOLD DBG_NONE
-#define DBG_MARGIN DBG_NONE
-#define DBG_PARSING DBG_NONE
-#define DBG_TOOLTIP DBG_NONE
-#define DBG_SPELL DBG_NONE
-#define DBG_IDENTIFIER DBG_NONE
-#define NUMSCANCHARS 127		/* 128 is ascii, but the last character is never scanned (DEL)
-								   and the Ttablerow has one more 16bit value. By setting this to 127 instead of 128
-								   we dont need padding to align the Ttablerow in memory
-								   (Ttablerow = (127+1)*16=2048 bits or 256 bytes) */
-/*****************************************************************/
-/* building the automata and autocompletion cache */
-/*****************************************************************/
-#define COMMENT_INDEX_INHERIT 255
-#define COMMENT_INDEX_NONE 254
-	typedef struct {
-	GArray *table; /* a pointer to the DFA table for this context */ 
-	GCompletion *ac;			/* autocompletion items in this context */
-	/*GHashTable *reference; *//* reference help for each autocompletion item */
-	GHashTable *patternhash;	/* a hash table where the pattern and its autocompletion string are the keys, and an integer to the ID of the pattern is the value */
-	GtkTextTag *contexttag;		/* if the context area itself needs some kind of style (to implement a string context for example) */
-	gchar *contexthighlight;	/* the string that has the id for the highlight */
-	/*guint16 startstate;*/			/* refers to the row number in scantable->table that is the start state for this context */
-	/*guint16 identstate;*/			/* refers to the row number in scantable->table that is the identifier-state
-								   for this context. The identifier state is a state that refers to itself for all characters
-								   except the characters (symbols) thay may be the begin or end of an identifier such
-								   as whitespace, ();[]{}*+-/ etc. */
-	guint8 has_tagclose_from_blockstack;	/* this context has xml end patterns that need autoclosing for generix xml tags, based on the tag that is on top of the blockstack */
-	guint8 comment_block;		/* block comment index in array scantable->comments 
-								   or COMMENT_INDEX_INHERIT (which means inherit) 
-								   or COMMENT_INDEX_NONE if there is no block comment  */
-	guint8 comment_line;		/* index in array scantable->comments for line comments; see comment_block */
-	guint8 autocomplete_case_insens;
-} Tcontext;
-
-typedef struct {
-	gchar *autocomplete_string;
-	guint8 autocomplete_backup_cursor;	/* number of characters to backup the cursor after autocompletion (max 256) */
-	/* guint8 autocomplete;whether or not this pattern should be added to the autocompletion; stored in the Tpattern so we can re-use it in another context */
-} Tpattern_autocomplete;
-
-typedef struct {
-	GtkTextTag *selftag;		/* the tag used to highlight this pattern */
-	GtkTextTag *blocktag;		/* if this pattern ends a context or a block, we can highlight
-								   the region within the start and end pattern with this tag */
-	gchar *reference;			/* the reference data, or NULL. may be inserted in hash tables for multiple keys in multiple contexts */
-	gchar *pattern;				/* the pattern itself. stored in the Tpattern so we can re-use it in another context */
-	GSList *autocomp_items;
-	gchar *selfhighlight;		/* a string with the highlight for this pattern. used when re-linking highlights and textstyles 
-								   if the user changed any of these in the preferences */
-	gchar *blockhighlight;		/* a string for the highlight corresponding to the  blocktag */
-
-	gint16 blockstartpattern;	/* the number of the pattern that may start this block, or -1 to end the last started block */
-	gint16 nextcontext;			/* 0, or if this pattern starts a new context the number of the context, or -1 or -2 etc.
-								   to pop a context of the stack */
-	/* TODO:  all following guint8 entries are booleans that can be combined into a single 8 bits integer with a bitmask */
-	guint8 starts_block;		/* wether or not this pattern may start a block */
-	guint8 ends_block;			/* wether or not this pattern may end a block */
-	guint8 case_insens;
-	guint8 is_regex;
-	guint8 tagclose_from_blockstack;	/* this is a generix xml close tag that needs the blockstack to autoclose */
-	guint8 stretch_blockstart; /* the end of this match is the new end-of-blockstart, used for HTML/XML tags */
-#ifdef IDENTSTORING
-	guint8 identmode;
-	guint8 identaction; /* bitwise, first bit is add to jump hashtable, second bit is autocomplete */
-#endif							/* IDENTSTORING */
-} Tpattern;
-/* 
-32bit size = 7 * 32 + 2 * 16 + 8 * 8 = 320 bits = 40 bytes 
-64bit size = 7 * 64 + 2 * 16 + 8 * 8 = 544 bits = 68 bytes
-*/
-
-typedef struct {
-	guint16 row[NUMSCANCHARS];	/* contains for each character the number of the next state
-								   because we use a 16bit unsigned number we can support only 65535 states
-								   at maximum!!!!!!! but we use half the size of the scanning table, which
-								   hopefully helps to keep the scanning table in the L2 cache of the CPU */
-	guint16 match;				/* 0 == no match, refers to the index number in array 'matches' */
-} Ttablerow;					/* a row in the DFA, right now exactly 256 bytes */
 
 typedef enum {
 	comment_type_block,
@@ -378,86 +278,14 @@ typedef struct {
 	guint8 allsymbols[128];		/* this lookup table holds all symbols for all contexts, and is used to trigger scanning if reduced_scan_triggers is enabled */
 	GArray *contexts;			/* dynamic sized array of Tcontext that translates a context number into a rownumber in the DFA table */
 	GArray *matches;			/* dynamic sized array of Tpattern */
-	GArray *comments;			/* Tcomment, has max. 256 entries, we use a guint8 as index */
+	GArray *comments;			/* array of Tcomment, has max. 256 entries, we use a guint8 as index */
+	GArray *blocks; 			/* array of Tpattern_block with a guint16 as index */
 } Tscantable;
 
-#define character_is_symbol(st,context,c) (g_array_index((GArray *)g_array_index(st->contexts, Tcontext, context).table, Ttablerow, 1).row[c] != 1)
-
-#define get_table(scantable, context) ((GArray *)g_array_index(scantable->contexts, Tcontext, context).table)
-
-#define get_tablerow(scantable, context, curstate) (g_array_index(g_array_index(scantable->contexts, Tcontext, context).table, Ttablerow, curstate))
-
-/*****************************************************************/
-/* scanning the text and caching the results */
-/*****************************************************************/
-typedef struct {
-	gpointer parentfblock;
-	guint32 start1_o;
-	guint32 end1_o;
-	guint32 start2_o;
-	guint32 end2_o;
-	gint16 patternum;			/* which pattern (number of the array element in scantable->matches) */
-	guint8 folded;
-	guint8 foldable;			/* FALSE on a single line */
-} Tfoundblock;					/* Once a start-of-block is found start1 and end1 are set
-								   and the Tfoundblock is added to the foundcache.
-								   The previous foundblock is set as parentfblock
-								   so we can later on find what the current blockstack looks like.
-								   Once the end-of-block is found, start2 and end2 are set
-								   The Tfoundblock is popped as current block, and the parent
-								   is active again. This is also put on the foundcache
-
-								   on 64bit this type has size 8+4+4+4+4+2+1+1 + 4 padding = 32 bytes
-								   on 32bit this type has size 4+4+4+4+4+2+1+1 NO padding = 24 bytes
-								 */
-
-typedef struct {
-	gpointer parentfcontext;
-	guint32 start_o;
-	guint32 end_o;
-	gint16 context;				/* number of the element in scantable->contexts */
-} Tfoundcontext;				/* Once a start-of-context is found start is set
-								   and the Tfoundcontext is added to the current foundcache.
-								   The previous fcontext is set in parentfcontext
-								   so we can later on find what the current contextstack looks like.
-								   once the end-of-context is found, end is set
-								   The Tfoundcontext is popped from the current stack and
-								   this entry is also added to the foundcache
-
-								   on 64bit this type has size 8+4+4+2 + 6 padding = 24 bytes
-								   on 32bit this type has size 4+4+4+2 + 2 padding = 16 bytes
-								 */
-
-typedef struct {
-	Tfoundcontext *fcontext;	/* if numcontextchange == 0 this points to the current active context
-								   if numcontextchange > 0 this points to the pushed context, which also happens to be the current context
-								   if numcontextchange < 0 this points to the top of the stack at this position, to get the current position 
-								   you'll have to pop N items (where N is -1 * numcontextchange). */
-	Tfoundblock *fblock;		/* if numblockchange == 0 this points to the current active block
-								   if numblockchange > 0 this points to the pushed block, which also happens to be the current block
-								   if numblockchange < 0 this points to the top of the stack at this position, to get the current position 
-								   you'll have to pop N items (where N is -1 * numblockchange). */
-	guint32 charoffset_o;
-	gint16 numblockchange;		/* there are files that have > 127 pops in a single position
-								   for example html files that don't close paragrahs or tablerows */
-	gint8 numcontextchange;		/* 0 means no change, 1 means 1 push, -2 means 2 popped etc. */
-} Tfound;						/*
-								   on 64bit this type has size 8+8+4+2+1 + 1 padding = 24 bytes
-								   on 32bit this type has size 4+4+4+2+1 + 1 padding = 16 bytes
-								 */
-
-
-#define IS_FOUNDMODE_CONTEXTPUSH(i)   (i->numcontextchange > 0)
-#define IS_FOUNDMODE_CONTEXTPOP(i)   (i->numcontextchange < 0)
-#define IS_FOUNDMODE_BLOCKPUSH(i)   (i->numblockchange ==1)
-#define IS_FOUNDMODE_BLOCKPOP(i)   (i->numblockchange < 0)
-
 typedef struct {
 	GSequence *foundcaches;		/* a sorted structure of Tfound for
 								   each position where the stack changes so we can restart scanning
 								   on any location */
-	guint32 valid_cache_offset;	/* from 0 to this offset the cache is considered valid. after this 
-								   position the cache could be outdated */
 } Tscancache;
 /********************************/
 /* language manager */
@@ -493,6 +321,8 @@ typedef enum {
 	BTV_COLOR_CURRENT_LINE,
 	BTV_COLOR_RIGHT_MARGIN,
 	BTV_COLOR_CURSOR,
+	BTV_COLOR_SELECTION,
+	BTV_COLOR_CURSOR_HIGHLIGHT,
 /*	BTV_COLOR_SEARCH_BG,
 	BTV_COLOR_SEARCH_FG,*/
 	BTV_COLOR_COUNT
@@ -526,6 +356,7 @@ struct _BluefishTextView {
 	GtkTextTag *needspellcheck;
 #endif							/*HAVE_LIBENCHANT */
 	GtkTextTag *blockmatch;
+	GtkTextTag *cursortag;
 	Tscancache scancache;
 
 	guint scanner_immediate; /* event ID for the high priority scanning run */
@@ -545,6 +376,7 @@ struct _BluefishTextView {
 	gboolean needs_autocomp;	/* a state of the widget, autocomplete is needed on user keyboard actions */
 	gboolean needs_blockmatch;	/* a state of the widget, if the cursor position was changed */
 	gboolean key_press_inserted_char;	/* FALSE if the key press was used by autocomplete popup, or simply not in our widget */
+	gdouble button_press_line; /* line location of the button press, used in the release */
 	/*gboolean key_press_was_autocomplete;  a state of the widget, if the last keypress was handled by the autocomplete popup window */
 	gboolean showing_blockmatch;	/* a state of the widget if we are currently showing a blockmatch */
 	gboolean insert_was_auto_indent;	/* a state of the widget if the last keypress (enter) caused 
@@ -589,6 +421,7 @@ void bluefish_text_view_set_colors(BluefishTextView * btv, gchar * const *colors
 void bluefish_text_view_select_language(BluefishTextView * btv, const gchar * mime, const gchar * filename);
 gboolean bluefish_text_view_get_show_blocks(BluefishTextView * btv);
 void bluefish_text_view_set_show_blocks(BluefishTextView * btv, gboolean show);
+void bluefish_text_view_set_show_symbols_redraw(BluefishTextView * btv, gboolean show);
 gboolean bluefish_text_view_get_show_line_numbers(BluefishTextView * btv);
 void bluefish_text_view_set_show_line_numbers(BluefishTextView * btv, gboolean show);
 gboolean bluefish_text_view_get_show_visible_spacing(BluefishTextView * btv);
