@@ -24,6 +24,7 @@
 #include <stdlib.h>				/* qsort */
 
 #include "bftextview2.h"
+#include "bftextview2_private.h"
 #include "bftextview2_langmgr.h"
 #include "bftextview2_patcompile.h"
 
@@ -58,6 +59,18 @@ typedef struct {
 #endif
 } Tbflangparsing;
 
+typedef enum {
+	ac_mode_none,
+	ac_mode_string,
+	ac_mode_append
+} Tautocomplete_mode;
+
+typedef struct {
+	gchar *string;
+	Tautocomplete_mode mode;
+	gint backup_cursor;
+} Tautocomplete;
+
 typedef struct {
 	GHashTable *bflang_lookup;
 	GList *bflang_list;
@@ -204,9 +217,11 @@ langmgr_reload_user_styles(void)
 	gint i=0;
 	GtkTextTag *tag;
 	const gchar *defaultarr[][7] = {
+		/* name, foreground-color, background-color, bold, italics, spell-check */
 		{"preprocessor", "#aaaa00", "", "0", "0", "0", NULL},
 		{"comment", "#555555", "", "0", "1", "1", NULL},
 		{"string", "#009900", "", "0", "0", "1", NULL},
+		{"string-no-spell-check", "#009900", "", "0", "0", "0", NULL},
 		{"special-string", "#005400", "", "0", "0", "1", NULL},
 		{"special-string2", "#00D000", "", "0", "0", "1", NULL},
 		{"special-string3", "#999900", "", "0", "0", "1", NULL},
@@ -228,7 +243,7 @@ langmgr_reload_user_styles(void)
 		{"special-attribute", "#FF0000", "", "", "", "0", NULL},
 		{"brackets", "#000000", "", "1", "0", "0", NULL},
 		{"warning", "#FF0000", "", "1", "0", "0", NULL},
-		{"special-warning", "", "#FF0000", "", "", "", NULL},
+		{"special-warning", "", "#FF0000", "", "", "0", NULL},
 		{NULL, NULL, NULL, NULL, NULL, NULL, NULL}
 	};
 	/* because the order of the styles is important (last added GtkTextTag is most important)
@@ -288,11 +303,25 @@ lookup_user_option(const gchar * lang, const gchar * option)
 	return NULL;
 }
 
+static gboolean
+lookup_block_foldable(const gchar *lang_name, const gchar *block_name)
+{
+	gchar *name;
+	const gchar *tmp;
+	gboolean foldable;
+	name = g_strconcat(block_name, "_foldable", NULL);
+	tmp = lookup_user_option(lang_name, name);
+	foldable = !(tmp && tmp[0] == '0');
+	g_free(name);
+	return foldable;
+}
+
 void
 langmgr_reload_user_options(void)
 {
 	GList *tmplist;
 	const gchar *tmp;
+	gint i;
 	g_hash_table_remove_all(langmgr.bflang_options);
 	for (tmplist = g_list_first(main_v->props.bflang_options); tmplist; tmplist = tmplist->next) {
 		gchar **arr = (gchar **) tmplist->data;
@@ -305,6 +334,17 @@ langmgr_reload_user_options(void)
 	while (tmplist) {
 		tmp = lookup_user_option(BFLANG(tmplist->data)->name, "show_in_menu");
 		BFLANG(tmplist->data)->in_menu = !(tmp && tmp[0] == '0');
+		if (BFLANG(tmplist->data)->st) {
+			for (i = 1; i < BFLANG(tmplist->data)->st->blocks->len; i++) {
+				if (g_array_index(BFLANG(tmplist->data)->st->blocks, Tpattern_block, i).name) {
+					g_array_index(BFLANG(tmplist->data)->st->blocks, Tpattern_block, i).foldable 
+							= lookup_block_foldable(
+									BFLANG(tmplist->data)->name,
+									g_array_index(BFLANG(tmplist->data)->st->blocks, Tpattern_block, i).name
+									);
+				}
+			}
+		}
 		tmplist = tmplist->next;
 	}
 }
@@ -454,13 +494,21 @@ process_header(xmlTextReaderPtr reader, Tbflang * bflang)
 {
 	const gchar *tmp;
 	while (xmlTextReaderRead(reader) == 1) {
-		xmlChar *name = xmlTextReaderName(reader);
+		int nodetype = xmlTextReaderNodeType(reader);
+		xmlChar *name;
+		
+		if (nodetype == 14 /* #text */) {
+			continue;
+		}
+		
+		name = xmlTextReaderName(reader);
 		if (xmlStrEqual(name, (xmlChar *) "mime")) {
 			gchar *mimetype=NULL;
 			Tattrib attribs[] = {{"type", &mimetype, attribtype_string}};
 			parse_attributes(reader, attribs, 1);
-			if (mimetype)
+			if (mimetype) {
 				bflang->mimetypes = g_list_prepend(bflang->mimetypes, mimetype);
+			}
 		} else if (xmlStrEqual(name, (xmlChar *) "option")) {
 			gchar *optionname = NULL, *description = NULL;
 			gboolean defaultval = FALSE;
@@ -572,25 +620,104 @@ do_parse(Tbflangparsing * bfparser, const gchar *class, const gchar *notclass)
 	return TRUE;
 }
 
+static void
+free_autocomplete(GSList *list)
+{
+	GSList *tmpslist;
+	for (tmpslist = list;tmpslist;tmpslist = g_slist_next(tmpslist)) {
+		Tautocomplete *ac = tmpslist->data;
+		if (ac->mode == ac_mode_append)
+			g_free(ac->string);
+		g_slice_free(Tautocomplete, ac);
+	}
+	g_slist_free(list);
+}
+
+static void
+add_autocomplete(Tbflangparsing * bfparser, guint16 matchnum, GSList *list)
+{
+	GSList *tmpslist;
+	for (tmpslist = list;tmpslist;tmpslist = g_slist_next(tmpslist)) {
+		Tautocomplete *ac = tmpslist->data;
+		if (ac->mode != ac_mode_none) {
+			match_add_autocomp_item(bfparser->st, matchnum, 
+										(ac->mode == ac_mode_string) ? ac->string : NULL,
+										(ac->mode == ac_mode_append) ? ac->string : NULL,
+										ac->backup_cursor);
+		}
+	}
+}
+
+static void 
+process_autocomplete(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GSList **list) 
+{
+	gboolean enable=TRUE;
+	gchar *string=NULL, *append=NULL, *class=NULL, *notclass=NULL;
+	gint backup_cursor=0;
+	gboolean enabled;
+	Tautocomplete *ac;
+	Tattrib attribs[] = {
+			{"enable", &enable, attribtype_boolean},
+			{"string", &string, attribtype_string},
+			{"append", &append, attribtype_string},
+			{"class", &class, attribtype_string},
+			{"notclass", &notclass, attribtype_string},
+			{"backup_cursor", &backup_cursor, attribtype_int}
+	};
+	if (!bfparser->load_completion)
+		return;
+	
+	parse_attributes(reader, attribs, 6);
+	enabled = do_parse(bfparser, class, notclass);
+	g_free(class);
+	g_free(notclass);
+	if (!enabled || (string && string[0]=='\0') || (append && append[0] == '\0')) {
+		g_free(string);
+		g_free(append);
+		return;
+	}
+	ac = g_slice_new0(Tautocomplete);
+	if (!enable) {
+		ac->mode = ac_mode_none;
+		g_free(append);
+		g_free(string);
+	} else if (string) {
+		ac->mode = ac_mode_string;
+		ac->string = string;
+		g_free(append);
+	} else if (append) {
+		ac->mode = ac_mode_append;
+		ac->string = append;
+		g_free(string);
+	} else {
+		ac->mode = ac_mode_string;
+		ac->string = NULL; /* use the pattern */
+		g_free(string);
+		g_free(append);
+	}
+	ac->backup_cursor = backup_cursor;
+	/*g_print("prepend item %s to autocomplete\n",ac->string);*/
+	*list = g_slist_prepend(*list, ac);
+}
+
 /* declaration needed for recursive calling */
 static gint16 process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing * bfparser,
 									   GQueue * contextstack);
 
 static guint16
 process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gint16 context,
-						 GQueue * contextstack, gchar * ih_autocomplete_append, const gchar * ih_highlight,
-						 gint ih_case_insens, gint ih_is_regex, gint ih_autocomplete,
-						 gint ih_autocomplete_backup_cursor)
+						 GQueue * contextstack, const gchar * ih_highlight,
+						 gint ih_case_insens, gint ih_is_regex, GSList *ih_autocomplete)
 {
 	guint16 matchnum = 0;
 	gboolean add_element;
 	gchar *pattern = NULL, *idref = NULL, *highlight = NULL, *blockstartelement = NULL, *blockhighlight =
-		NULL, *class = NULL, *notclass = NULL, *autocomplete_append = NULL, *autocomplete_string = NULL, *id =
-		NULL;
+		NULL, *block_name = NULL, *class = NULL, *notclass = NULL, *id =	NULL;
 	gboolean starts_block = FALSE, ends_block = FALSE, is_empty, tagclose_from_blockstack = FALSE, stretch_blockstart=FALSE;
-	gint case_insens = UNDEFINED, is_regex = UNDEFINED, autocomplete = UNDEFINED;
-	gint ends_context = 0, autocomplete_backup_cursor = 0;
+	gint case_insens = UNDEFINED, is_regex = UNDEFINED;
+	gint ends_context = 0;
 	gint identifier_mode=0, identifier_jump=0, identifier_autocomp=0;
+	GSList *autocomplete=NULL;
 	is_empty = xmlTextReaderIsEmptyElement(reader);
 
 	Tattrib attribs[] = {{"pattern", &pattern, attribtype_string},
@@ -603,20 +730,17 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 					{"notclass", &notclass, attribtype_string},
 					{"is_regex", &is_regex, attribtype_boolean},
 					{"starts_block", &starts_block, attribtype_boolean},
+					{"block_name", &block_name, attribtype_string},
 					{"ends_block", &ends_block, attribtype_boolean},
 					{"case_insens", &case_insens, attribtype_boolean},
 					{"ends_context", &ends_context, attribtype_int},
-					{"tagclose_from_blockstack", &tagclose_from_blockstack, attribtype_int},
+					{"tagclose_from_blockstack", &tagclose_from_blockstack, attribtype_boolean},
 					{"stretch_blockstart", &stretch_blockstart, attribtype_int},
 					{"identifier_mode", &identifier_mode, attribtype_int},
 					{"identifier_jump", &identifier_jump, attribtype_int},
-					{"identifier_autocomp", &identifier_autocomp, attribtype_int},
-					{"autocomplete", &autocomplete, attribtype_boolean},
-					{"autocomplete_string", &autocomplete_string, attribtype_string},
-					{"autocomplete_append", &autocomplete_append, attribtype_string},
-					{"autocomplete_backup_cursor", &autocomplete_backup_cursor, attribtype_int}
+					{"identifier_autocomp", &identifier_autocomp, attribtype_int}
 					};
-	parse_attributes(reader, attribs, bfparser->load_completion ? 21 : 18);
+	parse_attributes(reader, attribs, 19);
 	if (stretch_blockstart && ends_block) {
 		g_print("Error in language file, id %s / pattern %s has mutually exclusive options stretch_blockstart and ends_block both enabled\n", id?id:"-", pattern?pattern:"null");
 		stretch_blockstart = FALSE;
@@ -645,6 +769,7 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 			}
 		} else if (pattern && pattern[0]) {
 			gchar *reference = NULL;
+			gboolean foldable=TRUE;
 			guint16 blockstartelementum = 0, nextcontext = 0;
 			DBG_PARSING("pattern %s\n", pattern);
 			if (ends_context) {
@@ -671,11 +796,16 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 								is_regex != UNDEFINED ? is_regex : (ih_is_regex !=UNDEFINED ? ih_is_regex : FALSE),
 								case_insens != UNDEFINED ? case_insens : (ih_case_insens != UNDEFINED ? ih_case_insens :FALSE),
 								context);
+			if (block_name) {
+				foldable = lookup_block_foldable(bfparser->bflang->name, block_name);
+			}
 			pattern_set_blockmatch(bfparser->st, matchnum,
 								starts_block, 
 								ends_block, 
 								blockstartelementum, 
-								blockhighlight);
+								blockhighlight,
+								block_name,
+								foldable);
 			pattern_set_runtime_properties(bfparser->st, matchnum,
 								 highlight ? highlight : ih_highlight,
 								 nextcontext,
@@ -691,7 +821,12 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 			/* now check if there is a deeper context */
 			if (!is_empty) {
 				while (xmlTextReaderRead(reader) == 1) {
-					xmlChar *name = xmlTextReaderName(reader);
+					xmlChar *name;
+					int nodetype = xmlTextReaderNodeType(reader);
+					if (nodetype == 14 /* #text */) {
+						continue;
+					}
+					name = xmlTextReaderName(reader);
 					if (xmlStrEqual(name, (xmlChar *) "context")) {
 						DBG_PARSING("in pattern, found countext\n");
 						nextcontext = process_scanning_context(reader, bfparser, contextstack);
@@ -705,42 +840,18 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 							skip_to_end_tag(reader, xmlTextReaderDepth(reader));
 						}
 					} else if (xmlStrEqual(name, (xmlChar *) "element")) {
-						DBG_PARSING("in pattern, found 'element' --> must be end-of-element\n");
+						DBG_PARSING("in element, found 'element' --> must be end-of-element\n");
 						xmlFree(name);
 						break;
-					} else if (xmlStrEqual(name, (xmlChar *) "autocomplete_append")) {
-						gchar *aclass, *anotclass;
-						aclass = (gchar *)xmlTextReaderGetAttribute(reader,(xmlChar *)"class");
-						anotclass = (gchar *)xmlTextReaderGetAttribute(reader,(xmlChar *)"notclass");
-						if (do_parse(bfparser, aclass, anotclass)) {
-							xmlChar *tmp;
-							tmp = xmlTextReaderGetAttribute(reader,(xmlChar *)"autocomplete_backup_cursor");
-							if (tmp)
-								autocomplete_backup_cursor = g_ascii_strtoll((gchar *)tmp, NULL, 10);
-							xmlFree(tmp);
-							if (autocomplete_append)
-								g_free(autocomplete_append);
-							autocomplete_append = (gchar *)xmlTextReaderReadInnerXml(reader);
-							DBG_PARSING("set autocomplete_append to %s with backup_cursor=%d\n", autocomplete_append, autocomplete_backup_cursor);
-						}
-						xmlFree(aclass);
-						xmlFree(anotclass);
-						DBG_PARSING("handled autocomplete_append, skip to end of this tag\n");
-						skip_to_end_tag(reader, xmlTextReaderDepth(reader));
+					} else if (nodetype == 1 && xmlStrEqual(name, (xmlChar *) "autocomplete")) {
+						process_autocomplete(reader, bfparser, &autocomplete);
 					} else {
 						DBG_PARSING("process_scanning_element, parsing element with name %s\n", name);
 					}
 					xmlFree(name);
 				}
 			}
-			
-			if (autocomplete == TRUE || (autocomplete == UNDEFINED && ih_autocomplete == TRUE)) {
-				match_add_autocomp_item(bfparser->st, matchnum, autocomplete_string,
-										autocomplete_append ? autocomplete_append : ih_autocomplete_append,
-										autocomplete_backup_cursor ? autocomplete_backup_cursor :
-										ih_autocomplete_backup_cursor);
-			}
-			
+			add_autocomplete(bfparser, matchnum, autocomplete?autocomplete:ih_autocomplete);
 			if (reference != NULL) {
 				bfparser->reference_size += strlen(reference);
 				match_set_reference(bfparser->st, matchnum, reference);
@@ -750,15 +861,14 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 		}
 	}
 	/* TODO cleanup! */
+	free_autocomplete(autocomplete);
+	
 	if (pattern)
 		xmlFree(pattern);
 	if (id)
 		xmlFree(id);
 	if (idref)
 		xmlFree(idref);
-	/*if (autocomplete_string) xmlFree(autocomplete_string); *//* stored in the pattern */
-	if (autocomplete_append)
-		xmlFree(autocomplete_append);
 	/*if (highlight) xmlFree(highlight); */
 	if (blockstartelement)
 		xmlFree(blockstartelement);
@@ -777,20 +887,22 @@ stringcmp(const void *sp1, const void *sp2)
 
 static guint16
 process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16 context,
-					 GQueue * contextstack, gchar * ih_autocomplete_append, gchar * ih_highlight,
+					 GQueue * contextstack, gchar * ih_highlight,
 					 gchar * ih_attrib_autocomplete_append, gchar * ih_attribhighlight,
-					 gint ih_autocomplete_backup_cursor, gint ih_attrib_autocomplete_backup_cursor)
+					 gint ih_attrib_autocomplete_backup_cursor, GSList *ih_autocomplete)
 {
 	gchar *tag = NULL, *idref = NULL, *highlight = NULL, *attributes = NULL, *attribhighlight = NULL, *class =
-		NULL, *notclass = NULL, *autocomplete_append = NULL, *attrib_autocomplete_append = NULL, *id = NULL;
-	gint autocomplete_backup_cursor = 0, attrib_autocomplete_backup_cursor = 0;
+		NULL, *notclass = NULL, *attrib_autocomplete_append = NULL, *id = NULL, *block_name = NULL;
+	gint attrib_autocomplete_backup_cursor = 0;
 	guint16 matchnum = 0, innercontext = context;
 	gboolean is_empty, case_insens = FALSE, sgml_shorttag = FALSE, no_close = FALSE;
 	gboolean add_tag;
+	GSList *autocomplete=NULL;
 	DBG_PARSING("processing tag...\n");
 	is_empty = xmlTextReaderIsEmptyElement(reader);
 
 	Tattrib attribs[] = {{"name", &tag, attribtype_string},
+					{"block_name", &block_name, attribtype_string},
 					{"id", &id, attribtype_string},
 					{"idref", &idref, attribtype_string},
 					{"highlight", &highlight, attribtype_string},
@@ -801,11 +913,9 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 					{"attribhighlight", &attribhighlight, attribtype_string},
 					{"sgml_shorttag", &sgml_shorttag, attribtype_boolean},
 					{"no_close", &no_close, attribtype_boolean},
-					{"autocomplete_append", &autocomplete_append, attribtype_string},
 					{"attrib_autocomplete_append", &attrib_autocomplete_append, attribtype_string},
-					{"autocomplete_backup_cursor", &autocomplete_backup_cursor, attribtype_int},
 					{"attrib_autocomplete_backup_cursor", &attrib_autocomplete_backup_cursor, attribtype_int}};
-	parse_attributes(reader, attribs, bfparser->load_completion ? 15 : 11);
+	parse_attributes(reader, attribs, bfparser->load_completion ? 14 : 12);
 	add_tag = do_parse(bfparser, class, notclass);
 	if (add_tag) {
 		if (idref && idref[0] && !tag) {
@@ -823,6 +933,7 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 			guint16 starttagmatch = 0, endtagmatch, matchstring;
 			gint contexttag = 0 /*, contextstring */ ;
 			gchar *tmp, *reference = NULL;
+			gboolean foldable=TRUE;
 
 			/* try to re-use the context of other tags. this is only possible if the other tags have exactly the same attribute set
 			   we check this by sorting the attributes, concatenating them comma separated together, and using this as a hash key
@@ -847,13 +958,16 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 				   'is_empty' is not a good measure for the next context,
 				   because a tag can also have a reference, but it is a start  */
 				contexttag = GPOINTER_TO_INT(g_hash_table_lookup(bfparser->contexts, attrib_context_id));
-				/*if (contexttag) {
-				   g_print("HIT for %s\n",attrib_context_id);
-				   } */
+				if (contexttag) {
+				   g_print("HIT for %s, saves %ld bytes\n",attrib_context_id, (long int)((GArray *)g_array_index(bfparser->st->contexts, Tcontext, contexttag).table)->len*sizeof(Ttablerow));
+				}
 			}
 			tmp = g_strconcat("<", tag, NULL);
 			matchnum = add_pattern_to_scanning_table(bfparser->st, tmp, FALSE, case_insens, context);
-			pattern_set_blockmatch(bfparser->st, matchnum,TRUE,FALSE,0,NULL);
+			if (block_name) {
+				foldable = lookup_block_foldable(bfparser->bflang->name, block_name);
+			}
+			pattern_set_blockmatch(bfparser->st, matchnum,TRUE,FALSE,0,NULL,block_name,foldable);
 			pattern_set_runtime_properties(bfparser->st, matchnum, 
 								highlight ? highlight : ih_highlight,
 								contexttag, TRUE, FALSE,0, FALSE, FALSE);
@@ -921,7 +1035,7 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 								highlight ? highlight : ih_highlight,
 								-1, FALSE, FALSE,0, FALSE, FALSE);
 					pattern_set_blockmatch(bfparser->st, tmpnum,
-								FALSE,TRUE, -1,NULL);
+								FALSE,TRUE, -1,NULL,NULL,TRUE);
 					if (bfparser->autoclose_tags)
 						match_add_autocomp_item(bfparser->st, tmpnum, NULL, NULL, 0);
 					match_autocomplete_reference(bfparser->st, tmpnum, contexttag);
@@ -931,7 +1045,7 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 				pattern_set_runtime_properties(bfparser->st, starttagmatch, 
 								highlight ? highlight : ih_highlight,
 								-1, FALSE, bfparser->stretch_tag_block,0, FALSE, FALSE);
-				pattern_set_blockmatch(bfparser->st, starttagmatch, FALSE, FALSE, matchnum /* blockstartpattern for stretch_block */, NULL);
+				pattern_set_blockmatch(bfparser->st, starttagmatch, FALSE, FALSE, matchnum /* blockstartpattern for stretch_block */, NULL, NULL, TRUE);
 				
 				if (bfparser->autoclose_tags && !no_close)
 					match_add_autocomp_item(bfparser->st, starttagmatch, NULL, tmp, tmp ? strlen(tmp) : 0);
@@ -949,7 +1063,14 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 
 			if (!is_empty) {
 				while (xmlTextReaderRead(reader) == 1) {
-					xmlChar *name = xmlTextReaderName(reader);
+					int nodetype = xmlTextReaderNodeType(reader);
+					xmlChar *name;
+					
+					if (nodetype == 14) {
+						continue;
+					}
+					
+					name = xmlTextReaderName(reader);
 					if (xmlStrEqual(name, (xmlChar *) "reference")) {
 						if (!xmlTextReaderIsEmptyElement(reader)) {
 							if (langmgr.load_reference && bfparser->load_reference)
@@ -969,25 +1090,8 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 					} else if (xmlStrEqual(name, (xmlChar *) "tag")) {
 						xmlFree(name);
 						break;
-					} else if (xmlStrEqual(name, (xmlChar *) "autocomplete_append")) {
-						gchar *aclass, *anotclass;
-						aclass = (gchar *)xmlTextReaderGetAttribute(reader,(xmlChar *)"class");
-						anotclass = (gchar *)xmlTextReaderGetAttribute(reader,(xmlChar *)"notclass");
-						if (do_parse(bfparser, aclass, anotclass)) {
-							xmlChar *tmp;
-							tmp = xmlTextReaderGetAttribute(reader,(xmlChar *)"autocomplete_backup_cursor");
-							if (tmp)
-								autocomplete_backup_cursor = g_ascii_strtoll((gchar *)tmp, NULL, 10);
-							xmlFree(tmp);
-							if (autocomplete_append)
-								g_free(autocomplete_append);
-							autocomplete_append = (gchar *)xmlTextReaderReadInnerXml(reader);
-							DBG_PARSING("set autocomplete_append to %s with backup_cursor=%d\n", autocomplete_append, autocomplete_backup_cursor);
-						}
-						xmlFree(aclass);
-						xmlFree(anotclass);
-						DBG_PARSING("handled autocomplete_append, skip to end of this tag\n");
-						skip_to_end_tag(reader, xmlTextReaderDepth(reader));
+					} else if (nodetype == 1 && xmlStrEqual(name, (xmlChar *) "autocomplete")) {
+						process_autocomplete(reader, bfparser, &autocomplete);
 					} else {
 						DBG_PARSING("process_scanning_tag, parsing element with name %s\n", name);
 					}
@@ -995,15 +1099,16 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 				}
 			}
 
-			if (!no_close && autocomplete_append == NULL && ih_autocomplete_append == NULL) {
+			if (!no_close && autocomplete == NULL && ih_autocomplete == NULL) {
 				gchar *tmp2 = g_strconcat("></", tag, ">", NULL);
 				match_add_autocomp_item(bfparser->st, matchnum, NULL, tmp2, strlen(tag) + 3);
 				g_free(tmp2);
 			} else {
-				match_add_autocomp_item(bfparser->st, matchnum, NULL,
+				add_autocomplete(bfparser, matchnum, autocomplete?autocomplete:ih_autocomplete);
+				/*match_add_autocomp_item(bfparser->st, matchnum, NULL,
 										autocomplete_append ? autocomplete_append : ih_autocomplete_append,
 										autocomplete_backup_cursor ? autocomplete_backup_cursor :
-										ih_autocomplete_backup_cursor);
+										ih_autocomplete_backup_cursor);*/
 			}
 
 			if (reference != NULL) {
@@ -1020,7 +1125,7 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 									(innercontext == context) ? 0 : -2,
 									FALSE, FALSE,0, FALSE, FALSE);
 				pattern_set_blockmatch(bfparser->st, endtagmatch,
-									FALSE,TRUE,matchnum,NULL);
+									FALSE,TRUE,matchnum,NULL,NULL, TRUE);
 /*				g_print("context %d: matchnum %d is ended by endtagmatch %d while working on id %s\n",innercontext,matchnum, endtagmatch, id);*/
 				match_add_autocomp_item(bfparser->st, endtagmatch, NULL, NULL, 0);
 				match_autocomplete_reference(bfparser->st, endtagmatch, innercontext);
@@ -1034,13 +1139,13 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 			}
 		}
 	}
+	free_autocomplete(autocomplete);
 	g_free(id);
 	g_free(idref);
 	g_free(tag);
 	/*g_free(highlight); */
 	g_free(class);
 	g_free(notclass);
-	g_free(autocomplete_append);
 	g_free(attrib_autocomplete_append);
 	/*g_free(attribhighlight); */
 	g_free(attributes);
@@ -1051,16 +1156,16 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 /* ih stands for InHerit */
 static void
 process_scanning_group(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gint context,
-					   GQueue * contextstack, gchar * ih_autocomplete_append, gchar * ih_highlight,
+					   GQueue * contextstack, gchar * ih_highlight,
 					   gchar * ih_class, gchar * ih_attrib_autocomplete_append, gchar * ih_attribhighlight,
-					   gint ih_case_insens, gint ih_is_regex, gint ih_autocomplete,
-					   gint ih_autocomplete_backup_cursor, gint ih_attrib_autocomplete_backup_cursor)
+					   gint ih_case_insens, gint ih_is_regex, gint ih_attrib_autocomplete_backup_cursor,
+					   GSList *ih_autocomplete)
 {
-	gchar *autocomplete_append = NULL, *highlight = NULL, *class = NULL, *notclass =
+	gchar *highlight = NULL, *class = NULL, *notclass =
 		NULL, *attrib_autocomplete_append = NULL, *attribhighlight = NULL;
 	gint case_insens = UNDEFINED, is_regex = UNDEFINED;
-	gint autocomplete = UNDEFINED;
-	gint autocomplete_backup_cursor = 0, attrib_autocomplete_backup_cursor = 0;
+	gint attrib_autocomplete_backup_cursor = 0;
+	GSList *autocomplete=NULL;
 	gint depth;
 	gboolean add_group;
 	if (xmlTextReaderIsEmptyElement(reader)) {
@@ -1073,12 +1178,9 @@ process_scanning_group(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gint
 					{"is_regex", &is_regex, attribtype_boolean},
 					{"case_insens", &case_insens, attribtype_boolean},
 					{"attribhighlight", &attribhighlight, attribtype_string},
-					{"autocomplete", &autocomplete, attribtype_boolean},
-					{"autocomplete_append", &autocomplete_append, attribtype_string},
 					{"attrib_autocomplete_append", &attrib_autocomplete_append, attribtype_string},
-					{"autocomplete_backup_cursor", &autocomplete_backup_cursor, attribtype_int},
 					{"attrib_autocomplete_backup_cursor", &attrib_autocomplete_backup_cursor, attribtype_int}};
-	parse_attributes(reader, attribs, bfparser->load_completion ? 11 : 6);
+	parse_attributes(reader, attribs, bfparser->load_completion ? 8 : 6);
 	add_group = do_parse(bfparser, class, notclass);
 	DBG_PARSING("add_group=%d for class=%s and notclass=%s\n",add_group, class, notclass);
 	if (!add_group) {
@@ -1087,27 +1189,29 @@ process_scanning_group(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gint
 		skip_to_end_tag(reader, depth);
 	} else {
 		while (xmlTextReaderRead(reader) == 1) {
-			xmlChar *name = xmlTextReaderName(reader);
+			int nodetype = xmlTextReaderNodeType(reader);
+			xmlChar *name;
+			
+			if (nodetype == 14 /* #text */) {
+				continue;
+			} 
+			
+			name = xmlTextReaderName(reader);
 			if (xmlStrEqual(name, (xmlChar *) "element")) {
 				process_scanning_element(reader, bfparser, context, contextstack,
-										 autocomplete_append ? autocomplete_append : ih_autocomplete_append,
-										 highlight ? highlight : ih_highlight,
-										 case_insens != UNDEFINED ? case_insens : ih_case_insens,
-										 is_regex != UNDEFINED ? is_regex : ih_is_regex,
-										 autocomplete != UNDEFINED ? autocomplete : ih_autocomplete,
-										 autocomplete_backup_cursor ? autocomplete_backup_cursor :
-										 ih_autocomplete_backup_cursor);
+									highlight ? highlight : ih_highlight,
+									case_insens != UNDEFINED ? case_insens : ih_case_insens,
+									is_regex != UNDEFINED ? is_regex : ih_is_regex,
+									autocomplete ? autocomplete : ih_autocomplete);
 			} else if (xmlStrEqual(name, (xmlChar *) "tag")) {
 				process_scanning_tag(reader, bfparser, context, contextstack,
-									 autocomplete_append ? autocomplete_append : ih_autocomplete_append,
-									 highlight ? highlight : ih_highlight,
-									 attrib_autocomplete_append ? attrib_autocomplete_append :
-									 ih_attrib_autocomplete_append,
-									 attribhighlight ? attribhighlight : ih_attribhighlight,
-									 autocomplete_backup_cursor ? autocomplete_backup_cursor :
-									 ih_autocomplete_backup_cursor,
-									 attrib_autocomplete_backup_cursor ? attrib_autocomplete_backup_cursor :
-									 ih_attrib_autocomplete_backup_cursor);
+									highlight ? highlight : ih_highlight,
+									attrib_autocomplete_append ? attrib_autocomplete_append :
+									ih_attrib_autocomplete_append,
+									attribhighlight ? attribhighlight : ih_attribhighlight,
+									attrib_autocomplete_backup_cursor ? attrib_autocomplete_backup_cursor :
+									ih_attrib_autocomplete_backup_cursor,
+									autocomplete ? autocomplete : ih_autocomplete);
 			} else if (xmlStrEqual(name, (xmlChar *) "group")) {
 				if (xmlTextReaderDepth(reader) == depth) {
 					DBG_PARSING("end-of-group\n");
@@ -1115,27 +1219,26 @@ process_scanning_group(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gint
 					break;
 				} else {
 					process_scanning_group(reader, bfparser, context, contextstack,
-										   autocomplete_append ? autocomplete_append : ih_autocomplete_append,
-										   highlight ? highlight : ih_highlight, class ? class : ih_class,
-										   attrib_autocomplete_append ? attrib_autocomplete_append :
-										   ih_attrib_autocomplete_append,
-										   attribhighlight ? attribhighlight : ih_attribhighlight,
-										   case_insens != UNDEFINED ? case_insens : ih_case_insens,
-										   is_regex != UNDEFINED ? is_regex : ih_is_regex,
-										   autocomplete != UNDEFINED ? autocomplete : ih_autocomplete,
-										   autocomplete_backup_cursor ? autocomplete_backup_cursor :
-										   ih_autocomplete_backup_cursor,
-										   attrib_autocomplete_backup_cursor ?
-										   attrib_autocomplete_backup_cursor :
-										   ih_attrib_autocomplete_backup_cursor);
+									highlight ? highlight : ih_highlight, class ? class : ih_class,
+									attrib_autocomplete_append ? attrib_autocomplete_append :
+									ih_attrib_autocomplete_append,
+									attribhighlight ? attribhighlight : ih_attribhighlight,
+									case_insens != UNDEFINED ? case_insens : ih_case_insens,
+									is_regex != UNDEFINED ? is_regex : ih_is_regex,
+									attrib_autocomplete_backup_cursor ?
+									attrib_autocomplete_backup_cursor :
+									ih_attrib_autocomplete_backup_cursor,
+									autocomplete ? autocomplete : ih_autocomplete);
 				}
+			} else if (nodetype == 1 && xmlStrEqual(name, (xmlChar *) "autocomplete")) {
+				process_autocomplete(reader, bfparser, &autocomplete);
 			} else {
 				DBG_PARSING("found %s\n", name);
 			}
 			xmlFree(name);
 		}
 	}
-	g_free(autocomplete_append);
+	free_autocomplete(autocomplete);
 	/*g_free(highlight); stored in the structure */
 	g_free(class);
 	g_free(notclass);
@@ -1232,13 +1335,13 @@ process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GQu
 		xmlChar *name = xmlTextReaderName(reader);
 		DBG_PARSING("parsing context, found node id %s\n", name);
 		if (xmlStrEqual(name, (xmlChar *) "element")) {
-			process_scanning_element(reader, bfparser, context, contextstack, NULL, NULL, UNDEFINED,
-									 UNDEFINED, UNDEFINED, 0);
+			process_scanning_element(reader, bfparser, context, contextstack, NULL, UNDEFINED,
+									 UNDEFINED, NULL);
 		} else if (xmlStrEqual(name, (xmlChar *) "tag")) {
-			process_scanning_tag(reader, bfparser, context, contextstack, NULL, NULL, NULL, NULL, 0, 0);
+			process_scanning_tag(reader, bfparser, context, contextstack, NULL, NULL, NULL, 0, NULL);
 		} else if (xmlStrEqual(name, (xmlChar *) "group")) {
-			process_scanning_group(reader, bfparser, context, contextstack, NULL, NULL, NULL, NULL, NULL,
-								   UNDEFINED, UNDEFINED, UNDEFINED, 0, 0);
+			process_scanning_group(reader, bfparser, context, contextstack, NULL, NULL, NULL, NULL,
+								   UNDEFINED, UNDEFINED, 0, NULL);
 		} else if (xmlStrEqual(name, (xmlChar *) "context")) {
 			xmlFree(name);
 			DBG_PARSING("parsing context, end-of-context, return context %d\n", context);
@@ -1313,10 +1416,10 @@ build_lang_thread(gpointer data)
 			if (xmlTextReaderIsEmptyElement(reader)) {
 				DBG_PARSING("empty <definition />\n");
 				/* empty <definition />, probably text/plain */
-				
-				/* TODO: MEMLEAK: BUG: cleanup the DFA table for each context !!!!!!!!!!!!! */
 				g_array_free(bfparser->st->matches, TRUE);
 				g_array_free(bfparser->st->contexts, TRUE);
+				g_array_free(bfparser->st->comments, TRUE);
+				g_array_free(bfparser->st->blocks, TRUE);
 				g_slice_free(Tscantable, bfparser->st);
 				bfparser->st = NULL;
 				xmlFree(name);
@@ -1392,7 +1495,10 @@ build_lang_thread(gpointer data)
 			g_realloc(bfparser->st->contexts->data, (bfparser->st->contexts->len + 1) * sizeof(Tcontext));
 		bfparser->st->matches->data =
 			g_realloc(bfparser->st->matches->data, (bfparser->st->matches->len + 1) * sizeof(Tpattern));
-
+		bfparser->st->comments->data =
+			g_realloc(bfparser->st->comments->data, (bfparser->st->comments->len + 1) * sizeof(Tcomment));
+		bfparser->st->blocks->data =
+			g_realloc(bfparser->st->blocks->data, (bfparser->st->blocks->len + 1) * sizeof(Tpattern_block));
 		/* now optimise the DFA tables for each context */
 		for (i = 1; i < bfparser->st->contexts->len; i++) {
 			g_array_index(bfparser->st->contexts, Tcontext, i).table->data = g_realloc(g_array_index(bfparser->st->contexts, Tcontext, i).table->data, (g_array_index(bfparser->st->contexts, Tcontext, i).table->len + 1) * sizeof(Ttablerow));
@@ -1410,6 +1516,8 @@ build_lang_thread(gpointer data)
 				1.0 * bfparser->st->contexts->len * sizeof(Tcontext) / 1024.0);
 		g_print("matches       %5d (%9.2f Kbytes)\n", bfparser->st->matches->len,
 				1.0 * bfparser->st->matches->len * sizeof(Tpattern) / 1024.0);
+		g_print("blocks        %5d (%9.2f Kbytes)\n", bfparser->st->blocks->len,
+				1.0 * bfparser->st->blocks->len * sizeof(Tpattern_block) / 1024.0);
 		/*print_DFA(bfparser->st, '&','Z'); */
 		/*print_DFA_subset(bfparser->st, "<PpIi>"); */
 
@@ -1468,37 +1576,51 @@ static Tbflang *
 parse_bflang2_header(const gchar * filename)
 {
 	xmlTextReaderPtr reader;
-	Tbflang *bflang = NULL;
+	Tbflang *bflang;
+	gboolean failed=FALSE;
 	gchar *bflangversion = NULL;
+
 	reader = xmlNewTextReaderFilename(filename);
 	xmlTextReaderSetParserProp(reader, XML_PARSER_SUBST_ENTITIES, TRUE);
-	if (reader != NULL) {
-		bflang = g_slice_new0(Tbflang);
-		bflang->size_table = 2;	/* bare minimum sizes */
-		bflang->size_matches = 2;
-		bflang->size_contexts = 2;
-		bflang->filename = g_strdup(filename);
-		while (xmlTextReaderRead(reader) == 1) {
-			xmlChar *name = xmlTextReaderName(reader);
-			if (xmlStrEqual(name, (xmlChar *) "bflang")) {
-				Tattrib attribs[] = {{"name", &bflang->name, attribtype_string},
-						{"version", &bflangversion, attribtype_string},
-						{"table", &bflang->size_table, attribtype_int},
-						{"matches", &bflang->size_matches, attribtype_int},
-						{"contexts", &bflang->size_contexts, attribtype_int}};
-				parse_attributes(reader, attribs, 5);
-			} else if (xmlStrEqual(name, (xmlChar *) "header")) {
-				process_header(reader, bflang);
-				xmlFree(name);
-				break;
-			}
+	if (!reader) {
+		return NULL;
+	}
+	bflang = g_slice_new0(Tbflang);
+	bflang->size_table = 2;	/* bare minimum sizes */
+	bflang->size_matches = 2;
+	bflang->size_contexts = 2;
+	bflang->filename = g_strdup(filename);
+	while (xmlTextReaderRead(reader) == 1) {
+		xmlChar *name = xmlTextReaderName(reader);
+		if (xmlStrEqual(name, (xmlChar *) "bflang")) {
+			Tattrib attribs[] = {{"name", &bflang->name, attribtype_string},
+					{"version", &bflangversion, attribtype_string},
+					{"table", &bflang->size_table, attribtype_int},
+					{"matches", &bflang->size_matches, attribtype_int},
+					{"contexts", &bflang->size_contexts, attribtype_int}};
+			parse_attributes(reader, attribs, 5);
+		} else if (xmlStrEqual(name, (xmlChar *) "header")) {
+			process_header(reader, bflang);
 			xmlFree(name);
+			break;
 		}
-		xmlFreeTextReader(reader);
-		if (bflang->name == NULL) {
-			g_print("Language file %s has no name.. abort..\n", filename);
-			return NULL;
-		}
+		xmlFree(name);
+	}
+	xmlFreeTextReader(reader);
+	
+	if (bflang->name == NULL) {
+		g_print("Language file %s has no name.. abort..\n", filename);
+		failed=TRUE;
+	}
+	if (g_strcmp0(bflangversion, CURRENT_BFLANG2_VERSION)!=0) {
+		g_print("Language file %s has incompatible version %s (need version "CURRENT_BFLANG2_VERSION"), abort..\n", filename, bflangversion);
+		failed=TRUE;
+	} 
+	g_free(bflangversion);
+	if (failed) {
+		g_free(bflang->name);
+		g_slice_free(Tbflang, bflang);
+		return NULL;
 	}
 	return bflang;
 }
@@ -1567,7 +1689,7 @@ bflang_cleanup_scantable(Tbflang * bflang)
 		   use the same string in memory for this value... for example if they are part of
 		   the same <group>
 		   g_free(g_array_index(bflang->st->matches, Tpattern, i).selfhighlight); */
-		g_free(g_array_index(bflang->st->matches, Tpattern, i).blockhighlight);
+		/*g_free(g_array_index(bflang->st->matches, Tpattern, i).blockhighlight);*/
 		for (slist = g_array_index(bflang->st->matches, Tpattern, i).autocomp_items; slist;
 			 slist = g_slist_next(slist)) {
 			g_slice_free(Tpattern_autocomplete, slist->data);
@@ -1586,9 +1708,14 @@ bflang_cleanup_scantable(Tbflang * bflang)
 		g_free(g_array_index(bflang->st->comments, Tcomment, i).so);
 		g_free(g_array_index(bflang->st->comments, Tcomment, i).eo);
 	}
+	for (i = 1; i < bflang->st->blocks->len; i++) {
+		g_free(g_array_index(bflang->st->blocks, Tpattern_block, i).name);
+		g_free(g_array_index(bflang->st->blocks, Tpattern_block, i).highlight);
+	}
 	g_array_free(bflang->st->matches, TRUE);
 	g_array_free(bflang->st->contexts, TRUE);
 	g_array_free(bflang->st->comments, TRUE);
+	g_array_free(bflang->st->blocks, TRUE);
 	g_slice_free(Tscantable, bflang->st);
 	bflang->st = NULL;
 }
