@@ -20,6 +20,7 @@
 /* for the design docs see bftextview2.h */
 #include <string.h>
 #include "bluefish.h"
+#include "bftextview2_private.h"
 #include "bftextview2_patcompile.h"
 #include "bftextview2_langmgr.h"
 #include "bf_lib.h"
@@ -87,11 +88,18 @@ bftextview2_scantable_rematch_highlights(Tscantable * st, const gchar * lang)
 				g_print("Possible error in language file, no textstyle found for highlight %s\n",
 						g_array_index(st->matches, Tpattern, i).selfhighlight);
 		}
-		if (g_array_index(st->matches, Tpattern, i).blockhighlight) {
+		/*if (g_array_index(st->matches, Tpattern, i).blockhighlight) {
 			g_array_index(st->matches, Tpattern, i).blocktag =
 				langmrg_lookup_tag_highlight(lang, g_array_index(st->matches, Tpattern, i).blockhighlight);
 			if (g_array_index(st->matches, Tpattern, i).blocktag)
 				retlist = g_list_prepend(retlist, g_array_index(st->matches, Tpattern, i).blocktag);
+		}*/
+	}
+	for (i = 0; i < (st->blocks->len); i++) {
+		if (g_array_index(st->blocks, Tpattern_block, i).highlight) {
+			g_array_index(st->blocks, Tpattern_block, i).tag = langmrg_lookup_tag_highlight(lang, g_array_index(st->blocks, Tpattern_block, i).highlight);
+			if (g_array_index(st->blocks, Tpattern_block, i).tag)
+				retlist = g_list_prepend(retlist, g_array_index(st->blocks, Tpattern_block, i).tag);
 		}
 	}
 
@@ -738,7 +746,7 @@ match_autocomplete_reference(Tscantable * st, guint16 matchnum, guint16 context)
 
 		while (tmpslist) {
 			Tpattern_autocomplete *pac = tmpslist->data;
-			list = g_list_append(list, pac->autocomplete_string);
+			list = g_list_prepend(list, pac->autocomplete_string);
 			g_hash_table_insert(g_array_index(st->contexts, Tcontext, context).patternhash,
 								pac->autocomplete_string, GINT_TO_POINTER(pattern_id));
 			tmpslist = g_slist_next(tmpslist);
@@ -778,6 +786,7 @@ match_autocomplete_reference(Tscantable * st, guint16 matchnum, guint16 context)
 #endif
 }
 
+
 void
 match_add_autocomp_item(Tscantable * st, guint16 matchnum, const gchar * autocomplete_string,
 						const gchar * autocomplete_append, guint8 autocomplete_backup_cursor)
@@ -785,7 +794,7 @@ match_add_autocomp_item(Tscantable * st, guint16 matchnum, const gchar * autocom
 	Tpattern_autocomplete *pac = g_slice_new(Tpattern_autocomplete);
 	if (autocomplete_string) {
 		pac->autocomplete_string = (gchar *) autocomplete_string;
-	} else if (autocomplete_append && autocomplete_append[0]!='\0') {
+	} else if (autocomplete_append) {
 		pac->autocomplete_string =
 			g_strconcat(g_array_index(st->matches, Tpattern, matchnum).pattern, autocomplete_append, NULL);
 	} else {
@@ -824,12 +833,34 @@ pattern_set_blockmatch(Tscantable * st, guint16 matchnum,
 							gboolean starts_block,
 							gboolean ends_block, 
 							guint blockstartpattern,
-							const gchar * blockhighlight) 
+							const gchar *blockhighlight,
+							const gchar *blockname,
+							gboolean foldable) 
 {
-	g_array_index(st->matches, Tpattern, matchnum).ends_block = ends_block;
-	g_array_index(st->matches, Tpattern, matchnum).starts_block = starts_block;
+	if (starts_block == TRUE && ends_block == TRUE) {
+		g_warning("Error in language file or Bluefish bug: pattern %s both starts and ends a block\n", 
+						g_array_index(st->matches, Tpattern, matchnum).pattern);
+	}
+	if (blockname && !starts_block) {
+		g_warning("Error in language file or Bluefish bug: block_name %s can only be set on a block start\n", blockname);
+	}
+	
+	if (starts_block) {
+		guint16 blocknum = 0;
+		if (blockname || blockhighlight) { /* only create a block if we need it */
+			blocknum = st->blocks->len;
+			g_array_set_size(st->blocks, st->blocks->len + 1);
+			g_array_index(st->blocks, Tpattern_block, blocknum).highlight = (gchar *) blockhighlight;
+			g_array_index(st->blocks, Tpattern_block, blocknum).name = (gchar *) blockname;
+			g_array_index(st->blocks, Tpattern_block, blocknum).foldable = foldable;
+			g_array_index(st->blocks, Tpattern_block, blocknum).tag = NULL;
+		}
+		g_array_index(st->matches, Tpattern, matchnum).starts_block = 1;
+		g_array_index(st->matches, Tpattern, matchnum).block = blocknum;
+	} else if (ends_block) {
+		g_array_index(st->matches, Tpattern, matchnum).ends_block = 1;
+	}
 	g_array_index(st->matches, Tpattern, matchnum).blockstartpattern = blockstartpattern;
-	g_array_index(st->matches, Tpattern, matchnum).blockhighlight = (gchar *) blockhighlight;
 }
 
 void
@@ -1006,8 +1037,10 @@ scantable_new(guint size_table, guint size_matches, guint size_contexts)
 	st->contexts = g_array_sized_new(TRUE, TRUE, sizeof(Tcontext), size_contexts);
 	st->matches = g_array_sized_new(TRUE, TRUE, sizeof(Tpattern), size_matches);
 	st->comments = g_array_sized_new(TRUE, FALSE, sizeof(Tcomment), 8);
+	st->blocks = g_array_sized_new(TRUE, FALSE, sizeof(Tpattern_block), 8);
 	st->matches->len = 1;		/* match 0 means no match */
 	st->contexts->len = 1;		/* a match with nextcontext 0 means no context change, so we cannot use context 0 */
+	st->blocks->len = 1;			/* block 0 means no block */
 	return st;
 }
 
