@@ -16,7 +16,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-
+/*#define DEVELOPMENT*/
 /*#define HL_PROFILING*/
 /*#define DUMP_SCANCACHE*/
 
@@ -31,6 +31,7 @@
 #endif
 /* for the design docs see bftextview2.h */
 #include "bluefish.h"
+#include "bftextview2_private.h"
 #include "bftextview2_scanner.h"
 #include "bftextview2_identifier.h"
 
@@ -57,6 +58,10 @@ G_SLICE=always-malloc G_DEBUG=gc-friendly,resident-modules valgrind --tool=memch
 #define MAX_CONTINUOUS_SCANNING_INTERVAL 0.1	/* float in seconds */
 #endif
 
+#ifdef DEVELOPMENT
+static void scancache_check_integrity(BluefishTextView * btv, GTimer *timer);
+#endif
+
 typedef struct {
 	GtkTextIter start;
 	GtkTextIter end;
@@ -260,13 +265,13 @@ remove_cache_entry(BluefishTextView * btv, Tfound ** found, GSequenceIter ** sit
 		return 0;
 	
 	*found = get_foundcache_next(btv, siter);
-	DBG_SCANCACHE("remove_cache_entry, remove %p at offset %d and any children, numblockchange=%d, numcontextchange=%d\n", tmpfound1,
+	DBG_SCANCACHE("remove_cache_entry, STARTED, remove %p at offset %d and any children, numblockchange=%d, numcontextchange=%d\n", tmpfound1,
 				  tmpfound1->charoffset_o, tmpfound1->numblockchange, tmpfound1->numcontextchange);
 		/* if this entry pops blocks or contexts, mark the ends of those as undefined */
 	if (tmpfound1->numblockchange < 0) {
 		Tfoundblock *tmpfblock = tmpfound1->fblock;
-		DBG_SCANCACHE("remove_cache_entry, mark end of %d fblock's as undefined, fblock=%p\n",
-					  tmpfound1->numblockchange, tmpfound1->fblock);
+		DBG_SCANCACHE("remove_cache_entry, found %p pops blocks, mark end of %d fblock's as undefined, fblock=%p\n",
+					  tmpfound1, tmpfound1->numblockchange, tmpfound1->fblock);
 		blockstackcount = tmpfound1->numblockchange;
 		while (tmpfblock && blockstackcount < 0) {
 			if (!curblockstack || is_fblock_on_stack(curblockstack, tmpfblock)) {
@@ -280,8 +285,8 @@ remove_cache_entry(BluefishTextView * btv, Tfound ** found, GSequenceIter ** sit
 	}
 	if (tmpfound1->numcontextchange < 0) {
 		Tfoundcontext *tmpfcontext = tmpfound1->fcontext;
-		DBG_SCANCACHE("remove_cache_entry, mark end of %d fcontext's as undefined\n",
-					  tmpfound1->numcontextchange);
+		DBG_SCANCACHE("remove_cache_entry, found %p pops contexts, mark end of %d fcontext's as undefined, fcontext=%p\n",
+					  tmpfound1, tmpfound1->numcontextchange, tmpfound1->fcontext);
 		contextstackcount = tmpfound1->numcontextchange;
 		while (tmpfcontext && contextstackcount < 0) {
 			if (!curcontextstack || is_fcontext_on_stack(curcontextstack, tmpfcontext)) {
@@ -297,7 +302,7 @@ remove_cache_entry(BluefishTextView * btv, Tfound ** found, GSequenceIter ** sit
 	contextstackcount = MAX(0, tmpfound1->numcontextchange);
 	/* remove the children */
 	DBG_SCANCACHE
-		("remove_cache_entry, remove all children of found %p (numblockchange=%d,numcontextchange=%d) with offset %d, next found in cache=%p\n",
+		("remove_cache_entry, remove in loop all children of found %p (numblockchange=%d,numcontextchange=%d) with offset %d, next found in cache=%p\n",
 		 tmpfound1, tmpfound1->numblockchange, tmpfound1->numcontextchange, tmpfound1->charoffset_o, *found);
 	while (*found && (blockstackcount > 0 || contextstackcount > 0)) {
 		Tfound *tmpfound2 = *found;
@@ -307,8 +312,8 @@ remove_cache_entry(BluefishTextView * btv, Tfound ** found, GSequenceIter ** sit
 		contextstackcount += tmpfound2->numcontextchange;
 
 		DBG_SCANCACHE
-			("in loop: remove Tfound %p with offset %d from the cache and free, contextstackcount=%d, blockstackcount=%d\n",
-			 tmpfound2, tmpfound2->charoffset_o, contextstackcount, blockstackcount);
+			("in loop: remove Tfound %p with offset %d, fcontext=%p, numcontextchange=%d, fblock=%p, numblockchange=%d, from the cache and free, contextstackcount=%d, blockstackcount=%d, nextfound=%p\n",
+			 tmpfound2, tmpfound2->charoffset_o, tmpfound2->fcontext, tmpfound2->numcontextchange, tmpfound2->fblock, tmpfound2->numblockchange, contextstackcount, blockstackcount, *found);
 		invalidoffset = tmpfound2->charoffset_o;
 		g_sequence_remove(tmpsiter2);
 		found_free_lcb(tmpfound2, btv);
@@ -545,8 +550,6 @@ static inline Tfoundblock *
 found_start_of_block(BluefishTextView * btv, Tmatch * match, Tscanning * scanning)
 {
 	Tfoundblock *fblock;
-	DBG_BLOCKMATCH("found_start_of_block, put block for pattern %d (%s) on blockstack\n", match->patternum,
-				   g_array_index(btv->bflang->st->matches, Tpattern, match->patternum).pattern);
 #ifdef HL_PROFILING
 	hl_profiling.numblockstart++;
 	hl_profiling.fblock_refcount++;
@@ -558,6 +561,9 @@ found_start_of_block(BluefishTextView * btv, Tmatch * match, Tscanning * scannin
 	fblock->start2_o = BF2_OFFSET_UNDEFINED;
 	fblock->end2_o = BF2_OFFSET_UNDEFINED;
 	fblock->patternum = match->patternum;
+	DBG_BLOCKMATCH("found_start_of_block, %d:%d, put block for pattern %d (%s) on blockstack\n", 
+					fblock->start1_o,fblock->start2_o,match->patternum,
+				   g_array_index(btv->bflang->st->matches, Tpattern, match->patternum).pattern);
 	fblock->parentfblock = scanning->curfblock;
 	DBG_BLOCKMATCH("found_start_of_block, new block at %p with parent %p\n", fblock, fblock->parentfblock);
 	scanning->curfblock = fblock;
@@ -572,7 +578,11 @@ found_end_of_block(BluefishTextView * btv, Tmatch * match, Tscanning * scanning,
 {
 	Tfoundblock *retfblock, *fblock = scanning->curfblock;
 	GtkTextIter iter;
-	DBG_BLOCKMATCH("found_end_of_block(), blockstartpattern %d, curfblock=%p\n", pat->blockstartpattern,
+	gboolean allowfold=TRUE;
+	DBG_BLOCKMATCH("found_end_of_block(), found %d (%s), blockstartpattern %d, curfblock=%p\n", 
+					match->patternum,
+					g_array_index(btv->bflang->st->matches, Tpattern, match->patternum).pattern,
+					pat->blockstartpattern,
 				   scanning->curfblock);
 
 	if (G_UNLIKELY(!scanning->curfblock))
@@ -635,15 +645,18 @@ found_end_of_block(BluefishTextView * btv, Tmatch * match, Tscanning * scanning,
 
 	fblock->start2_o = gtk_text_iter_get_offset(&match->start);
 	fblock->end2_o = gtk_text_iter_get_offset(&match->end);
-	DBG_BLOCKMATCH("set end for block %p to %d:%d\n", fblock, fblock->start2_o, fblock->end2_o);
 	gtk_text_buffer_get_iter_at_offset(btv->buffer, &iter, fblock->end1_o);
-	if (G_UNLIKELY(pat->blocktag)) {
-		gtk_text_buffer_apply_tag(btv->buffer, pat->blocktag, &iter, &match->start);
+	if (G_UNLIKELY(g_array_index(btv->bflang->st->matches, Tpattern, fblock->patternum).block)) {
+		if (g_array_index(btv->bflang->st->blocks, Tpattern_block, g_array_index(btv->bflang->st->matches, Tpattern, fblock->patternum).block).tag
+			 	) {
+			gtk_text_buffer_apply_tag(btv->buffer, g_array_index(btv->bflang->st->blocks, Tpattern_block, g_array_index(btv->bflang->st->matches, Tpattern, fblock->patternum).block).tag, &iter, &match->start);
+		}
+		allowfold = g_array_index(btv->bflang->st->blocks, Tpattern_block, g_array_index(btv->bflang->st->matches, Tpattern, fblock->patternum).block).foldable;
 	}
-	if ((gtk_text_iter_get_line(&iter) + 1) < gtk_text_iter_get_line(&match->start)) {
+	if (allowfold && (gtk_text_iter_get_line(&iter)) < gtk_text_iter_get_line(&match->start)) {
 		fblock->foldable = TRUE;
 	}
-
+	DBG_BLOCKMATCH("found_end_of_block, set end for block %p to %d:%d, foldable=%d\n", fblock, fblock->start2_o, fblock->end2_o, fblock->foldable);
 	scanning->curfblock = fblock->parentfblock;
 	(*numblockchange)--;
 	return retfblock;
@@ -824,13 +837,14 @@ static guint
 remove_invalid_cache(BluefishTextView * btv, guint match_end_o, Tscanning * scanning)
 {
 	guint invalidoffset=0;
-	DBG_SCANNING("remove_invalid_cache, scanning->nextfound=%p, scanning->curfblock=%p, scanning->curfblock=%p\n", scanning->nextfound, scanning->curfblock, scanning->curfblock);
+	DBG_SCANNING("remove_invalid_cache, match_end_o=%d, scanning->nextfound=%p, scanning->curfblock=%p, scanning->curfblock=%p\n", match_end_o, scanning->nextfound, scanning->curfblock, scanning->curfblock);
 	do {
 		gint ret = remove_cache_entry(btv, &scanning->nextfound, &scanning->siter, scanning->curfblock, scanning->curfcontext);
-		/* reove cache entry may return 0 if nothing was removed */
+		/* remove cache entry may return 0 if nothing was removed */
+		DBG_SCANNING("remove_invalid_cache, scanning->nextfound=%p with offset %d\n", scanning->nextfound, scanning->nextfound ? scanning->nextfound->charoffset_o : -1);
 		if (ret > invalidoffset) 
 			invalidoffset = ret; 
-	} while (scanning->nextfound && (scanning->nextfound->charoffset_o < match_end_o || !nextcache_valid(scanning)));
+	} while (scanning->nextfound && (scanning->nextfound->charoffset_o <= match_end_o || !nextcache_valid(scanning)));
 	DBG_SCANNING("remove_invalid_cache, return invalidoffset %d\n", invalidoffset);
 	return invalidoffset; 
 /*	guint invalidoffset;
@@ -1171,7 +1185,7 @@ bftextview2_run_scanner(BluefishTextView * btv, GtkTextIter * visible_end)
 	/*GArray *matchstack; */
 	Tscanning scanning;
 	guint pos = 0, newpos, reconstruction_o;
-	gboolean end_of_region = FALSE, last_character_run = FALSE, continue_loop = TRUE;
+	gboolean end_of_region = FALSE, last_character_run = FALSE, continue_loop = TRUE, finished;
 	gint loop = 0;
 #ifdef IDENTSTORING
 	GtkTextIter itcursor;
@@ -1270,7 +1284,6 @@ bftextview2_run_scanner(BluefishTextView * btv, GtkTextIter * visible_end)
 	}
 	DBG_SCANNING("scanning from %d to %d\n", gtk_text_iter_get_offset(&scanning.start),
 				 gtk_text_iter_get_offset(&scanning.end));
-	btv->scancache.valid_cache_offset = gtk_text_iter_get_offset(&scanning.start);
 #ifdef HL_PROFILING
 	stage2 = g_timer_elapsed(scanning.timer, NULL);
 #endif
@@ -1399,6 +1412,9 @@ bftextview2_run_scanner(BluefishTextView * btv, GtkTextIter * visible_end)
 	if (gtk_text_iter_compare(&iter, &scanning.end) < 0) {
 		gtk_text_buffer_apply_tag(btv->buffer, btv->needscanning, &iter, &scanning.end);
 	}
+	
+	finished = gtk_text_iter_is_end(&iter);
+	
 #ifdef HL_PROFILING
 	stage4 = g_timer_elapsed(scanning.timer, NULL);
 	hl_profiling.total_runs++;
@@ -1437,6 +1453,12 @@ bftextview2_run_scanner(BluefishTextView * btv, GtkTextIter * visible_end)
 	/* tune the loops_per_timer, try to have 10 timer checks per loop, so we have around 10% deviation from the set interval */
 	if (!end_of_region)
 		loops_per_timer = MAX(loop / 10, 200);
+
+#ifdef DEVELOPMENT
+	if (finished)
+		scancache_check_integrity(btv, scanning.timer);
+#endif
+
 	g_timer_destroy(scanning.timer);
 
 #ifdef VALGRIND_PROFILING
@@ -1446,9 +1468,8 @@ bftextview2_run_scanner(BluefishTextView * btv, GtkTextIter * visible_end)
 #ifdef DUMP_SCANCACHE
 	dump_scancache(btv);
 #endif
-
 	DBG_MSG("cleaned scanning run, finished this run\n");
-	return !gtk_text_iter_is_end(&iter);
+	return !finished;
 }
 
 GQueue *
@@ -1632,6 +1653,75 @@ scan_for_tooltip(BluefishTextView * btv, GtkTextIter * mstart, GtkTextIter * pos
 	return FALSE;
 }
 
+#ifdef DEVELOPMENT
+
+static void
+scancache_check_integrity(BluefishTextView * btv, GTimer *timer) {
+	GQueue contexts;
+	GQueue blocks;
+	GSequenceIter *siter;
+	gfloat start;
+	guint32 prevfound_o=0;
+	
+	start = g_timer_elapsed(timer, NULL);
+	g_queue_init(&contexts);
+	g_queue_init(&blocks);
+	siter = g_sequence_get_begin_iter(btv->scancache.foundcaches);
+	while (siter && !g_sequence_iter_is_end(siter)) {
+		Tfound *found = g_sequence_get(siter);
+		if (!found)
+			break;
+		
+		if (found->charoffset_o <= prevfound_o)
+			g_warning("previous found had offset %d, the next found has offset %d, not ordered correctly?!?!!\n",found->charoffset_o, prevfound_o);
+		prevfound_o = found->charoffset_o;
+		if (found->numcontextchange > 0) {
+			/* push context */
+			if (found->fcontext->parentfcontext != g_queue_peek_head(&contexts)) {
+				g_warning("pushing context at %d:%d, parent contexts at %d:%d do not match!\n"
+									,found->fcontext->start_o, found->fcontext->end_o
+									,((Tfoundcontext *)found->fcontext->parentfcontext)->start_o
+									,((Tfoundcontext *)found->fcontext->parentfcontext)->end_o);
+			}
+			g_queue_push_head(&contexts, found->fcontext);
+		} else {
+			gint i;
+			/* check the current context */
+			if (found->fcontext != g_queue_peek_head(&contexts)) {
+				g_warning("contexts don't match\n");
+			}
+			i = found->numcontextchange;
+			while (i < 0) {
+				g_queue_pop_head(&contexts);
+				i++;
+			}
+		}
+		
+		if (found->numblockchange > 0) {
+			if (found->fblock->parentfblock != g_queue_peek_head(&blocks)) {
+				g_warning("pushing block, parent blocks do not match!\n");
+			}
+			g_queue_push_head(&blocks, found->fblock);
+		} else {
+			gint i;
+			/* check the current context */
+			if (found->fblock != g_queue_peek_head(&blocks)) {
+				g_warning("blocks don't match\n");
+			}
+			i = found->numblockchange;
+			while (i < 0) {
+				g_queue_pop_head(&blocks);
+				i++;
+			}
+		}
+		siter = g_sequence_iter_next(siter);
+	}
+	g_queue_clear(&contexts);
+	g_queue_clear(&blocks);
+	g_print("scancache integrity check done in %3f ms.\n", 1000.0 * (g_timer_elapsed(timer, NULL) - start));
+}
+#endif /* DEVELOPMENT */
+
 void
 cleanup_scanner(BluefishTextView * btv)
 {
