@@ -65,10 +65,11 @@ enum {
 void
 bfwin_fullscreen_toggle(Tbfwin * bfwin, gboolean active)
 {
-	if (active)
-		gtk_window_fullscreen(GTK_WINDOW(bfwin->main_window));
-	else
+	if (active) {
+		gtk_window_fullscreen(GTK_WINDOW(bfwin->main_window));		
+	} else {
 		gtk_window_unfullscreen(GTK_WINDOW(bfwin->main_window));
+	}
 }
 
 static void
@@ -296,7 +297,11 @@ bfwin_side_panel_show_hide_toggle(Tbfwin * bfwin, gboolean first_time, gboolean
 		}
 	}
 	if (show) {
+#if GTK_CHECK_VERSION(3,0,0)
+		bfwin->hpane = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
+#else
 		bfwin->hpane = gtk_hpaned_new();
+#endif
 		if (main_v->props.left_panel_left) {
 			DEBUG_MSG("set paned position to %d (left)\n", main_v->globses.left_panel_width);
 			gtk_paned_set_position(GTK_PANED(bfwin->hpane), main_v->globses.left_panel_width);
@@ -427,6 +432,11 @@ bfwin_configure_event(GtkWidget * widget, GdkEvent * revent, Tbfwin * bfwin)
 				DEBUG_MSG("bfwin_configure_event, NOT-maximized, setting width=%d\n",
 						  main_v->globses.main_window_w);
 			}
+			if (main_v->props.hide_bars_on_fullscreen && (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN)) {
+				gboolean fullscreen = (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN);
+				widget_set_visible(bfwin->toolbarbox, !fullscreen);
+				widget_set_visible(gtk_widget_get_parent(bfwin->statusbar), !fullscreen);
+			}
 		}
 	}
 
@@ -700,8 +710,8 @@ gotoline_entry_insert_text(GtkEditable * editable, gchar * text, gint length, gi
 	}
 }
 
-static void
-gotoline_close_button_clicked(GtkButton * button, Tbfwin * bfwin)
+void
+bfwin_gotoline_search_bar_close(Tbfwin *bfwin)
 {
 	gtk_widget_hide(bfwin->gotoline_frame);
 
@@ -718,9 +728,14 @@ gotoline_close_button_clicked(GtkButton * button, Tbfwin * bfwin)
 
 	g_signal_handlers_unblock_matched(bfwin->gotoline_entry,
 									  G_SIGNAL_MATCH_FUNC, 0, 0, NULL, gotoline_entry_changed, NULL);
+}
+
+static void
+gotoline_close_button_clicked(GtkButton * button, Tbfwin * bfwin)
+{
+	bfwin_gotoline_search_bar_close(bfwin);
 	if (bfwin->current_document)
 		gtk_widget_grab_focus(bfwin->current_document->view);
-	
 }
 
 static gboolean simplesearch_start(Tbfwin *bfwin, gboolean allow_single_char_search) {
@@ -802,9 +817,11 @@ gotoline_frame_create(Tbfwin * bfwin)
 	g_signal_connect(bfwin->gotoline_entry, "changed", G_CALLBACK(gotoline_entry_changed), bfwin);
 	g_signal_connect(bfwin->gotoline_entry, "insert-text", G_CALLBACK(gotoline_entry_insert_text), NULL);
 	g_signal_connect(G_OBJECT(bfwin->gotoline_entry), "key-press-event", G_CALLBACK(gotoline_entries_key_press_event), bfwin);
-	
+#if (GTK_CHECK_VERSION(3,0,0))
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_separator_new(GTK_ORIENTATION_VERTICAL), FALSE, FALSE, 6);
+#else
 	gtk_box_pack_start(GTK_BOX(hbox), gtk_vseparator_new(), FALSE, FALSE, 6);
-
+#endif
 	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Find:")), FALSE, FALSE, 0);
 	bfwin->simplesearch_entry = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(hbox), bfwin->simplesearch_entry, FALSE, FALSE, 0);
@@ -1184,7 +1201,11 @@ bfwin_create_main(Tbfwin * bfwin)
 	}
 
 	/* the area for the middlebox and the outputbox */
+#if GTK_CHECK_VERSION(3,0,0)
+	bfwin->vpane = gtk_paned_new(GTK_ORIENTATION_VERTICAL);
+#else
 	bfwin->vpane = gtk_vpaned_new();
+#endif
 	gtk_box_pack_start(GTK_BOX(vbox), bfwin->vpane, TRUE, TRUE, 0);
 	/* then the area for left panel and */
 	bfwin->middlebox = gtk_hbox_new(TRUE, 0);
@@ -1213,6 +1234,9 @@ bfwin_create_main(Tbfwin * bfwin)
 	gtk_box_pack_start(GTK_BOX(bfwin->notebook_box), bfwin->notebook, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(bfwin->notebook_box), bfwin->notebook_fake, TRUE, TRUE, 0);
 
+	/* here we ask any plugins to build there gui, BEFORE WE START THE SIDEPANEL */
+	g_slist_foreach(main_v->plugins, bfplugins_gui, bfwin);
+
 	/* output_box * /
 	   init_output_box(bfwin, vbox); */
 	bfwin->outputbox = NULL;
@@ -1251,9 +1275,6 @@ bfwin_create_main(Tbfwin * bfwin)
 			gtk_widget_show(hbox);
 	}
 
-	/* here we ask any plugins to build there gui */
-	g_slist_foreach(main_v->plugins, bfplugins_gui, bfwin);
-
 	/* We have to know when the notebook changes */
 	bfwin->notebook_switch_signal =
 		g_signal_connect_after(G_OBJECT(bfwin->notebook), "switch-page", G_CALLBACK(notebook_switch_page),
@@ -1489,7 +1510,12 @@ bfwin_set_title(Tbfwin * bfwin, Tdocument * doc, gint num_modified_change)
 	}
 	tablabel = gtk_label_get_text(GTK_LABEL(doc->tab_label));
 	if (doc->uri) {
-		gchar *curi = g_file_get_uri(doc->uri);
+		gchar *curi;
+		if (g_file_is_native(doc->uri)) {
+			curi = g_file_get_parse_name(doc->uri);
+		} else {
+			curi = g_file_get_uri(doc->uri);
+		}
 		if (bfwin->project) {
 			prfilepart = g_strconcat(bfwin->project->name, " - ", tablabel, " (", curi, ")", NULL);
 		} else {
