@@ -34,6 +34,7 @@
 #include "pixmap.h"
 #include "doc_comments.h"
 #include "doc_text_tools.h"
+#include "doc_extern_tools.h"
 #include "encodings_dialog.h"
 #include "external_commands.h"
 #include "file_dialogs.h"
@@ -43,7 +44,7 @@
 #include "stringlist.h"
 #include "snr3.h"
 #include "undo_redo.h"
-
+#include "print.h"
 
 #define MAIN_MENU_UI PKGDATADIR"/ui/bluefish_menu_ui.xml"
 #define MAIN_TOOLBAR_UI PKGDATADIR"/ui/bluefish_toolbar_ui.xml"
@@ -444,6 +445,13 @@ ui_insert_doc(GtkAction * action, gpointer user_data)
 		file_insert_doc(bfwin);
 }
 
+static void
+ui_print_doc(GtkAction * action, gpointer user_data)
+{
+	Tbfwin *bfwin = BFWIN(user_data);
+	doc_print(bfwin->current_document);
+}
+
 static void
 ui_new_doc(GtkAction * action, gpointer user_data)
 {
@@ -683,6 +691,24 @@ ui_word_count(GtkAction * action, gpointer user_data)
 		doc_word_count(bfwin);
 }
 
+static void
+ui_lorem_ipsum(GtkAction * action, gpointer user_data)
+{
+	lorem_ipsum_dialog(BFWIN(user_data));
+}
+
+static void
+ui_insert_relative_filename(GtkAction * action, gpointer user_data)
+{
+	doc_insert_filename(BFWIN(user_data)->current_document, TRUE);
+}
+
+static void
+ui_insert_absolute_filename(GtkAction * action, gpointer user_data)
+{
+	doc_insert_filename(BFWIN(user_data)->current_document, FALSE);
+}
+
 /* view action callbacks */
 
 static void
@@ -730,13 +756,14 @@ ui_browser_preview(GtkAction * action, gpointer user_data)
 	/* TODO find the default browser and start it */
 	for (list = g_list_first(main_v->props.external_command); list; list = list->next) {
 		gchar **arr = list->data;
-		/*  arr[0] = name
-		 *  arr[1] = command
-		 *  arr[2] = is_default_browser
+		/* arr[0] = enabled
+		 * arr[1] = name
+		 * arr[2] = command
+		 * arr[3] = is_default_browser
 		 */
-		if (g_strv_length(arr) == 3 && arr[2][0] == '1') {
-			DEBUG_MSG("ui_browser_preview, start %s\n",arr[1]);
-			external_command(BFWIN(user_data), arr[1]);
+		if (g_strv_length(arr) == 4 && arr[3][0] == '1') {
+			DEBUG_MSG("ui_browser_preview, start %s\n",arr[2]);
+			external_command(BFWIN(user_data), arr[2]);
 		}
 	}
 }
@@ -759,7 +786,10 @@ static const GtkActionEntry top_level_menus[] = {
 	{"ToolsMenu", NULL, N_("_Tools")},
 	{"ToolsCommands", NULL, N_("_Commands")},
 	{"ToolsFilters", NULL, N_("_Filters")},
-	{"ToolsOutputBox", NULL, N_("_Outputbox")}
+	{"ToolsOutputBox", NULL, N_("_Outputbox")},
+	{"ToolsFormat", NULL, N_("Fo_rmat")},
+	{"ToolsInsert", NULL, N_("_Insert")},
+	{"ToolsConvert", NULL, N_("Co_nvert")}
 };
 
 static const GtkActionEntry global_actions[] = {
@@ -773,6 +803,7 @@ static const GtkActionEntry global_actions[] = {
 	{"FileOpenSelection", NULL, N_("Open _from Selection"), NULL, N_("Open from Selection"),
 	 G_CALLBACK(ui_open_from_selection)},
 	{"FileInsert", NULL, N_("_Insert..."), NULL, N_("Insert file"), G_CALLBACK(ui_insert_doc)},
+	{"FilePrint", NULL, N_("_Print"), NULL, N_("Print"), G_CALLBACK(ui_print_doc)},
 	{"FileUploadDownload", NULL, N_("U_pload / Download..."), NULL, NULL,
 	 G_CALLBACK(ui_upload_download_dialog)},
 	{"FileQuit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q", N_("Quit Bluefish"), G_CALLBACK(ui_quit)},
@@ -785,11 +816,11 @@ static const GtkActionEntry global_actions[] = {
 	 G_CALLBACK(ui_encoding_add_remove)},
 	{"DocFloatingWindow", NULL, N_("Floati_ng Window"), NULL, N_("Open current document in floating window"),
 	 G_CALLBACK(ui_floating_window)},
-	{"DocFontSizeIncrease", NULL, N_("_Increase"), NULL, N_("Increase document font size"),
+	{"DocFontSizeIncrease", NULL, N_("_Increase"), "<control>equal", N_("Increase document font size"),
 	 G_CALLBACK(ui_font_size_increase)},
-	{"DocFontSizeDecrease", NULL, N_("_Decrease"), NULL, N_("Decrease document font size"),
+	{"DocFontSizeDecrease", NULL, N_("_Decrease"), "<control>minus", N_("Decrease document font size"),
 	 G_CALLBACK(ui_font_size_decrease)},
-	{"DocFontSizeReset", NULL, N_("_Reset"), NULL, N_("Reset document font size"),
+	{"DocFontSizeReset", NULL, N_("_Reset"), "<Control>KP_0", N_("Reset document font size"),
 	 G_CALLBACK(ui_font_size_reset)},
 	{"DocTabSizeIncrease", NULL, N_("_Increase"), NULL, N_("Increase document tab size"),
 	 G_CALLBACK(ui_tab_size_increase)},
@@ -817,6 +848,9 @@ static const GtkActionEntry global_actions[] = {
 	 {"DeleteLine", NULL, N_("Delete Line"), "<control>y", N_("Delete the current line"),
 	 G_CALLBACK(ui_delete_line)},
 	{"WordCount", NULL, N_("Word Cou_nt"), NULL, N_("Word count"), G_CALLBACK(ui_word_count)},
+	{"LoremIpsum", NULL, N_("Lorem Ipsum generator"), NULL, N_("Lorem Ipsum generator"), G_CALLBACK(ui_lorem_ipsum)},
+	{"RelativeFilename", NULL, N_("Insert Relative Filename"), NULL, N_("Insert Relative Filename"), G_CALLBACK(ui_insert_relative_filename)},
+	{"AbsoluteFilename", NULL, N_("Insert Absolute Filename"), NULL, N_("Insert Absolute Filename"), G_CALLBACK(ui_insert_absolute_filename)},
 	{"IndentingToSpaces", NULL, N_("Indenting to S_paces"), NULL, N_("Indenting to spaces"),
 	 G_CALLBACK(ui_indenting_to_spaces)},
 	{"IndentingToTabs", NULL, N_("Indenting to T_abs"), NULL, N_("Indenting to tabs"),
@@ -1018,7 +1052,7 @@ bfwin_main_ui_init(Tbfwin * bfwin, GtkWidget * vbox)
 	manager = bfwin->uimanager;
 
 	gtk_window_add_accel_group(GTK_WINDOW(bfwin->main_window), gtk_ui_manager_get_accel_group(manager));
-
+	
 	action_group = gtk_action_group_new("topLevelMenus");
 	gtk_action_group_set_translation_domain(action_group, GETTEXT_PACKAGE);
 	gtk_action_group_add_actions(action_group, top_level_menus, G_N_ELEMENTS(top_level_menus), bfwin);
@@ -1202,8 +1236,8 @@ bfwin_set_document_menu_items(Tdocument * doc)
 	bfwin_action_set_sensitive(manager, "/MainMenu/EditMenu/Paste", !doc->readonly);
 	bfwin_action_set_sensitive(manager, "/MainMenu/EditMenu/Replace", !doc->readonly);
 	/*bfwin_action_set_sensitive(manager, "/MainMenu/EditMenu/ReplaceAgain", !doc->readonly);*/
-	bfwin_action_set_sensitive(manager, "/MainMenu/EditMenu/EditIndent", !doc->readonly);
-	bfwin_action_set_sensitive(manager, "/MainMenu/EditMenu/EditUnindent", !doc->readonly);
+	bfwin_action_set_sensitive(manager, "/MainMenu/ToolsMenu/ToolsFormat/EditIndent", !doc->readonly);
+	bfwin_action_set_sensitive(manager, "/MainMenu/ToolsMenu/ToolsFormat/EditUnindent", !doc->readonly);
 }
 
 void
@@ -1333,8 +1367,8 @@ static void
 commands_menu_activate(GtkAction * action, gpointer user_data)
 {
 	gchar **arr = g_object_get_data(G_OBJECT(action), "adata");
-	DEBUG_MSG("commands_menu_activate, call external_command with bfwin=%p and command_string=%s\n",user_data,arr[1]);
-	external_command(BFWIN(user_data), arr[1]);
+	DEBUG_MSG("commands_menu_activate, call external_command with bfwin=%p and command_string=%s\n",user_data,arr[2]);
+	external_command(BFWIN(user_data), arr[2]);
 }
 
 void
@@ -1351,24 +1385,21 @@ bfwin_commands_menu_create(Tbfwin * bfwin)
 	bfwin->commands_merge_id = gtk_ui_manager_new_merge_id(bfwin->uimanager);
 	for (list = g_list_first(main_v->props.external_command); list; list = list->next) {
 		gchar **arr = list->data;
-		/*  arr[0] = name
-		 *  arr[1] = command
-		 *  arr[2] = is_default_browser
+		/*  arr[0] = state
+		 *  arr[1] = name
+		 *  arr[2] = command
+		 *  arr[3] = is_default_browser
 		 */
-		if (g_strv_length(arr) == 3) {
-			
-			if (arr[2][0] == '1') {
+		if (g_strv_length(arr) == 4 &&(arr[0][0]=='1'||arr[0][0]=='3')) {
+			if (arr[3][0] == '1') {
 				dynamic_menu_item_create(bfwin->uimanager,bfwin->commands_group, bfwin->commands_merge_id, 
 						"/MainMenu/ToolsMenu/DefaultBrowserPlaceholder", 
-						arr[0], arr[0], -1, NULL, G_CALLBACK(commands_menu_activate), bfwin, arr);
+						arr[1], arr[1], -1, NULL, G_CALLBACK(commands_menu_activate), bfwin, arr);
 			} else {
 				dynamic_menu_item_create(bfwin->uimanager,bfwin->commands_group, bfwin->commands_merge_id, 
 						"/MainMenu/ToolsMenu/ToolsCommands/CommandsPlaceholder", 
-						arr[0], arr[0], -1, NULL, G_CALLBACK(commands_menu_activate), bfwin, arr);
+						arr[1], arr[1], -1, NULL, G_CALLBACK(commands_menu_activate), bfwin, arr);
 			}
-		} else {
-			DEBUG_MSG("bfwin_commands_menu_create, CORRUPT ENTRY IN command action; array count =%d\n",
-					  g_strv_length(arr));
 		}
 	}
 }
@@ -1446,7 +1477,7 @@ filter_dialog_response(GtkWidget * widget, gint response_id, gpointer user_data)
 	doc_restore_selection(fd->selsave, TRUE);	/* the restore will also free the Tselectionsave */
 	if (response_id == 1 && fd->bfwin->current_document)
 		doc_get_selection(fd->bfwin->current_document, &begin, &end);
-
+	DEBUG_MSG("filter_dialog_response, calling filter_command for %s\n", fd->command);
 	filter_command(fd->bfwin, fd->command, begin, end);
 	g_slice_free(Tfilterdialog, fd);
 }
@@ -1456,7 +1487,7 @@ filters_menu_activate(GtkAction * action, gpointer user_data)
 {
 	Tbfwin *bfwin = BFWIN(user_data);
 	gchar **arr = g_object_get_data(G_OBJECT(action), "adata");
-	const gchar *command = arr[1];
+	const gchar *command = arr[2];
 	gint begin = 0, end = -1;
 	/* if we have a selection, and the filter can be used on a selection,
 	   we should ask if it should be the complete file or the selection */
@@ -1502,13 +1533,14 @@ bfwin_filters_menu_create(Tbfwin * bfwin)
 
 	for (list = g_list_first(main_v->props.external_filter); list; list = list->next) {
 		gchar **arr = list->data;
-		/*  arr[0] = name
-		 *  arr[1] = command
+		/* arr[0] = state 
+		 * arr[1] = name
+		 * arr[2] = command
 		 */
-		if (g_strv_length(arr) == 2) {
+		if (g_strv_length(arr) == 3 && (arr[0][0]=='1' || arr[0][0]=='3')) {
 			dynamic_menu_item_create(bfwin->uimanager, bfwin->filters_group, 
 						bfwin->filters_merge_id, "/MainMenu/ToolsMenu/ToolsFilters/FiltersPlaceholder", 
-						arr[0], arr[0], -1, NULL,  
+						arr[1], arr[1], -1, NULL,  
 						G_CALLBACK(filters_menu_activate), bfwin, arr);
 		} else {
 			DEBUG_MSG("bfwin_filters_menu_create, CORRUPT ENTRY IN filter actions; array count =%d\n",
@@ -1521,7 +1553,8 @@ static void
 outputbox_menu_activate(GtkAction * action, gpointer user_data)
 {
 	gchar **arr = g_object_get_data(G_OBJECT(action), "adata");
-	outputbox(BFWIN(user_data), arr[1], atoi(arr[2]), atoi(arr[3]), atoi(arr[4]), arr[5]);
+	DEBUG_MSG("outputbox_menu_activate, open outputbox with command %s and filter %s\n",arr[6],arr[2]);
+	outputbox(BFWIN(user_data), arr[2], atoi(arr[3]), atoi(arr[4]), atoi(arr[5]), arr[6]);
 }
 
 void
@@ -1540,18 +1573,18 @@ bfwin_outputbox_menu_create(Tbfwin * bfwin)
 
 	for (list = g_list_first(main_v->props.external_outputbox); list; list = list->next) {
 		gchar **arr = list->data;
-		/* arr[0] = name
-		 * arr[1] = pattern
-		 * arr[2] = file subpattern     gint
-		 * arr[3] = line subpattern     gint
-		 * arr[4] = output subpattern   gint
-		 * arr[5] = command
-		 * arr[6] = show_all_output     gboolean not used
+		/* arr[0] = state
+		 * arr[1] = name
+		 * arr[2] = pattern
+		 * arr[3] = file subpattern     gint
+		 * arr[4] = line subpattern     gint
+		 * arr[5] = output subpattern   gint
+		 * arr[6] = command
 		 */
-		if (g_strv_length(arr) == 6) {
+		if (g_strv_length(arr) == 7 && (arr[0][0]=='1'||arr[0][0]=='3')) {
 			dynamic_menu_item_create(bfwin->uimanager, bfwin->outputbox_group, 
 						bfwin->outputbox_merge_id, "/MainMenu/ToolsMenu/ToolsOutputBox/OutputBoxPlaceholder", 
-						arr[0], arr[0], -1, NULL,  
+						arr[1], arr[1], -1, NULL,  
 						G_CALLBACK(outputbox_menu_activate), bfwin, arr);
 		} else {
 			DEBUG_MSG("bfwin_outputbox_menu_create, CORRUPT ENTRY IN outputbox action; array count =%d\n",
