@@ -666,6 +666,8 @@ bmark_check_remove(Tbfwin * bfwin, Tbmark * b)
 				if (b->doc)
 					b->doc->bmark_parent = NULL;
 			}
+			bluefish_text_view_set_show_symbols_redraw(BLUEFISH_TEXT_VIEW(b->doc->view), FALSE);
+			
 			return FALSE;
 		}
 	}
@@ -1394,7 +1396,6 @@ bookmark_data_new(void)
 											NULL);
 	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(bmd->bookmarkstore),
 										 GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
-	/* BUG: shouldn't we free the data in the hash table when we close a project? */
 	bmd->bmarkfiles = g_hash_table_new_full(g_file_hash, (GEqualFunc) g_file_equal, bmark_hash_key_free, bmark_hash_value_free);
 	DEBUG_MSG("bookmark_data_new, created bookmarkstore at %p\n", bmd->bookmarkstore);
 	return bmd;
@@ -1537,6 +1538,7 @@ void
 bmark_clean_for_doc(Tdocument * doc)
 {
 	GtkTreeIter tmpiter;
+	GtkTreePath *path;
 	gboolean cont;
 
 	if (doc->bmark_parent == NULL)
@@ -1582,7 +1584,11 @@ bmark_clean_for_doc(Tdocument * doc)
 			  doc);
 	gtk_tree_store_set(GTK_TREE_STORE(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore),
 					   doc->bmark_parent, PTR_COLUMN, NULL, -1);
+	gtk_tree_model_filter_convert_child_iter_to_iter(BFWIN(doc->bfwin)->bmarkfilter,&tmpiter,doc->bmark_parent);
 	doc->bmark_parent = NULL;
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(BFWIN(doc->bfwin)->bmarkfilter),&tmpiter);
+	gtk_tree_view_collapse_row(BFWIN(doc->bfwin)->bmark,path);
+	gtk_tree_path_free(path);
 }
 
 static gboolean
@@ -1649,6 +1655,10 @@ bookmark_needs_repositioning(Tbmark * mark, GtkTextIter * it)
 void
 bmark_set_for_doc(Tdocument * doc, gboolean check_positions)
 {
+	gboolean cont2;
+	GtkTreeIter child;
+	GtkTreePath *path;
+
 	if (!doc->uri) {
 		DEBUG_MSG("bmark_set_for_doc, document %p does not have a filename, returning\n", doc);
 		return;
@@ -1666,43 +1676,46 @@ bmark_set_for_doc(Tdocument * doc, gboolean check_positions)
 	}
 
 	doc->bmark_parent = g_hash_table_lookup(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bmarkfiles, doc->uri);
-	if (doc->bmark_parent) {
-		gboolean cont2;
-		GtkTreeIter child;
-		/*g_print("bmark_set_for_doc, we found a bookmark for document %s at offset=%d!\n",gtk_label_get_text(GTK_LABEL(doc->tab_menu)),mark->offset); */
-		gtk_tree_store_set(GTK_TREE_STORE(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore),
-						   doc->bmark_parent, PTR_COLUMN, doc, -1);
+	if (!doc->bmark_parent)
+		return;
+		
+	/*g_print("bmark_set_for_doc, we found a bookmark for document %s at offset=%d!\n",gtk_label_get_text(GTK_LABEL(doc->tab_menu)),mark->offset); */
+	gtk_tree_store_set(GTK_TREE_STORE(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore),
+					   doc->bmark_parent, PTR_COLUMN, doc, -1);
 
-		cont2 =
-			gtk_tree_model_iter_children(GTK_TREE_MODEL
-										 (BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore), &child,
-										 doc->bmark_parent);
-		while (cont2) {			/* loop the bookmarks for this document  */
-			Tbmark *mark = NULL;
-			gtk_tree_model_get(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore), &child,
-							   PTR_COLUMN, &mark, -1);
-			if (mark) {
-				GtkTextIter it;
-				mark->doc = doc;
-				DEBUG_MSG("bmark_set_for_doc, next bookmark at offset=%d!\n", mark->offset);
-				gtk_text_buffer_get_iter_at_offset(doc->buffer, &it, mark->offset);
-				if (check_positions && bookmark_needs_repositioning(mark, &it)) {	/* repositioning required ! */
-					if (bookmark_reposition(mark, gtk_text_buffer_get_char_count(doc->buffer) - mark->len)) {
-						gtk_text_buffer_get_iter_at_offset(doc->buffer, &it, mark->offset);
-					} else {
-						/* BUG: bookmark not restored, what to do now ???? - just put it where it was ??  */
-					}
+	cont2 =
+		gtk_tree_model_iter_children(GTK_TREE_MODEL
+									 (BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore), &child,
+									 doc->bmark_parent);
+	while (cont2) {			/* loop the bookmarks for this document  */
+		Tbmark *mark = NULL;
+		gtk_tree_model_get(GTK_TREE_MODEL(BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore), &child,
+						   PTR_COLUMN, &mark, -1);
+		if (mark) {
+			GtkTextIter it;
+			mark->doc = doc;
+			DEBUG_MSG("bmark_set_for_doc, next bookmark at offset=%d!\n", mark->offset);
+			gtk_text_buffer_get_iter_at_offset(doc->buffer, &it, mark->offset);
+			if (check_positions && bookmark_needs_repositioning(mark, &it)) {	/* repositioning required ! */
+				if (bookmark_reposition(mark, gtk_text_buffer_get_char_count(doc->buffer) - mark->len)) {
+					gtk_text_buffer_get_iter_at_offset(doc->buffer, &it, mark->offset);
+				} else {
+					/* BUG: bookmark not restored, what to do now ???? - just put it where it was ??  */
 				}
-				mark->mark = gtk_text_buffer_create_mark(doc->buffer, NULL, &it, TRUE);
 			}
-			cont2 =
-				gtk_tree_model_iter_next(GTK_TREE_MODEL
-										 (BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore), &child);
+			mark->mark = gtk_text_buffer_create_mark(doc->buffer, NULL, &it, TRUE);
 		}
-
-	} else {
-		DEBUG_MSG("no bookmarks for doc %p in hashtable\n", doc);
+		cont2 =
+			gtk_tree_model_iter_next(GTK_TREE_MODEL
+									 (BMARKDATA(BFWIN(doc->bfwin)->bmarkdata)->bookmarkstore), &child);
 	}
+	
+	/* expand it */
+	gtk_tree_model_filter_convert_child_iter_to_iter(BFWIN(doc->bfwin)->bmarkfilter,&child,doc->bmark_parent);
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(BFWIN(doc->bfwin)->bmarkfilter),&child);
+	gtk_tree_view_expand_row(BFWIN(doc->bfwin)->bmark,path,TRUE);
+	gtk_tree_path_free(path);
+	bluefish_text_view_set_show_symbols_redraw(BLUEFISH_TEXT_VIEW(doc->view), TRUE);
 }
 
 /* this is called by the editor widget to show bookmarks in the left margin.
@@ -1795,9 +1808,8 @@ bmark_add_backend(Tdocument * doc, GtkTextIter * itoffset, gint offset, const gc
 	if (!m->is_temp) {
 		bmark_store(BFWIN(doc->bfwin), m);
 	}
-	gtk_widget_queue_draw(doc->view);
-	if (doc->slave)
-		gtk_widget_queue_draw(doc->slave);
+	DEBUG_MSG("bmark_add_backend, set show symbols to true and call for redraw\n");
+	bluefish_text_view_set_show_symbols_redraw(BLUEFISH_TEXT_VIEW(doc->view), TRUE);
 	return m;
 }
 
