@@ -28,6 +28,8 @@
 #include "document.h"
 #include "gtk_easy.h"
 #include "outputbox.h"
+#include "dialog_utils.h"
+#include "external_commands.h"
 
 #ifndef WIN32
 #define USEBINSH
@@ -50,21 +52,15 @@ typedef struct {
 	gint end; /* -1 for end of the file */
 	const gchar *formatstring; /* the command from the configuration, so including placeholders such as %i, %f, |,  etc. */
 	gchar *commandstring; /* the command that will be started by g_spawn, the placeholders should have been replaced/removed */
+	CustomCommandCallback customcommand_cb;
 
-	gchar *securedir; /* if we need any local temporary files for input or output we'll use fifo's */
-	gchar *fifo_in; /* the path for the iput and output fifo */
-	gchar *fifo_out;
+	gchar *securedir; /* if we need any local temporary files for input or output */
 	gchar *tmp_in; /* the path for the input/output temporary filename */
 	gchar *tmp_out;
 	gchar *inplace; /* if we use in-place editing, so the input file will become the output file */
 	gboolean pipe_in; /* if we use pipes for input and output */
 	gboolean pipe_out;
 
-#ifdef WIN32
-	gpointer fifo_in_handle; /* the path for 'named pipes' on win32 */
-	gpointer fifo_out_handle;
-#endif
-
 	gboolean include_stderr; /* if stderr should be included in the output */
 	
 	gint standard_input; /* the pipes that will be filled by g_spawn_async_with_pipes */
@@ -90,24 +86,6 @@ static void externalp_unref(Texternalp *ep) {
 	if (ep->refcount <= 0) {
 		DEBUG_MSG("externalp_unref, cleanup!\n");
 		if (ep->commandstring) g_free(ep->commandstring);
-		if (ep->fifo_in) {
-			#ifdef WIN32
-				CloseHandle(ep->fifo_in_handle);
-				g_free(ep->fifo_in_handle);
-			#else
-				unlink(ep->fifo_in);
-			#endif
-			g_free(ep->fifo_in);
-		}
-		if (ep->fifo_out) {
-			#ifdef WIN32
-				CloseHandle(ep->fifo_out_handle);
-				g_free(ep->fifo_out_handle);
-			#else
-				unlink(ep->fifo_out);
-			#endif
-			g_free(ep->fifo_out);
-		}
 		if (ep->tmp_in) {
 			unlink(ep->tmp_in);
 			g_free(ep->tmp_in);
@@ -183,7 +161,7 @@ static void child_watch_lcb(GPid pid,gint status,gpointer data) {
 	GError *gerror=NULL;
 	DEBUG_MSG("child_watch_lcb, child exited with status=%d\n",status);
 	
-	if ((ep->pipe_in || ep->fifo_in) && !ep->buffer_out) {
+	if (ep->pipe_in && !ep->buffer_out) {
 		/* the child has exited before we actually started to write data to the child, just abort now */
 		g_source_remove(ep->start_command_idle_id);
 		externalp_unref(ep); /* unref twice, once for the start_command_idle, once for child_watch_lcb */
@@ -222,8 +200,6 @@ static gboolean
 start_command_idle(gpointer data)
 {
 	Texternalp *ep = data;
-	GError *gerror=NULL;
-
 	if (ep->pipe_in) {
 		DEBUG_MSG("start_command_idle, creating channel_in from pipe\n");
 #ifdef WIN32
@@ -231,20 +207,8 @@ start_command_idle(gpointer data)
 #else
 		ep->channel_in = g_io_channel_unix_new(ep->standard_input);
 #endif
-	} else if (ep->fifo_in) {
-		DEBUG_MSG("start_command_idle, connecting channel_in to fifo %s\n",ep->fifo_in);
-		/* problem: this can hang if there is no process actually reading from this fifo... so if the 
-		command died in some way (a nonexisting command), this call will hang bluefish */
-		ep->channel_in = g_io_channel_new_file(ep->fifo_in,"w",&gerror);
-		if (gerror) {
-			DEBUG_MSG("start_command_idle, error connecting to fifo %s: %s\n",ep->fifo_in,gerror->message);
-			g_warning("failed to create fifo %s %d: %s\n",ep->fifo_in, gerror->code, gerror->message);
-			g_error_free(gerror);
-			/* BUG TODO: free Texternalp */
-			return FALSE;
-		}
 	}
-	if (ep->pipe_in || ep->fifo_in) {
+	if (ep->pipe_in) {
 		ep->refcount++;
 		ep->buffer_out = ep->buffer_out_position = doc_get_chars(ep->bfwin->current_document,ep->begin,ep->end);
 		g_io_channel_set_flags(ep->channel_in,G_IO_FLAG_NONBLOCK,NULL);
@@ -259,17 +223,8 @@ start_command_idle(gpointer data)
 		ep->channel_out = g_io_channel_unix_new(ep->standard_output);
 #endif
 		DEBUG_MSG("start_command_idle, created channel_out from pipe\n");
-	} else if (ep->fifo_out) {
-		ep->channel_out = g_io_channel_new_file(ep->fifo_out,"r",&gerror);
-		DEBUG_MSG("start_command_idle, created channel_out from fifo %s\n",ep->fifo_out);
-		if (gerror) {
-			g_warning("failed to start reading from file %s %d: %s\n",ep->fifo_out, gerror->code, gerror->message);
-			g_error_free(gerror);
-			/* BUG TODO: free Texternalp */
-			return FALSE;
-		}
 	}
-	if (ep->channel_out_lcb && (ep->pipe_out || ep->fifo_out)) {
+	if (ep->channel_out_lcb && ep->pipe_out) {
 		ep->refcount++;
 		g_io_channel_set_flags(ep->channel_out,G_IO_FLAG_NONBLOCK,NULL);
 		DEBUG_MSG("start_command_idle, add watch for channel_out\n");
@@ -277,7 +232,6 @@ start_command_idle(gpointer data)
 	}
 	externalp_unref(ep);
 	
-	
 	return FALSE; /* don't call me again */
 }
 
@@ -301,40 +255,8 @@ static void start_command_backend(Texternalp *ep) {
     g_shell_parse_argv(ep->commandstring, NULL, &argv, NULL);
 #endif
 	DEBUG_MSG("start_command_backend,commandstring=%s\n",ep->commandstring);
-#ifdef WIN32
 	ep->include_stderr = FALSE;
-	if (ep->fifo_in) {
-		ep->fifo_in = g_strdup("\\\\.\\pipe\\bluefish_in");
-		if ((ep->fifo_in_handle = CreateNamedPipe(ep->fifo_in, PIPE_ACCESS_INBOUND, PIPE_TYPE_BYTE | PIPE_NOWAIT, 
-			 PIPE_UNLIMITED_INSTANCES, 2048, 2048, 100, NULL)) == INVALID_HANDLE_VALUE) {
-			g_print("some error happened creating fifo %s??\n", ep->fifo_in);
-			return;
-		}
-	}
-	if (ep->fifo_out) {
-		ep->fifo_out = g_strdup("\\\\.\\pipe\\bluefish_out");
-		if ((ep->fifo_out_handle = CreateNamedPipe(ep->fifo_out, PIPE_ACCESS_OUTBOUND, PIPE_TYPE_BYTE | PIPE_NOWAIT, 
-			 PIPE_UNLIMITED_INSTANCES, 2048, 2048, 100, NULL)) == INVALID_HANDLE_VALUE) {
-			g_print("some error happened creating fifo %s??\n", ep->fifo_out);
-			return;
-		}
-	}
-#else
-	if (ep->fifo_in) {
-		if (mkfifo(ep->fifo_in, 0600) != 0) {
-			DEBUG_MSG("some error happened creating fifo %s??\n",ep->fifo_in);
-			return;
-		}
-		DEBUG_MSG("start_command_backend, created fifo %s\n",ep->fifo_in);
-	}
-	if (ep->fifo_out) {
-		if (mkfifo(ep->fifo_out, 0600) != 0) {
-			DEBUG_MSG("some error happened creating fifo %s??\n",ep->fifo_out);
-			return;
-		}
-	}
-#endif
-	DEBUG_MSG("start_command_backend, pipe_in=%d, pipe_out=%d, fifo_in=%s, fifo_out=%s,include_stderr=%d\n",ep->pipe_in,ep->pipe_out,ep->fifo_in,ep->fifo_out,ep->include_stderr);
+	DEBUG_MSG("start_command_backend, pipe_in=%d, pipe_out=%d, include_stderr=%d\n",ep->pipe_in,ep->pipe_out,ep->include_stderr);
 	DEBUG_MSG("start_command_backend, about to spawn process /bin/sh -c %s\n",argv[2]);
 	g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH, (ep->include_stderr)?spawn_setup_lcb:NULL, ep, &ep->child_pid, 
 				(ep->pipe_in) ? &ep->standard_input : NULL,
@@ -383,21 +305,20 @@ gboolean operatable_on_selection(const gchar *formatstring) {
 } 
 
 /* The format string should have new options:
-    * %flocal full path (function should abort for remote files)
+    * %f local full path (function should abort for remote files)
     * %c local directory of file (function should abort for remote files)
     * %n filename without path
     * %u URL
     * %p preview URL if basedir and preview dir are set in project settings, else identical to %u
-    * %i temporary fifo for input, if the document is not modified and local equal to %f
-    * %o temporary fifo for output of filters or outputbox
-    * %I temporary filename for input (fifo is faster), if the document is not modified and local equal to %f
-    * %O temporary filename for output of filters or outputbox (fifo is faster) (previously %f)
+    * %i temporary filename for input (fifo is faster), if the document is not modified and local equal to %f
+    * %o temporary filename for output of filters or outputbox (fifo is faster) (previously %f)
     * %t temporary filename for both input and output (for in-place-editing filters)
+    * %a user supplied arguments
 */
 static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gboolean discard_output) {
 	Tconvert_table *table;
 	gchar *localname=NULL, *localfilename=NULL, *retstring, *curi=NULL;
-	gchar *formatstring;
+	gchar *formatstring, *user_argument=NULL;
 	gint formatstringlen;
 	gboolean is_local_non_modified;
 	gboolean need_local = FALSE,
@@ -405,11 +326,10 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 		need_pipeout = FALSE,
 		need_tmpin = FALSE,
 		need_tmpout = FALSE,
-		need_fifoin = FALSE,
-		need_fifoout = FALSE,
 		need_inplace = FALSE,
 		need_preview_uri = FALSE,
-		need_filename=FALSE;
+		need_filename=FALSE,
+		need_arguments=FALSE;
 	gint items = 2, cur=0;
 	
 	if (!ep->bfwin->current_document) {
@@ -428,7 +348,7 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 		need_pipeout = TRUE;
 		formatstring[formatstringlen-1]=' ';
 	}
-	
+	need_arguments = (strstr(formatstring, "%a") != NULL);
 	need_filename = need_local = (strstr(formatstring, "%c") != NULL || strstr(formatstring, "%f") != NULL);
 	need_preview_uri = (strstr(formatstring, "%p") != NULL);
 	if (!need_filename) { /* local already implies we need a filename */
@@ -446,53 +366,70 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 	if (need_local && !localname) {
 		g_free(formatstring);
 		g_free(localname);
-
 		return NULL;
 	}
 
 	need_tmpin = (strstr(formatstring, "%I") != NULL);
 	/* %f is for backwards compatibility with bluefish 1.0 */
 	need_tmpout = (strstr(formatstring, "%O") != NULL);
-	need_fifoin = (strstr(formatstring, "%i") != NULL);
-	need_fifoout = (strstr(formatstring, "%o") != NULL);
 	need_inplace = (strstr(formatstring, "%t") != NULL);
 	
-	if ((need_tmpout || need_fifoout || need_inplace || need_pipeout) && discard_output) {
+	if ((need_tmpout || need_inplace || need_pipeout) && discard_output) {
 		DEBUG_MSG("create_commandstring, external_commands should not have %%o\n");
-		/*  BUG: give a warning that external commands should not use %o */
 		g_free(formatstring);
 		g_free(localname);
-
 		return NULL;
 	}
-	if ((need_tmpin && (need_fifoin || need_inplace || need_pipein)) 
-			|| (need_fifoin && (need_inplace || need_pipein))
+	if ((need_tmpin && (need_inplace || need_pipein)) 
 			|| (need_inplace && need_pipein)) {
 		DEBUG_MSG("create_commandstring, cannot have multiple inputs\n");
 		/*  BUG: give a warning that you cannot have multiple inputs */
 		g_free(formatstring);
 		g_free(localname);
-
 		return NULL;
 	}
-	if ((need_tmpout && (need_fifoout || need_inplace || need_pipeout)) 
-			|| (need_fifoout && (need_inplace || need_pipeout))
+	if ((need_tmpout && (need_inplace || need_pipeout)) 
 			|| (need_inplace && need_pipeout)) {
-		DEBUG_MSG("create_commandstring, cannot have multiple outputs\n");
+		DEBUG_MSG("create_commandstring, cannot have multiple outputs, tmpout=%d, pipeout=%d, inplace=%d\n",need_tmpout,need_pipeout,need_inplace);
 		/*  BUG: give a warning that you cannot have multiple outputs */
 		g_free(formatstring);
 		g_free(localname);
 
 		return NULL;
 	}
+	
+	if (need_arguments) {
+		GtkWidget *dialog, *entry;
+		gchar *tmp;
+		gint result;
+		dialog = gtk_dialog_new_with_buttons(_("Supply extra arguments"),
+											  GTK_WINDOW(ep->bfwin->main_window),
+											  GTK_DIALOG_DESTROY_WITH_PARENT,
+											  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+											  GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
+		tmp = g_strdup_printf(_("Supply arguments to define %%a in '%s'"), formatstring);
+		entry = dialog_entry_labeled(NULL, tmp, gtk_dialog_get_content_area(GTK_DIALOG(dialog)), 6);
+		g_free(tmp);
+		gtk_widget_show_all(dialog);
+		result = gtk_dialog_run(GTK_DIALOG (dialog));
+		switch (result) {
+		case GTK_RESPONSE_ACCEPT:
+			user_argument = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
+			if (!user_argument || user_argument[0]=='\0') {
+				need_arguments = FALSE;
+			}
+		break;
+		default:
+			gtk_widget_destroy(dialog);
+			return NULL;
+		break;
+		}
+		gtk_widget_destroy(dialog);
+	}
+	
+	
 	if (need_pipein)  ep->pipe_in = TRUE;
 	if (need_pipeout) ep->pipe_out = TRUE;
-/*	if (!need_tmpin && !need_fifoin && !need_inplace && !need_filename && !need_preview_uri) {
-		ep->pipe_in = TRUE;
-	}
-	if (!discard_output && !need_tmpout && !need_fifoout && !need_inplace) {
-		ep->pipe_out = TRUE;
-	}*/
 	DEBUG_MSG("create_commandstring, formatstring '%s' seems OK\n",formatstring);
 
 	is_local_non_modified = (ep->bfwin->current_document->uri 
@@ -504,7 +441,7 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 	if (need_filename || is_local_non_modified) {
 		curi = g_file_get_uri(ep->bfwin->current_document->uri);
 
-		if (need_local || (is_local_non_modified && (need_tmpin||need_fifoin))) {
+		if (need_local || (is_local_non_modified && need_tmpin)) {
 			#ifdef WIN32
 				localfilename = strrchr(localname, '\\')+1;
 			#else
@@ -518,9 +455,8 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 	}
 	if (need_tmpin) items++;
 	if (need_tmpout) items++;
-	if (need_fifoin) items++;
-	if (need_fifoout) items++;
 	if (need_inplace) items++;
+	if (need_arguments) items++;
 
 	table = g_new(Tconvert_table, items+1);
 	if (need_filename) {
@@ -552,17 +488,6 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 			table[cur].my_char = g_strdup(ep->tmp_in);
 		}
 		cur++;
-	} else if (need_fifoin) {
-		table[cur].my_int = 'i';
-		if (is_local_non_modified) {
-			table[cur].my_char = g_strdup(localname);;
-		} else {
-			ep->fifo_in = create_secure_dir_return_filename();
-			table[cur].my_char = g_strdup(ep->fifo_in);
-		}
-		
-		DEBUG_MSG("create_commandstring, %%i will be at %s\n",table[cur].my_char);
-		cur++;
 	} else if (need_inplace) {
 		table[cur].my_int = 't';
 		ep->inplace = create_secure_dir_return_filename();
@@ -575,12 +500,6 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 		ep->tmp_out = create_secure_dir_return_filename();
 		table[cur].my_char = g_strdup(ep->tmp_out);
 		cur++;
-	} else if (need_fifoout) {
-		table[cur].my_int = 'o';
-		ep->fifo_out = create_secure_dir_return_filename();
-		table[cur].my_char = g_strdup(ep->fifo_out);
-		DEBUG_MSG("create_commandstring, %%o will be at %s\n",ep->fifo_out);
-		cur++;
 	}
 	if (need_local) {
 		table[cur].my_int = 'f';
@@ -590,6 +509,11 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 		table[cur].my_char = g_strndup(localname, strlen(localname) - strlen(localfilename));
 		cur++;
 	}
+	if (need_arguments) {
+		table[cur].my_int = 'a';
+		table[cur].my_char = g_strdup(user_argument);
+		cur++;
+	}
 	table[cur].my_int = '\0';
 	table[cur].my_char = NULL;
 	if (cur == 0) {
@@ -599,6 +523,7 @@ static gchar *create_commandstring(Texternalp *ep, const gchar *formatstr, gbool
 	}
 	free_convert_table(table);
 	g_free(formatstring);
+	g_free(user_argument);
 	DEBUG_MSG("create_commandstring, returning %s\n",retstring);
 	return retstring;
 }
@@ -661,9 +586,9 @@ static gboolean outputbox_io_watch_lcb(GIOChannel *channel,GIOCondition conditio
 	return TRUE;
 }
 
-static gboolean filter_io_watch_lcb(GIOChannel *channel,GIOCondition condition,gpointer data) {
+static gboolean filter_custom_io_watch_lcb(GIOChannel *channel,GIOCondition condition,gpointer data) {
 	Texternalp *ep = data;
-	DEBUG_MSG("filter_io_watch_lcb, started with condition=%d\n",condition);
+	DEBUG_MSG("filter_custom_io_watch_lcb, started with condition=%d\n",condition);
 	if (condition & G_IO_IN) {
 		gchar *str_return;
 		gsize length;
@@ -672,26 +597,40 @@ static gboolean filter_io_watch_lcb(GIOChannel *channel,GIOCondition condition,g
 		
 		status = g_io_channel_read_to_end(channel,&str_return,&length,&gerror);
 		if (gerror) {
-			g_warning("error while trying to read data for outputbox, %d: %s\n",gerror->code,gerror->message);
+			g_warning("error while trying to read data for filter or custom command, %d: %s\n",gerror->code,gerror->message);
 			g_error_free(gerror);
 		}
 	
 		if (status == G_IO_STATUS_NORMAL && str_return) {
 			gint end=ep->end;
 			GError *error=NULL;
-			DEBUG_MSG("filter_io_watch_lcb, received '%s'\n",str_return);
-			if (ep->bfwin->current_document) {
+			if (ep->customcommand_cb) {
+				ep->customcommand_cb(str_return, ep->bfwin);
+			} else {
+				GtkTextIter iter;
+				GtkTextBuffer *buffer = ep->bfwin->current_document->buffer;
+				DEBUG_MSG("filter_custom_io_watch_lcb, received '%s'\n",str_return);
+				gint line=-1,offset=-1;
 				if (ep->end == -1) {
-					end = gtk_text_buffer_get_char_count(ep->bfwin->current_document->buffer);
+					end = gtk_text_buffer_get_char_count(buffer);
+				}
+				if (!gtk_text_buffer_get_has_selection(buffer)) {
+					gtk_text_buffer_get_iter_at_mark(buffer, &iter,gtk_text_buffer_get_insert(buffer));
+					line = gtk_text_iter_get_line(&iter);
+					offset = gtk_text_iter_get_line_offset(&iter);
 				}
 				doc_replace_text(ep->bfwin->current_document, str_return, ep->begin, end);
+				if (line != -1) {
+					gtk_text_buffer_get_iter_at_line_offset(buffer, &iter, line, offset);
+					gtk_text_buffer_place_cursor(buffer, &iter);
+				}
 			}
 			g_io_channel_shutdown(channel,TRUE,&error);
 			externalp_unref(ep);
 			g_free(str_return);
 			return FALSE;
 		} else {
-			DEBUG_MSG("filter_io_watch_lcb, status=%d\n",status);
+			DEBUG_MSG("filter_custom_io_watch_lcb, status=%d\n",status);
 		}
 		if (str_return) g_free(str_return);
 	}
@@ -720,11 +659,12 @@ static gboolean filter_io_watch_lcb(GIOChannel *channel,GIOCondition condition,g
 typedef enum {
 	mode_filter,
 	mode_outputbox,
-	mode_command
+	mode_command,
+	mode_custom
 } Tcommandmode;
 
 static void command_backend(Tbfwin *bfwin, gint begin, gint end, 
-								const gchar *formatstring, Tcommandmode commandmode) {
+								const gchar *formatstring, Tcommandmode commandmode, CustomCommandCallback customcommand_cb) {
 	Texternalp *ep;
 	
 	DEBUG_MSG("command_backend, started\n");
@@ -743,7 +683,7 @@ static void command_backend(Tbfwin *bfwin, gint begin, gint end,
 	}
 	if (commandmode == mode_filter)	{
 		ep->include_stderr = FALSE;
-		ep->channel_out_lcb = filter_io_watch_lcb;
+		ep->channel_out_lcb = filter_custom_io_watch_lcb;
 		ep->channel_out_data = ep;
 	} else if (commandmode == mode_outputbox)	{
 		ep->include_stderr = TRUE;
@@ -753,20 +693,31 @@ static void command_backend(Tbfwin *bfwin, gint begin, gint end,
 		ep->include_stderr = FALSE;
 		ep->channel_out_lcb = NULL;
 		ep->channel_out_data = NULL;
+	} else if (commandmode == mode_custom) {
+		ep->include_stderr = FALSE;
+		ep->customcommand_cb = customcommand_cb;
+		ep->channel_out_lcb = filter_custom_io_watch_lcb;
+		ep->channel_out_data = ep;
 	}
 
 	start_command(ep);
 }
 
 void outputbox_command(Tbfwin *bfwin, const gchar *formatstring) {
-	command_backend(bfwin, 0, -1,formatstring, mode_outputbox);
+	command_backend(bfwin, 0, -1,formatstring, mode_outputbox, NULL);
 }
 
 
 void filter_command(Tbfwin *bfwin, const gchar *formatstring, gint begin, gint end) {
-	command_backend(bfwin, begin, end,formatstring, mode_filter);
+	command_backend(bfwin, begin, end,formatstring, mode_filter, NULL);
 }
 
 void external_command(Tbfwin *bfwin, const gchar *formatstring) {
-	command_backend(bfwin, 0, -1,formatstring, mode_command);
+	command_backend(bfwin, 0, -1,formatstring, mode_command, NULL);
+}
+
+void
+custom_command(Tbfwin *bfwin, const gchar *formatstring, CustomCommandCallback func)
+{
+	command_backend(bfwin, 0, -1,formatstring, mode_custom, func);
 }
