@@ -425,8 +425,12 @@ autosave_init(gboolean recover, Tbfwin * bfwin)
 				gint pid;
 				found_journal = TRUE;
 				pid = get_int_from_string(tmp);
-#ifndef WIN32					//FIXME
+#ifdef WIN32
+				HANDLE hProc;
+				if (pid > 0 && !(hProc = OpenProcess(PROCESS_QUERY_INFORMATION, FALSE, pid))) {
+#else
 				if (pid > 0 && kill(pid, 0) != 0) {	/* process pid is not alive, or not our process which also means we should recover */
+#endif
 					gchar *path;
 					GError *error = NULL;
 					GFile *file;
@@ -441,6 +445,9 @@ autosave_init(gboolean recover, Tbfwin * bfwin)
 					g_free(path);
 					g_object_unref(file);
 				}
+#ifdef WIN32
+				else
+					CloseHandle(hProc);
 #endif
 			}
 			tmp = g_dir_read_name(gdir);
