@@ -31,24 +31,41 @@
 #include "../gtk_easy.h"
 #include "../stringlist.h"
 
+static void
+table_border_clicked_lcb(GtkWidget * checkbutton, Thtml_diag * dg)
+{
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton))){
+		gtk_widget_set_sensitive(dg->spin[4], FALSE);
+		gtk_widget_set_sensitive(dg->check[2], FALSE);
+	} else {
+		gtk_widget_set_sensitive(dg->spin[4], TRUE);
+		gtk_widget_set_sensitive(dg->check[2], TRUE);
+	}
+}
+
 static void
 tabledialogok_lcb(GtkWidget * widget, Thtml_diag * dg)
 {
 	gchar *thestring, *finalstring;
 
 	thestring = g_strdup(cap("<TABLE"));
-	thestring = insert_string_if_entry(GTK_ENTRY(dg->spin[1]), cap("CELLPADDING"), thestring, NULL);
-	thestring = insert_string_if_entry(GTK_ENTRY(dg->spin[3]), cap("CELLSPACING"), thestring, NULL);
-	thestring = insert_string_if_entry(GTK_ENTRY(dg->spin[4]), cap("BORDER"), thestring, NULL);
+	thestring = insert_integer_if_spin(dg->spin[1], cap("CELLPADDING"), thestring,
+							   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dg->check[4])), 0);							   
+	thestring = insert_integer_if_spin(dg->spin[3], cap("CELLSPACING"), thestring,
+							   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dg->check[5])), 0);	
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dg->check[3]))) {
+		thestring = insert_attr_if_checkbox(dg->check[3],
+			main_v->props.xhtml == 1 ? cap("BORDER=\"border\"") : cap("BORDER"), thestring);
+	} else {	thestring =	insert_integer_if_spin(dg->spin[4], cap("BORDER"), thestring,
+							   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dg->check[2])), 0);
+	}						   
 	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[1]), cap("ALIGN"), thestring, NULL);
-	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[2]), cap("VALIGN"), thestring, NULL);
 	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[3]), cap("BGCOLOR"), thestring, NULL);
-	thestring =
-		insert_integer_if_spin(dg->spin[2], cap("WIDTH"), thestring,
+	thestring =	insert_integer_if_spin(dg->spin[2], cap("WIDTH"), thestring,
 							   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dg->check[1])), 0);
-	thestring =
-		insert_string_if_entry(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(dg->combo[4]))), cap("CLASS"), thestring,
-							   NULL);
+	thestring =	insert_string_if_entry(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(dg->combo[4]))), cap("CLASS"), 
+								thestring, NULL);
+	thestring =	insert_string_if_entry(GTK_ENTRY(dg->entry[3]), cap("ID"), thestring, NULL);
 	thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[2]), cap("STYLE"), thestring, NULL);
 	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[5]), cap("FRAME"), thestring, NULL);
 	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[6]), cap("RULES"), thestring, NULL);
@@ -70,14 +87,14 @@ tabledialogok_lcb(GtkWidget * widget, Thtml_diag * dg)
 void
 tabledialog_dialog(Tbfwin * bfwin, Ttagpopup * data)
 {
-	GList *alignlist = NULL, *popuplist;
+	GList *alignlist = NULL, *popuplist = NULL;
 	GtkWidget *var_but, *dgtable;
 
 	static gchar *tagitems[] =
-		{ "cellpadding", "cellspacing", "border", "align", "valign", "bgcolor", "width", "class", "style",
-"rules", "frame", NULL
+		{ "cellpadding", "cellspacing", "border", "align", "bgcolor", "width", "class", "style",
+"rules", "frame", "id", NULL
 	};
-	gchar *tagvalues[12];
+	gchar *tagvalues[13];
 	gchar *custom = NULL;
 	Thtml_diag *dg;
 
@@ -86,19 +103,31 @@ tabledialog_dialog(Tbfwin * bfwin, Ttagpopup * data)
 
 	dgtable = html_diag_table_in_vbox(dg, 5, 8);
 
-	dg->spin[1] = spinbut_with_value(tagvalues[0], 0, 100, 1.0, 5.0);
-	dialog_mnemonic_label_in_table(_("Cell _Padding:"), dg->spin[1], dgtable, 0, 1, 0, 1);
+	dg->spin[1] = spinbut_with_value(NULL, 0, 100, 1.0, 5.0);
+	dg->check[4] = gtk_check_button_new_with_label("%");
+	dialog_mnemonic_label_in_table(_("<span color=\"#006000\">Cell _Padding:</span>"), 
+				dg->spin[1], dgtable, 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[1], 1, 2, 0, 1);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->check[4], 2, 3, 0, 1);
+	/*g_print("cell padding='%s'\n",tagvalues[0]);*/
+	parse_integer_for_dialog(tagvalues[0], dg->spin[1], NULL, dg->check[4]);
 
-	dg->spin[3] = spinbut_with_value(tagvalues[1], 0, 100, 1.0, 5.0);
-	dialog_mnemonic_label_in_table(_("C_ell Spacing:"), dg->spin[3], dgtable, 0, 1, 1, 2);
+	dg->spin[3] = spinbut_with_value(NULL, 0, 100, 1.0, 5.0);
+	dg->check[5] = gtk_check_button_new_with_label("%");
+	dialog_mnemonic_label_in_table(_("<span color=\"#006000\">C_ell Spacing:</span>"), 
+				dg->spin[3], dgtable, 0, 1, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[3], 1, 2, 1, 2);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->check[5], 2, 3, 1, 2);
+	parse_integer_for_dialog(tagvalues[1], dg->spin[3], NULL, dg->check[5]);
 
-	dg->combo[4] = combobox_with_popdown_sized(tagvalues[7], bfwin->session->classlist, 1, 80);
+	dg->entry[3] = dialog_entry_in_table(tagvalues[10], dgtable, 4, 5, 1, 2);
+	dialog_mnemonic_label_in_table(_("_Id:"), dg->entry[3], dgtable, 3, 4, 1, 2);
+
+	dg->combo[4] = html_diag_combobox_with_popdown_sized(tagvalues[6], bfwin->session->classlist, 1, 90);
 	dialog_mnemonic_label_in_table(_("Cl_ass:"), dg->combo[4], dgtable, 0, 1, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[4], 1, 2, 2, 3);
 
-	dg->entry[2] = dialog_entry_in_table(tagvalues[8], dgtable, 1, 4, 3, 4);
+	dg->entry[2] = dialog_entry_in_table(tagvalues[7], dgtable, 1, 4, 3, 4);
 	dialog_mnemonic_label_in_table(_("St_yle:"), dg->entry[2], dgtable, 0, 1, 3, 4);
 	var_but = style_but_new(dg->entry[2], dg->dialog);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), var_but, 4, 5, 3, 4);
@@ -106,44 +135,39 @@ tabledialog_dialog(Tbfwin * bfwin, Ttagpopup * data)
 	dg->entry[1] = dialog_entry_in_table(custom, dgtable, 1, 5, 4, 5);
 	dialog_mnemonic_label_in_table(_("Custo_m:"), dg->entry[1], dgtable, 0, 1, 4, 5);
 
+	alignlist = g_list_append(NULL, "");
 	alignlist = g_list_append(alignlist, "left");
 	alignlist = g_list_append(alignlist, "right");
 	alignlist = g_list_append(alignlist, "center");
-	dg->combo[1] = combobox_with_popdown_sized(tagvalues[3], alignlist, 0, 80);
+	dg->combo[1] = html_diag_combobox_with_popdown_sized(tagvalues[3], alignlist, 0, 90);
 	g_list_free(alignlist);
-	dialog_mnemonic_label_in_table(_("Ali_gn:"), dg->combo[1], dgtable, 2, 3, 0, 1);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[1], 3, 5, 0, 1);
+	dialog_mnemonic_label_in_table(_("<span color=\"#006000\">_Align:</span>"), dg->combo[1], dgtable, 3, 4, 0, 1);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[1], 4, 5, 0, 1);
 
-	alignlist = NULL;
-	alignlist = g_list_insert(alignlist, "top", 0);
-	alignlist = g_list_insert(alignlist, "middle", 1);
-	alignlist = g_list_insert(alignlist, "bottom", 2);
-	alignlist = g_list_insert(alignlist, "texttop", 3);
-	alignlist = g_list_insert(alignlist, "baseline", 4);
-	alignlist = g_list_insert(alignlist, "absmiddle", 5);
-	alignlist = g_list_insert(alignlist, "absbottom", 6);
-	dg->combo[2] = combobox_with_popdown_sized(tagvalues[4], alignlist, 1, 80);
-	g_list_free(alignlist);
-
-	dialog_mnemonic_label_in_table(_("_Valign:"), dg->combo[2], dgtable, 2, 3, 1, 2);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[2], 3, 5, 1, 2);
-
-	dg->combo[3] = combobox_with_popdown_sized(tagvalues[5], bfwin->session->colorlist, 1, 80);
+	dg->combo[3] = html_diag_combobox_with_popdown_sized(tagvalues[4], bfwin->session->colorlist, 1, 90);
 	var_but = color_but_new(gtk_bin_get_child(GTK_BIN(dg->combo[3])), dg->dialog);
-	dialog_mnemonic_label_in_table(_("Backgrou_nd Color:"), dg->combo[3], dgtable, 2, 3, 2, 3);
+	dialog_mnemonic_label_in_table(_("<span color=\"red\">_bgcolor:</span>"), dg->combo[3], dgtable, 2, 3, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[3], 3, 4, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), var_but, 4, 5, 2, 3);
 
 	dg->spin[2] = spinbut_with_value(NULL, 0, 10000, 1.0, 5.0);
 	dg->check[1] = gtk_check_button_new_with_label("%");
-	dialog_mnemonic_label_in_table(_("_Width:"), dg->spin[2], dgtable, 5, 6, 0, 1);
+	dialog_mnemonic_label_in_table(_("<span color=\"#006000\">_Width:</span>"), dg->spin[2], dgtable, 5, 6, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[2], 6, 7, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->check[1], 7, 8, 0, 1);
-	parse_integer_for_dialog(tagvalues[6], dg->spin[2], NULL, dg->check[1]);
+	parse_integer_for_dialog(tagvalues[5], dg->spin[2], NULL, dg->check[1]);
 
-	dg->spin[4] = spinbut_with_value(tagvalues[2], 0, 100, 1.0, 5.0);
-	dialog_mnemonic_label_in_table(_("Bo_rder:"), dg->spin[4], dgtable, 5, 6, 1, 2);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[4], 6, 8, 1, 2);
+	dg->spin[4] = spinbut_with_value(NULL, 0, 100, 1.0, 5.0);
+	dg->check[2] = gtk_check_button_new_with_label("%");
+	dialog_mnemonic_label_in_table(_("<span color=\"#006000\">Bo_rder:</span>"), dg->spin[4], dgtable, 5, 6, 1, 2);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[4], 6, 7, 1, 2);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->check[2], 7, 8, 1, 2);
+	parse_integer_for_dialog(tagvalues[2], dg->spin[4], NULL, dg->check[2]);
+
+	dg->check[3] = gtk_check_button_new();
+	dialog_mnemonic_label_in_table(_("<span color=\"#A36A00\">_Border:</span>"), dg->check[3], dgtable, 5,6, 2,3);
+	g_signal_connect(dg->check[3], "clicked", G_CALLBACK(table_border_clicked_lcb), dg);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->check[3], 6,7, 2,3);	
 
 	popuplist = g_list_append(NULL, "");
 	popuplist = g_list_append(popuplist, "void");
@@ -155,9 +179,9 @@ tabledialog_dialog(Tbfwin * bfwin, Ttagpopup * data)
 	popuplist = g_list_append(popuplist, "rhs");
 	popuplist = g_list_append(popuplist, "box");
 	popuplist = g_list_append(popuplist, "border");
-	dg->combo[5] = combobox_with_popdown_sized(tagvalues[10], popuplist, 0, 80);
-	dialog_mnemonic_label_in_table(_("_Frame:"), dg->combo[5], dgtable, 5, 6, 2, 3);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[5], 6, 8, 2, 3);
+	dg->combo[5] = html_diag_combobox_with_popdown_sized(tagvalues[9], popuplist, 0, 90);
+	dialog_mnemonic_label_in_table(_("<span color=\"#006000\">_Frame:</span>"), dg->combo[5], dgtable, 5, 6, 3, 4);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[5], 6, 8, 3, 4);
 	g_list_free(popuplist);
 
 	popuplist = g_list_append(NULL, "");
@@ -166,9 +190,9 @@ tabledialog_dialog(Tbfwin * bfwin, Ttagpopup * data)
 	popuplist = g_list_append(popuplist, "rows");
 	popuplist = g_list_append(popuplist, "cols");
 	popuplist = g_list_append(popuplist, "all");
-	dg->combo[6] = combobox_with_popdown_sized(tagvalues[9], popuplist, 0, 80);
-	dialog_mnemonic_label_in_table(_("R_ules:"), dg->combo[6], dgtable, 5, 6, 3, 4);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[6], 6, 8, 3, 4);
+	dg->combo[6] = html_diag_combobox_with_popdown_sized(tagvalues[8], popuplist, 0, 90);
+	dialog_mnemonic_label_in_table(_("<span color=\"#006000\">R_ules:</span>"), dg->combo[6], dgtable, 5, 6, 4, 5);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[6], 6, 8, 4, 5);
 	g_list_free(popuplist);
 
 	html_diag_finish(dg, G_CALLBACK(tabledialogok_lcb));
@@ -183,7 +207,7 @@ tablerowdialogok_lcb(GtkWidget * widget, Thtml_diag * dg)
 	gchar *thestring, *finalstring;
 
 	thestring = g_strdup(cap("<TR"));
-	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[2]), cap("ALIGN"), thestring, NULL);
+	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[1]), cap("ALIGN"), thestring, NULL);
 	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[2]), cap("VALIGN"), thestring, NULL);
 	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[3]), cap("BGCOLOR"), thestring, NULL);
 	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[4]), cap("CLASS"), thestring, NULL);
@@ -220,35 +244,33 @@ tablerowdialog_dialog(Tbfwin * bfwin, Ttagpopup * data)
 
 	dgtable = html_diag_table_in_vbox(dg, 4, 5);
 
+	alignlist = g_list_append(NULL, "");
 	alignlist = g_list_insert(alignlist, "left", 0);
 	alignlist = g_list_insert(alignlist, "right", 1);
 	alignlist = g_list_insert(alignlist, "center", 2);
-	dg->combo[1] = combobox_with_popdown_sized(tagvalues[0], alignlist, 0, 90);
+	dg->combo[1] = html_diag_combobox_with_popdown_sized(tagvalues[0], alignlist, 0, 90);
 	g_list_free(alignlist);
-	dialog_mnemonic_label_in_table(_("Ali_gn:"), dg->combo[1], dgtable, 0, 1, 0, 1);
+	dialog_mnemonic_label_in_table(_("<span color=\"#006000\">_Align:</span>"), dg->combo[1], dgtable, 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(dg->combo[1]), 1, 2, 0, 1);
 
-	alignlist = NULL;
+	alignlist = g_list_append(NULL, "");
 	alignlist = g_list_insert(alignlist, "top", 0);
 	alignlist = g_list_insert(alignlist, "middle", 1);
 	alignlist = g_list_insert(alignlist, "bottom", 2);
-	alignlist = g_list_insert(alignlist, "texttop", 3);
-	alignlist = g_list_insert(alignlist, "baseline", 4);
-	alignlist = g_list_insert(alignlist, "absmiddle", 5);
-	alignlist = g_list_insert(alignlist, "absbottom", 6);
-	dg->combo[2] = combobox_with_popdown_sized(tagvalues[1], alignlist, 0, 90);
+	alignlist = g_list_insert(alignlist, "baseline", 3);
+	dg->combo[2] = html_diag_combobox_with_popdown_sized(tagvalues[1], alignlist, 0, 90);
 	g_list_free(alignlist);
 
-	dialog_mnemonic_label_in_table(_("_Valign:"), dg->combo[2], dgtable, 0, 1, 1, 2);
+	dialog_mnemonic_label_in_table(_("<span color=\"#006000\">_Valign:</span>"), dg->combo[2], dgtable, 0, 1, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_BIN(dg->combo[2])), 1, 2, 1, 2);
 
-	dg->combo[4] = combobox_with_popdown_sized(tagvalues[3], bfwin->session->classlist, 1, 90);
+	dg->combo[4] = html_diag_combobox_with_popdown_sized(tagvalues[3], bfwin->session->classlist, 1, 90);
 	dialog_mnemonic_label_in_table(_("Cl_ass:"), dg->combo[4], dgtable, 2, 3, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->combo[4], 3, 5, 0, 1);
 
-	dg->combo[3] = combobox_with_popdown_sized(tagvalues[2], bfwin->session->colorlist, 1, 90);
+	dg->combo[3] = html_diag_combobox_with_popdown_sized(tagvalues[2], bfwin->session->colorlist, 1, 90);
 	color_but = color_but_new(gtk_bin_get_child(GTK_BIN(dg->combo[3])), dg->dialog);
-	dialog_mnemonic_label_in_table(_("Backgrou_nd Color:"), dg->combo[3], dgtable, 2, 3, 1, 2);
+	dialog_mnemonic_label_in_table(_("<span color=\"red\">_bgcolor:</span>"), dg->combo[3], dgtable, 2, 3, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_BIN(dg->combo[3])), 3, 4, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(color_but), 4, 5, 1, 2);
 
@@ -276,24 +298,22 @@ table_head_and_data_dialogok_lcb(gint type, GtkWidget * widget, Thtml_diag * dg)
 		thestring = g_strdup(cap("<TD"));
 	} else {
 		thestring = g_strdup(cap("<TH"));
+		thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[5]), cap("SCOPE"), thestring, NULL);
 	}
-	thestring =
-		insert_integer_if_spin(dg->spin[1], cap("WIDTH"), thestring,
-							   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dg->check[2])), 0);
-	thestring =
-		insert_integer_if_spin(dg->spin[3], cap("HEIGHT"), thestring,
-							   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dg->check[3])), 0);
+	thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[3]), cap("HEADERS"), thestring, NULL);
 	thestring = insert_integer_if_spin(dg->spin[5], cap("COLSPAN"), thestring, FALSE, 0);
 	thestring = insert_integer_if_spin(dg->spin[4], cap("ROWSPAN"), thestring, FALSE, 0);
+	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[4]), cap("CLASS"), thestring, NULL);
+	thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[2]), cap("STYLE"), thestring, NULL);
+	thestring =	insert_integer_if_spin(dg->spin[1], cap("WIDTH"), thestring,
+							gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dg->check[2])), 0);
+	thestring =	insert_integer_if_spin(dg->spin[3], cap("HEIGHT"), thestring,
+							gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dg->check[3])), 0);
 	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[1]), cap("ALIGN"), thestring, NULL);
 	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[2]), cap("VALIGN"), thestring, NULL);
 	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[3]), cap("BGCOLOR"), thestring, NULL);
-	thestring = insert_string_if_combobox(GTK_COMBO_BOX(dg->combo[4]), cap("CLASS"), thestring, NULL);
-	thestring =
-		insert_attr_if_checkbox(dg->check[1],
-								main_v->props.xhtml == 1 ? cap("NOWRAP=\"nowrap\"") : cap("NOWRAP"),
-								thestring);
-	thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[2]), cap("STYLE"), thestring, NULL);
+	thestring = insert_attr_if_checkbox(dg->check[1],
+							main_v->props.xhtml == 1 ? cap("NOWRAP=\"nowrap\"") : cap("NOWRAP"), thestring);
 	thestring = insert_string_if_entry(GTK_ENTRY(dg->entry[1]), NULL, thestring, NULL);
 	finalstring = g_strconcat(thestring, ">", NULL);
 	g_free(thestring);
@@ -329,10 +349,10 @@ table_head_and_data_dialog_cb(gint type, Tbfwin * bfwin, Ttagpopup * data)
 	GList *alignlist = NULL;
 
 	static gchar *tagitems[] =
-		{ "width", "align", "colspan", "height", "valign", "rowspan", "bgcolor", "nowrap", "class", "style",
-NULL
+		{ "width", "align", "colspan", "height", "valign", "rowspan", "bgcolor", "nowrap", "class", "style", 
+		"headers", "scope", NULL
 	};
-	gchar *tagvalues[11];
+	gchar *tagvalues[12];
 	gchar *custom = NULL;
 	Thtml_diag *dg;
 	GtkWidget *dgtable, *var_but;
@@ -345,72 +365,88 @@ NULL
 
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, dg);
 
-	dgtable = html_diag_table_in_vbox(dg, 5, 7);
+	dgtable = html_diag_table_in_vbox(dg, 6, 7);
 
+	alignlist = g_list_append(NULL, "");
 	alignlist = g_list_insert(alignlist, "left", 0);
 	alignlist = g_list_insert(alignlist, "right", 1);
 	alignlist = g_list_insert(alignlist, "center", 2);
 	alignlist = g_list_insert(alignlist, "justify", 3);
-	dg->combo[1] = combobox_with_popdown_sized(tagvalues[1], alignlist, 1, 80);
+	dg->combo[1] = html_diag_combobox_with_popdown_sized(tagvalues[1], alignlist, 0, 90);
 	g_list_free(alignlist);
-	dialog_mnemonic_label_in_table(_("Ali_gn:"), dg->combo[1], dgtable, 0, 1, 0, 1);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_BIN(dg->combo[1])), 1, 2, 0, 1);
+	dialog_mnemonic_label_in_table(_("<span color=\"#006000\">_Align:</span>"), dg->combo[1], dgtable, 2, 3, 0, 1);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_BIN(dg->combo[1])), 3, 4, 0, 1);
 
-	alignlist = NULL;
+	alignlist = g_list_append(NULL, "");
 	alignlist = g_list_insert(alignlist, "top", 0);
 	alignlist = g_list_insert(alignlist, "middle", 1);
 	alignlist = g_list_insert(alignlist, "bottom", 2);
-	alignlist = g_list_insert(alignlist, "texttop", 3);
-	alignlist = g_list_insert(alignlist, "baseline", 4);
-	alignlist = g_list_insert(alignlist, "absmiddle", 5);
-	alignlist = g_list_insert(alignlist, "absbottom", 6);
-	dg->combo[2] = combobox_with_popdown_sized(tagvalues[4], alignlist, 1, 90);
+	alignlist = g_list_insert(alignlist, "baseline", 3);
+	dg->combo[2] = html_diag_combobox_with_popdown_sized(tagvalues[4], alignlist, 0, 90);
 	g_list_free(alignlist);
 
-	dialog_mnemonic_label_in_table(_("_Valign:"), dg->combo[2], dgtable, 0, 1, 1, 2);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_BIN(dg->combo[2])), 1, 2, 1, 2);
+	dialog_mnemonic_label_in_table(_("<span color=\"#006000\">_Valign:</span>"), dg->combo[2], dgtable, 2, 3, 1, 2);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_BIN(dg->combo[2])), 3, 4, 1, 2);
 
-	dg->combo[4] = combobox_with_popdown_sized(tagvalues[8], bfwin->session->classlist, 1, 80);
+	dg->combo[4] = html_diag_combobox_with_popdown_sized(tagvalues[8], bfwin->session->classlist, 1, 90);
 	dialog_mnemonic_label_in_table(_("Cl_ass:"), dg->combo[4], dgtable, 0, 1, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_BIN(dg->combo[4])), 1, 2, 2, 3);
 
-	dg->entry[2] = dialog_entry_in_table(tagvalues[9], dgtable, 1, 6, 3, 4);
-	dialog_mnemonic_label_in_table(_("St_yle:"), dg->entry[2], dgtable, 0, 1, 3, 4);
-	var_but = style_but_new(dg->entry[2], dg->dialog);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), var_but, 6, 7, 3, 4);
+	dg->entry[3] = dialog_entry_in_table(tagvalues[10], dgtable, 1, 4, 3, 4);
+	dialog_mnemonic_label_in_table(_("<span color=\"#A36A00\">_Headers:</span>"), dg->entry[3], dgtable, 0, 1, 3, 4);
+	gtk_widget_set_tooltip_text(dg->entry[3],_("Set of space-separated IDs of th elements."));
 
-	dg->entry[1] = dialog_entry_in_table(custom, dgtable, 1, 7, 4, 5);
-	dialog_mnemonic_label_in_table(_("Custo_m:"), dg->entry[1], dgtable, 0, 1, 4, 5);
+	if (type == 0) {
+		alignlist = g_list_append(NULL, "");
+		alignlist = g_list_insert(alignlist, "row", 0);
+		alignlist = g_list_insert(alignlist, "col", 1);
+		alignlist = g_list_insert(alignlist, "rowgroup", 2);
+		alignlist = g_list_insert(alignlist, "colgroup", 3);
+		alignlist = g_list_insert(alignlist, "auto", 4);		
+		dg->combo[5] = html_diag_combobox_with_popdown_sized(tagvalues[11], alignlist, 0, 90);
+		g_list_free(alignlist);
+		dialog_mnemonic_label_in_table(_("<span color=\"#A36A00\">_Scope:</span>"), dg->combo[5], dgtable, 4, 5, 3, 4);
+		gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_BIN(dg->combo[5])), 5, 6, 3, 4);
+		gtk_widget_set_tooltip_text(dg->combo[5],_("the header cell applies to cells in the same:"));
+	}
+
+	dg->entry[2] = dialog_entry_in_table(tagvalues[9], dgtable, 1, 6, 4, 5);
+	dialog_mnemonic_label_in_table(_("St_yle:"), dg->entry[2], dgtable, 0, 1, 4, 5);
+	var_but = style_but_new(dg->entry[2], dg->dialog);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), var_but, 6, 7, 4, 5);
+
+	dg->entry[1] = dialog_entry_in_table(custom, dgtable, 1, 7, 5,6);
+	dialog_mnemonic_label_in_table(_("Custo_m:"), dg->entry[1], dgtable, 0, 1, 5, 6);
 
 	dg->spin[5] = spinbut_with_value(tagvalues[2], 0, 100, 1.0, 2.0);
-	dialog_mnemonic_label_in_table(_("Co_l Span:"), dg->spin[5], dgtable, 2, 3, 0, 1);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[5], 3, 4, 0, 1);
+	dialog_mnemonic_label_in_table(_("Co_l Span:"), dg->spin[5], dgtable, 0, 1, 0, 1);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[5], 1, 2, 0, 1);
 
 	dg->spin[4] = spinbut_with_value(tagvalues[5], 0, 100, 1.0, 5.0);
-	dialog_mnemonic_label_in_table(_("_Row Span:"), dg->spin[4], dgtable, 2, 3, 1, 2);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[4], 3, 4, 1, 2);
+	dialog_mnemonic_label_in_table(_("_Row Span:"), dg->spin[4], dgtable, 0, 1, 1, 2);
+	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[4], 1, 2, 1, 2);
 
 	dg->check[1] = gtk_check_button_new();
 	parse_existence_for_dialog(tagvalues[7], dg->check[1]);
-	dialog_mnemonic_label_in_table(_("No Wra_p:"), dg->check[1], dgtable, 2, 3, 2, 3);
+	dialog_mnemonic_label_in_table(_("<span color=\"red\">No _Wrap:</span>"), dg->check[1], dgtable, 2, 3, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->check[1], 3, 4, 2, 3);
 
 	dg->spin[1] = spinbut_with_value(NULL, 0, 10000, 1.0, 5.0);
 	dg->check[2] = gtk_check_button_new_with_label("%");
 	parse_integer_for_dialog(tagvalues[0], dg->spin[1], NULL, dg->check[2]);
-	dialog_mnemonic_label_in_table(_("_Width:"), dg->spin[1], dgtable, 4, 5, 0, 1);
+	dialog_mnemonic_label_in_table(_("<span color=\"red\">_Width:</span>"), dg->spin[1], dgtable, 4, 5, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[1], 5, 6, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->check[2], 6, 7, 0, 1);
 
 	dg->spin[3] = spinbut_with_value(NULL, 0, 10000, 1.0, 5.0);
 	dg->check[3] = gtk_check_button_new_with_label("%");
 	parse_integer_for_dialog(tagvalues[3], dg->spin[3], NULL, dg->check[3]);
-	dialog_mnemonic_label_in_table(_("_Height:"), dg->spin[3], dgtable, 4, 5, 1, 2);
+	dialog_mnemonic_label_in_table(_("<span color=\"red\">_Height:</span>"), dg->spin[3], dgtable, 4, 5, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->spin[3], 5, 6, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), dg->check[3], 6, 7, 1, 2);
 
-	dg->combo[3] = combobox_with_popdown_sized(tagvalues[6], bfwin->session->colorlist, 1, 80);
-	dialog_mnemonic_label_in_table(_("Backgrou_nd Color:"), dg->combo[3], dgtable, 4, 5, 2, 3);
+	dg->combo[3] = html_diag_combobox_with_popdown_sized(tagvalues[6], bfwin->session->colorlist, 1, 90);
+	dialog_mnemonic_label_in_table(_("<span color=\"red\">_bgcolor:</span>"), dg->combo[3], dgtable, 4, 5, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), GTK_WIDGET(GTK_BIN(dg->combo[3])), 5, 6, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable),
 							  GTK_WIDGET(color_but_new(gtk_bin_get_child(GTK_BIN(dg->combo[3])), dg->dialog)),
