@@ -93,6 +93,12 @@ htmlbar_dialog_div(GtkAction * action, gpointer user_data)
 	div_dialog(BFWIN(user_data), NULL);
 }
 
+static void
+htmlbar_dialog_columns(GtkAction * action, gpointer user_data)
+{
+	columns_dialog(BFWIN(user_data), NULL);
+}
+
 static void
 htmlbar_dialog_email(GtkAction * action, gpointer user_data)
 {
@@ -785,492 +791,12 @@ htmlbar_insert_variable_tag(GtkAction * action, gpointer user_data)
 	doc_insert_two_strings(BFWIN(user_data)->current_document, cap("<VAR>"), cap("</VAR>"));
 }
 
-static void
-htmlbar_insert_acute(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_ACUTE);
-}
-
-static void
-htmlbar_insert_cedilla(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_CEDILLA);
-}
-
-static void
-htmlbar_insert_umlaut(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_UMLAUT);
-}
-
-static void
-htmlbar_insert_aacute(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_A_ACUTE_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_A_ACUTE_LOWER);
-}
-
-static void
-htmlbar_insert_acirc(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_A_CIRC_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_A_CIRC_LOWER);
-}
-
-static void
-htmlbar_insert_aelig(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_A_ELIG_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_A_ELIG_LOWER);
-}
-
-static void
-htmlbar_insert_agrave(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_A_GRAVE_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_A_GRAVE_LOWER);
-}
-
-static void
-htmlbar_insert_aring(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_A_RING_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_A_RING_LOWER);
-}
-
-static void
-htmlbar_insert_atilde(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_A_TILDE_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_A_TILDE_LOWER);
-}
-
-static void
-htmlbar_insert_auml(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_A_UML_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_A_UML_LOWER);
-}
-
-static void
-htmlbar_insert_ccedil(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_C_CEDIL_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_C_CEDIL_LOWER);
-}
-
-static void
-htmlbar_insert_eacute(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_E_ACUTE_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_E_ACUTE_LOWER);
-}
-
-static void
-htmlbar_insert_ecirc(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_E_CIRC_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_E_CIRC_LOWER);
-}
-
-static void
-htmlbar_insert_egrave(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_E_GRAVE_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_E_GRAVE_LOWER);
-}
-
-static void
-htmlbar_insert_euml(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_E_UML_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_E_UML_LOWER);
-}
-
-static void
-htmlbar_insert_iacute(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_I_ACUTE_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_I_ACUTE_LOWER);
-}
-
-static void
-htmlbar_insert_icirc(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_I_CIRC_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_I_CIRC_LOWER);
-}
-
-static void
-htmlbar_insert_igrave(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_I_GRAVE_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_I_GRAVE_LOWER);
-}
-
-static void
-htmlbar_insert_iuml(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_I_UML_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_I_UML_LOWER);
-}
-
-static void
-htmlbar_insert_ntilde(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_N_TILDE_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_N_TILDE_LOWER);
-}
-
-static void
-htmlbar_insert_oacute(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_O_ACUTE_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_O_ACUTE_LOWER);
-}
-
-static void
-htmlbar_insert_ocirc(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_O_CIRC_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_O_CIRC_LOWER);
-}
-
-static void
-htmlbar_insert_oelig(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_O_ELIG_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_O_ELIG_LOWER);
-}
-
-static void
-htmlbar_insert_ograve(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_O_GRAVE_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_O_GRAVE_LOWER);
-}
-
-static void
-htmlbar_insert_oslash(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_O_SLASH_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_O_SLASH_LOWER);
-}
-
-static void
-htmlbar_insert_otilde(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_O_TILDE_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_O_TILDE_LOWER);
-}
-
-static void
-htmlbar_insert_ouml(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_O_UML_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_O_UML_LOWER);
-}
-
-static void
-htmlbar_insert_uacute(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_U_ACUTE_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_U_ACUTE_LOWER);
-}
-
-static void
-htmlbar_insert_ucirc(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_U_CIRC_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_U_CIRC_LOWER);
-}
-
-static void
-htmlbar_insert_ugrave(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_U_GRAVE_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_U_GRAVE_LOWER);
-}
-
-static void
-htmlbar_insert_uuml(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_U_UML_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_U_UML_LOWER);
-}
-
-static void
-htmlbar_insert_yacute(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_Y_ACUTE_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_Y_ACUTE_LOWER);
-}
-
-static void
-htmlbar_insert_yuml(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_Y_UML_LOWER);
-}
-
-static void
-htmlbar_insert_cent_sign(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_CENT);
-}
-
-static void
-htmlbar_insert_currency(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_CURRENCY);
-}
-
-static void
-htmlbar_insert_euro(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_EURO);
-}
-
-static void
-htmlbar_insert_pound(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_POUND);
-}
-
-static void
-htmlbar_insert_yen(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_YEN);
-}
-
-static void
-htmlbar_insert_division(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_DIVISION);
-}
-
-static void
-htmlbar_insert_logical_not(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_LOGICAL_NOT);
-}
-
-static void
-htmlbar_insert_times(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_TIMES);
-}
-
-static void
-htmlbar_insert_plus_minus(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_PLUS_MINUS);
-}
-
-static void
-htmlbar_insert_less_than(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_LESS_THAN);
-}
-
-static void
-htmlbar_insert_greater_than(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_GREATER_THAN);
-}
-
-static void
-htmlbar_insert_superscript1(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_SUPERSCRIPT_1);
-}
-
-static void
-htmlbar_insert_superscript2(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_SUPERSCRIPT_2);
-}
-
-static void
-htmlbar_insert_superscript3(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_SUPERSCRIPT_3);
-}
-
-static void
-htmlbar_insert_one_quarter(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_FRAC_QUARTER);
-}
-
-static void
-htmlbar_insert_one_half(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_FRAC_HALF);
-}
-
-static void
-htmlbar_insert_three_quarter(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_FRAC_THREE_QUARTER);
-}
-
-static void
-htmlbar_insert_degree(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_DEGREE);
-}
-
-static void
-htmlbar_insert_eth(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_ETH_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_ETH_LOWER);
-}
-
-static void
-htmlbar_insert_thorn(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_prefix(gtk_action_get_name(action), "Upper"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_THORN_UPPER);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_THORN_LOWER);
-}
-
-static void
-htmlbar_insert_sharp(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_SZLIG);
-}
-
-static void
-htmlbar_insert_micro_sign(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_MIRCO);
-}
-
 static void
 htmlbar_insert_nbsp(GtkAction * action, gpointer user_data)
 {
 	htmlbar_insert_special_char(BFWIN(user_data), BF_NBSP);
 }
 
-static void
-htmlbar_insert_section_sign(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_SECTION);
-}
-
-static void
-htmlbar_insert_copyright(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_COPYRIGHT);
-}
-
-static void
-htmlbar_insert_left_angle(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_LEFT_ANGLE);
-}
-
-static void
-htmlbar_insert_right_angle(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_RIGHT_ANGLE);
-}
-
-static void
-htmlbar_insert_trademark(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_TRADEMARK);
-}
-
-static void
-htmlbar_insert_inverted_exclamation(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_INVERTED_EXCLAMATION);
-}
-
-static void
-htmlbar_insert_inverted_question_mark(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_INVERTED_QUESTION_MARK);
-}
-
-static void
-htmlbar_insert_ordinal(GtkAction * action, gpointer user_data)
-{
-	if (g_str_has_suffix(gtk_action_get_name(action), "F"))
-		htmlbar_insert_special_char(BFWIN(user_data), BF_ORDINAL_F);
-	else
-		htmlbar_insert_special_char(BFWIN(user_data), BF_ORDINAL_M);
-}
-
-static void
-htmlbar_insert_paragraph(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_PARAGRAPH);
-}
-
 static void
 htmlbar_insert_broken_bar(GtkAction * action, gpointer user_data)
 {
@@ -1283,26 +809,24 @@ htmlbar_insert_soft_hyphen(GtkAction * action, gpointer user_data)
 	htmlbar_insert_special_char(BFWIN(user_data), BF_SOFT_HYPHEN);
 }
 
-static void
-htmlbar_insert_spacing_macron(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_SPACING_MACRON);
-}
-
-static void
-htmlbar_insert_middot(GtkAction * action, gpointer user_data)
-{
-	htmlbar_insert_special_char(BFWIN(user_data), BF_MIDDOT);
-}
-
 void
 htmlbar_toolbar_show(Thtmlbarwin * hbw, Thtmlbarsession *hbs, gboolean show)
 {
+	if (htmlbar_v.in_sidepanel)
+		return;
+	
 	if (show) {
-		if (hbw->handlebox)
+		if (hbw->handlebox) {
 			gtk_widget_show(hbw->handlebox);
-		else
-			htmlbar_toolbar_create(hbw, hbs);
+		} else {
+			GtkWidget *html_notebook;
+			html_notebook = htmlbar_toolbar_create(hbw, hbs);
+			hbw->handlebox = gtk_handle_box_new();
+			gtk_container_add(GTK_CONTAINER(hbw->handlebox), html_notebook);
+			gtk_notebook_set_tab_pos(GTK_NOTEBOOK(html_notebook), GTK_POS_TOP);
+			gtk_box_pack_start(GTK_BOX(hbw->bfwin->toolbarbox), hbw->handlebox, FALSE, FALSE, 0);
+			gtk_widget_show_all(hbw->handlebox);
+		}
 	} else {
 		if (hbw->handlebox)
 			gtk_widget_hide(hbw->handlebox);
@@ -1424,6 +948,14 @@ htmlbar_dialog_html5time(GtkAction * action, gpointer user_data)
 	html5time_dialog(BFWIN(user_data), NULL);
 }
 
+static void
+htmlbar_insert_color(GtkAction * action, gpointer user_data)
+{
+	gchar *color = return_color(NULL);
+	doc_insert_two_strings(BFWIN(user_data)->current_document,color, NULL);
+	g_free(color);
+}
+
 static const GtkActionEntry htmlbar_actions[] = {
 	{"TagsMenu", NULL, N_("T_ags")},
 	{"HeadingsMenu", NULL, N_("_Headings")},
@@ -1439,130 +971,10 @@ static const GtkActionEntry htmlbar_actions[] = {
 	 G_CALLBACK(htmlbar_insert_h5_tag)},
 	{"HeadingsH6", BF_STOCK_FONTH6, N_("H_6"), "<control><alt>6", N_("Insert H6 tag"),
 	 G_CALLBACK(htmlbar_insert_h6_tag)},
-	{"SpecialMenu", NULL, N_("_Special")},
-	{"AccentMenu", NULL, N_("_Accent")},
-	{"UppercaseMenu", NULL, N_("_Uppercase")},
-	{"UppercaseAIMenu", NULL, N_("_A-I")},
-	{"UppercaseAgrave", NULL, N_("A grave À"), NULL, NULL, G_CALLBACK(htmlbar_insert_agrave)},
-	{"UppercaseAacute", NULL, N_("A acute Á"), NULL, NULL, G_CALLBACK(htmlbar_insert_aacute)},
-	{"UppercaseAcircumflex", NULL, N_("A circumflex Â"), NULL, NULL, G_CALLBACK(htmlbar_insert_acirc)},
-	{"UppercaseAtilde", NULL, N_("A tilde Ã"), NULL, NULL, G_CALLBACK(htmlbar_insert_atilde)},
-	{"UppercaseAEligature", NULL, N_("AE ligature Æ"), NULL, NULL, G_CALLBACK(htmlbar_insert_aelig)},
-	{"UppercaseAdiaeresis", NULL, N_("A diaeresis Ä"), NULL, NULL, G_CALLBACK(htmlbar_insert_auml)},
-	{"UppercaseAring", NULL, N_("A ring Å"), NULL, NULL, G_CALLBACK(htmlbar_insert_aring)},
-	{"UppercaseCcedilla", NULL, N_("C cedilla Ç"), NULL, NULL, G_CALLBACK(htmlbar_insert_ccedil)},
-	{"UppercaseEgrave", NULL, N_("E grave È"), NULL, NULL, G_CALLBACK(htmlbar_insert_egrave)},
-	{"UppercaseEacute", NULL, N_("E acute É"), NULL, NULL, G_CALLBACK(htmlbar_insert_eacute)},
-	{"UppercaseEcircumflex", NULL, N_("E circumflex Ê"), NULL, NULL, G_CALLBACK(htmlbar_insert_ecirc)},
-	{"UppercaseEdiaeresis", NULL, N_("E diaeresis Ë"), NULL, NULL, G_CALLBACK(htmlbar_insert_euml)},
-	{"UppercaseIgrave", NULL, N_("I grave Ì"), NULL, NULL, G_CALLBACK(htmlbar_insert_igrave)},
-	{"UppercaseIacute", NULL, N_("I acute Í"), NULL, NULL, G_CALLBACK(htmlbar_insert_iacute)},
-	{"UppercaseIcircumflex", NULL, N_("I circumflex Î"), NULL, NULL, G_CALLBACK(htmlbar_insert_icirc)},
-	{"UppercaseIdiaeresis", NULL, N_("I diaeresis Ï"), NULL, NULL, G_CALLBACK(htmlbar_insert_iuml)},
-	{"UppercaseJZMenu", NULL, N_("_J-Z")},
-	{"UppercaseNtilde", NULL, N_("N tilde Ñ"), NULL, NULL, G_CALLBACK(htmlbar_insert_ntilde)},
-	{"UppercaseOgrave", NULL, N_("O grave Ò"), NULL, NULL, G_CALLBACK(htmlbar_insert_ograve)},
-	{"UppercaseOacute", NULL, N_("O acute Ó"), NULL, NULL, G_CALLBACK(htmlbar_insert_oacute)},
-	{"UppercaseOcircumflex", NULL, N_("O circumflex Ô"), NULL, NULL, G_CALLBACK(htmlbar_insert_ocirc)},
-	{"UppercaseOEligature", NULL, N_("OE ligature Œ"), NULL, NULL, G_CALLBACK(htmlbar_insert_oelig)},
-	{"UppercaseOtilde", NULL, N_("O tilde Õ"), NULL, NULL, G_CALLBACK(htmlbar_insert_otilde)},
-	{"UppercaseOdiaeresis", NULL, N_("O diaeresis Ö"), NULL, NULL, G_CALLBACK(htmlbar_insert_ouml)},
-	{"UppercaseOslash", NULL, N_("O slash Ø"), NULL, NULL, G_CALLBACK(htmlbar_insert_oslash)},
-	{"UppercaseUgrave", NULL, N_("U grave Ù"), NULL, NULL, G_CALLBACK(htmlbar_insert_ugrave)},
-	{"UppercaseUacute", NULL, N_("U acute Ú"), NULL, NULL, G_CALLBACK(htmlbar_insert_uacute)},
-	{"UppercaseUcircumflex", NULL, N_("U circumflex Û"), NULL, NULL, G_CALLBACK(htmlbar_insert_ucirc)},
-	{"UppercaseUdiaeresis", NULL, N_("U diaeresis Ü"), NULL, NULL, G_CALLBACK(htmlbar_insert_uuml)},
-	{"UppercaseYacute", NULL, N_("Y acute Ý"), NULL, NULL, G_CALLBACK(htmlbar_insert_yacute)},
-	{"LowercaseMenu", NULL, N_("_Lowercase")},
-	{"LowercaseAIMenu", NULL, N_("_A-I")},
-	{"LowercaseAgrave", NULL, N_("a grave à"), NULL, NULL, G_CALLBACK(htmlbar_insert_agrave)},
-	{"LowercaseAacute", NULL, N_("a acute á"), NULL, NULL, G_CALLBACK(htmlbar_insert_aacute)},
-	{"LowercaseAcircumflex", NULL, N_("a circumflex â"), NULL, NULL, G_CALLBACK(htmlbar_insert_acirc)},
-	{"LowercaseAtilde", NULL, N_("a tilde ã"), NULL, NULL, G_CALLBACK(htmlbar_insert_atilde)},
-	{"LowercaseAring", NULL, N_("a ring å"), NULL, NULL, G_CALLBACK(htmlbar_insert_aring)},
-	{"LowercaseAEligature", NULL, N_("ae ligature æ"), NULL, NULL, G_CALLBACK(htmlbar_insert_aelig)},
-	{"LowercaseAdiaeresis", NULL, N_("a diaeresis ä"), NULL, NULL, G_CALLBACK(htmlbar_insert_auml)},
-	{"LowercaseCcedilla", NULL, N_("c cedilla ç"), NULL, NULL, G_CALLBACK(htmlbar_insert_ccedil)},
-	{"LowercaseEgrave", NULL, N_("e grave è"), NULL, NULL, G_CALLBACK(htmlbar_insert_egrave)},
-	{"LowercaseEacute", NULL, N_("e acute é"), NULL, NULL, G_CALLBACK(htmlbar_insert_eacute)},
-	{"LowercaseEcircumflex", NULL, N_("e circumflex ê"), NULL, NULL, G_CALLBACK(htmlbar_insert_ecirc)},
-	{"LowercaseEdiaeresis", NULL, N_("e diaeresis ë"), NULL, NULL, G_CALLBACK(htmlbar_insert_euml)},
-	{"LowercaseIgrave", NULL, N_("i grave ì"), NULL, NULL, G_CALLBACK(htmlbar_insert_igrave)},
-	{"LowercaseIacute", NULL, N_("i acute í"), NULL, NULL, G_CALLBACK(htmlbar_insert_iacute)},
-	{"LowercaseIcircumflex", NULL, N_("i circumflex î"), NULL, NULL, G_CALLBACK(htmlbar_insert_icirc)},
-	{"LowercaseIdiaeresis", NULL, N_("i diaeresis ï"), NULL, NULL, G_CALLBACK(htmlbar_insert_iuml)},
-	{"LowercaseJZMenu", NULL, N_("_J-Z")},
-	{"LowercaseNtilde", NULL, N_("n tilde ñ"), NULL, NULL, G_CALLBACK(htmlbar_insert_ntilde)},
-	{"LowercaseOgrave", NULL, N_("o grave ò"), NULL, NULL, G_CALLBACK(htmlbar_insert_ograve)},
-	{"LowercaseOacute", NULL, N_("o acute ó"), NULL, NULL, G_CALLBACK(htmlbar_insert_oacute)},
-	{"LowercaseOEligature", NULL, N_("oe ligature œ"), NULL, NULL, G_CALLBACK(htmlbar_insert_oelig)},
-	{"LowercaseOcircumflex", NULL, N_("o circumflex ô"), NULL, NULL, G_CALLBACK(htmlbar_insert_ocirc)},
-	{"LowercaseOtilde", NULL, N_("o tilde õ"), NULL, NULL, G_CALLBACK(htmlbar_insert_otilde)},
-	{"LowercaseOdiaeresis", NULL, N_("o diaeresis ö"), NULL, NULL, G_CALLBACK(htmlbar_insert_ouml)},
-	{"LowercaseOslash", NULL, N_("o slash ø"), NULL, NULL, G_CALLBACK(htmlbar_insert_oslash)},
-	{"LowercaseUgrave", NULL, N_("u grave ù"), NULL, NULL, G_CALLBACK(htmlbar_insert_ugrave)},
-	{"LowercaseUacute", NULL, N_("u acute ú"), NULL, NULL, G_CALLBACK(htmlbar_insert_uacute)},
-	{"LowercaseUcircumflex", NULL, N_("u circumflex û"), NULL, NULL, G_CALLBACK(htmlbar_insert_ucirc)},
-	{"LowercaseUdiaeresis", NULL, N_("u diaeresis ü"), NULL, NULL, G_CALLBACK(htmlbar_insert_uuml)},
-	{"LowercaseYacute", NULL, N_("y acute ý"), NULL, NULL, G_CALLBACK(htmlbar_insert_yacute)},
-	{"LowercaseYdiaeresis", NULL, N_("y diaeresis ÿ"), NULL, NULL, G_CALLBACK(htmlbar_insert_yuml)},
-	{"Acute", NULL, N_("_Acute ´"), NULL, NULL, G_CALLBACK(htmlbar_insert_acute)},
-	{"Cedilla", NULL, N_("_Cedilla ¸"), NULL, NULL, G_CALLBACK(htmlbar_insert_cedilla)},
-	{"Umlaut", NULL, N_("U_mlaut ¨"), NULL, NULL, G_CALLBACK(htmlbar_insert_umlaut)},
-	{"CurrencyMenu", NULL, N_("_Currency")},
-	{"CentSign", NULL, N_("_Cent sign ¢"), NULL, NULL, G_CALLBACK(htmlbar_insert_cent_sign)},
-	{"PoundSterling", NULL, N_("_Pound sterling £"), NULL, NULL, G_CALLBACK(htmlbar_insert_pound)},
-	{"CurrencySign", NULL, N_("C_urrency sign ¤"), NULL, NULL, G_CALLBACK(htmlbar_insert_currency)},
-	{"YenSign", NULL, N_("_Yen sign ¥"), NULL, NULL, G_CALLBACK(htmlbar_insert_yen)},
-	{"Euro", NULL, N_("_Euro €"), NULL, NULL, G_CALLBACK(htmlbar_insert_euro)},
-	{"MathScienceMenu", NULL, N_("_Math-Science")},
-	{"LogicalNot", NULL, N_("Logical _not sign ¬"), NULL, NULL, G_CALLBACK(htmlbar_insert_logical_not)},
-	{"Multiplication", NULL, N_("_Multiplication sign ×"), NULL, NULL, G_CALLBACK(htmlbar_insert_times)},
-	{"Division", NULL, N_("_Division sign ÷"), "<control><alt>slash", NULL,
-	 G_CALLBACK(htmlbar_insert_division)},
-	{"PlusMinus", NULL, N_("_Plus-minus sign ±"), NULL, NULL, G_CALLBACK(htmlbar_insert_plus_minus)},
-	{"LessThan", NULL, N_("_Less-than sign <"), "<control><alt>comma", NULL,
-	 G_CALLBACK(htmlbar_insert_less_than)},
-	{"GreaterThan", NULL, N_("_Greater-than sign >"), "<control><alt>period", NULL,
-	 G_CALLBACK(htmlbar_insert_greater_than)},
-	{"Superscript1", NULL, N_("Superscript _1 ¹"), NULL, NULL, G_CALLBACK(htmlbar_insert_superscript1)},
-	{"Superscript2", NULL, N_("Superscript _2 ²"), NULL, NULL, G_CALLBACK(htmlbar_insert_superscript2)},
-	{"Superscript3", NULL, N_("Superscript _3 ³"), NULL, NULL, G_CALLBACK(htmlbar_insert_superscript3)},
-	{"OneQuarter", NULL, N_("One _quarter ¼"), NULL, NULL, G_CALLBACK(htmlbar_insert_one_quarter)},
-	{"OneHalf", NULL, N_("One _half ½"), NULL, NULL, G_CALLBACK(htmlbar_insert_one_half)},
-	{"ThreeQuarters", NULL, N_("_Three quarters ¾"), NULL, NULL,
-	 G_CALLBACK(htmlbar_insert_three_quarter)},
-	{"DegreeSign", NULL, N_("Deg_ree sign °"), NULL, NULL, G_CALLBACK(htmlbar_insert_degree)},
-	{"NonLatinMenu", NULL, N_("_Non Latin")},
-	{"UppercaseETH", NULL, N_("_ETH (Icelandic) Ð"), NULL, NULL, G_CALLBACK(htmlbar_insert_eth)},
-	{"UppercaseTHORN", NULL, N_("_THORN (Icelandic) Þ"), NULL, NULL, G_CALLBACK(htmlbar_insert_thorn)},
-	{"LowercaseETH", NULL, N_("et_h (Icelandic) ð"), NULL, NULL, G_CALLBACK(htmlbar_insert_eth)},
-	{"LowercaseTHORN", NULL, N_("th_orn þ"), NULL, NULL, G_CALLBACK(htmlbar_insert_thorn)},
-	{"NonLatinSharp", NULL, N_("Sharp _s ß"), NULL, NULL, G_CALLBACK(htmlbar_insert_sharp)},
-	{"NonLatinMicroSign", NULL, N_("_Micro sign µ"), NULL, NULL, G_CALLBACK(htmlbar_insert_micro_sign)},
-	{"SpecialOtherMenu", NULL, N_("_Other")},
+	{"SpecialMenu", NULL, N_("_Entities")},
 	{"NBSP", NULL, N_("_Non-breaking space"), NULL, NULL, G_CALLBACK(htmlbar_insert_nbsp)},
-	{"SectionSign", NULL, N_("_Section sign §"), NULL, NULL, G_CALLBACK(htmlbar_insert_section_sign)},
-	{"Copyright", NULL, N_("_Copyright sign ©"), NULL, NULL, G_CALLBACK(htmlbar_insert_copyright)},
-	{"LeftAngleQuote", NULL, N_("_Left angle quotes «"), NULL, NULL,
-	 G_CALLBACK(htmlbar_insert_left_angle)},
-	{"RightAngleQuote", NULL, N_("_Right angle quotes »"), NULL, NULL,
-	 G_CALLBACK(htmlbar_insert_right_angle)},
-	{"Trademark", NULL, N_("Registered _trademark ®"), NULL, NULL, G_CALLBACK(htmlbar_insert_trademark)},
-	{"InvertedExclamation", NULL, N_("Inverted _exclamation ¡"), NULL, NULL,
-	 G_CALLBACK(htmlbar_insert_inverted_exclamation)},
-	{"InvertedQuestionMark", NULL, N_("Inverted _question mark ¿"), NULL, NULL,
-	 G_CALLBACK(htmlbar_insert_inverted_question_mark)},
-	{"FeminineOrdinal", NULL, N_("_Feminine ordinal ª"), NULL, NULL, G_CALLBACK(htmlbar_insert_ordinal)},
-	{"MasculineOrdinal", NULL, N_("_Masculine ordinal º"), NULL, NULL,
-	 G_CALLBACK(htmlbar_insert_ordinal)},
-	{"Pilcrow", NULL, N_("_Pilcrow (paragraph sign) ¶"), NULL, NULL,
-	 G_CALLBACK(htmlbar_insert_paragraph)},
 	{"BrokenBar", NULL, N_("_Broken bar ¦"), NULL, NULL, G_CALLBACK(htmlbar_insert_broken_bar)},
 	{"SoftHyphen", NULL, N_("Soft _hyphen -"), NULL, NULL, G_CALLBACK(htmlbar_insert_soft_hyphen)},
-	{"SpacingMacron", NULL, N_("Spa_cing macron ¯"), NULL, NULL,
-	 G_CALLBACK(htmlbar_insert_spacing_macron)},
-	{"MidDot", NULL, N_("Middle _dot ·"), NULL, NULL, G_CALLBACK(htmlbar_insert_middot)},
 	{"FormatContextMenu", NULL, N_("_Format by Context")},
 	{"FormatStrong", BF_STOCK_BOLD, N_("_Strong"), "<control><alt>g", N_("Strong"),
 	 G_CALLBACK(htmlbar_insert_strong_tag)},
@@ -1713,7 +1125,7 @@ static const GtkActionEntry htmlbar_actions[] = {
 	{"DialogsTableData", BF_STOCK_TABLE_TD, N_("Table _Data..."), NULL, N_("Table Data..."),
 	 G_CALLBACK(htmlbar_dialog_table_data)},
 	{"DialogsCSSMenu", NULL, N_("_CSS")},
-	{"DialogsCreateStyle", BF_STOCK_CSSNEWSTYLE, N_("_Create Style..."), "<shift><alt>S",
+	{"DialogsCreateStyle", BF_STOCK_CSSNEWSTYLE, N_("_Create Style..."), "<control><shift><alt>S",
 	 N_("Create stylesheet..."),
 	 G_CALLBACK(htmlbar_dialog_style_create)},
 	{"DialogsSpan", BF_STOCK_CSSSPAN, N_("S_pan..."), NULL, N_("Span..."), G_CALLBACK(htmlbar_dialog_span)},
@@ -1721,6 +1133,8 @@ static const GtkActionEntry htmlbar_actions[] = {
 	 G_CALLBACK(htmlbar_dialog_div)},
 	{"MiscStyle", BF_STOCK_CSSSTYLE, N_("_Style"), NULL, N_("Style"),
 	 G_CALLBACK(htmlbar_misc_style)},
+	{"Dialogscolumns", BF_STOCK_CSS_COLUMNS, N_("C_olumns..."), NULL, N_("Columns..."),
+	 G_CALLBACK(htmlbar_dialog_columns)},	 
 	{"DialogsLinkStylesheet", BF_STOCK_LINK_STYLESHEET, N_("_Link to Stylesheet..."), NULL,
 	 N_("Link to Stylesheet..."),
 	 G_CALLBACK(htmlbar_dialog_style_link_to)},
@@ -1781,7 +1195,8 @@ static const GtkActionEntry htmlbar_actions[] = {
 	{"DialogsAudio", BF_STOCK_AUDIO, N_("Au_dio..."), NULL, N_("Audio..."), G_CALLBACK(htmlbar_dialog_audio)},
 	{"DialogsVideo", BF_STOCK_VIDEO, N_("_Video..."), NULL, N_("Video..."), G_CALLBACK(htmlbar_dialog_video)},
 	{"DialogsCanvas", BF_STOCK_CANVAS, N_("_Canvas..."), NULL, N_("Canvas..."), G_CALLBACK(htmlbar_dialog_canvas)},
-	{"DialogsHtml5Time", BF_STOCK_HTML5TIME, N_("_Time..."), NULL, N_("Time..."), G_CALLBACK(htmlbar_dialog_html5time)}
+	{"DialogsHtml5Time", BF_STOCK_HTML5TIME, N_("_Time..."), NULL, N_("Time..."), G_CALLBACK(htmlbar_dialog_html5time)},
+	{"InsertColor", GTK_STOCK_SELECT_COLOR, N_("Insert _Color"), NULL, N_("Insert Color"), G_CALLBACK(htmlbar_insert_color)},
 };
 
 void
@@ -2003,7 +1418,26 @@ notebook_switch_page_lcb(GtkNotebook *notebook,GtkWidget   *page,guint        pa
 	}
 }
 
-void
+static GtkWidget *new_html_subtoolbar(Thtmlbarwin * hbw, GtkWidget *html_notebook, GtkWidget *toolbar, const gchar *labeltext)
+{
+	GtkWidget *label;
+	
+	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
+	if (htmlbar_v.in_sidepanel) {
+		gtk_orientable_set_orientation(GTK_ORIENTABLE(toolbar), GTK_ORIENTATION_VERTICAL);
+	}
+	label = gtk_label_new(labeltext);
+/*	gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);*/
+	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, label);
+#if GTK_CHECK_VERSION(2,20,0)
+	gtk_container_child_set(GTK_CONTAINER(html_notebook), label, "tab-fill", TRUE, "tab-expand", TRUE, NULL);
+#else
+	gtk_notebook_set_tab_label_packing(GTK_NOTEBOOK(html_notebook),label,TRUE,TRUE,GTK_PACK_START);
+#endif
+	return toolbar;	
+}
+
+GtkWidget *
 htmlbar_toolbar_create(Thtmlbarwin * hbw, Thtmlbarsession *hbs)
 {
 	Tbfwin *bfwin = hbw->bfwin;
@@ -2012,70 +1446,46 @@ htmlbar_toolbar_create(Thtmlbarwin * hbw, Thtmlbarsession *hbs)
 	GtkWidget *toolbar;
 
 	html_notebook = gtk_notebook_new();
-	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(html_notebook), GTK_POS_TOP);
+	
 	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(html_notebook), TRUE);
 	gtk_notebook_set_show_border(GTK_NOTEBOOK(html_notebook), TRUE);
-	hbw->handlebox = gtk_handle_box_new();
-	gtk_container_add(GTK_CONTAINER(hbw->handlebox), html_notebook);
-	gtk_box_pack_start(GTK_BOX(bfwin->toolbarbox), hbw->handlebox, FALSE, FALSE, 0);
+	gtk_notebook_set_scrollable(GTK_NOTEBOOK(html_notebook), TRUE);
 
-	hbw->quickbar_toolbar = gtk_toolbar_new();
-	gtk_toolbar_set_style(GTK_TOOLBAR(hbw->quickbar_toolbar), GTK_TOOLBAR_ICONS);
-	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), hbw->quickbar_toolbar, gtk_label_new(_(" Quickbar ")));
+	hbw->quickbar_toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_toolbar_new(), _("Quickbar"));
 	htmlbar_load_quickbar(hbw, hbw->quickbar_toolbar);
 
-	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLStandardToolbar");
-	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
-	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" Standard ")));
+	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLStandardToolbar"), _("Standard"));
 	setup_items_for_quickbar(hbw, toolbar);
 
-	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLFontsToolbar");
-	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
-	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" Fonts ")));
+	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLHTML5Toolbar"), _("HTML 5"));
 	setup_items_for_quickbar(hbw, toolbar);
 
-	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLFormattingToolbar");
-	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
-	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" Formatting ")));
+	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLFormattingToolbar"), _("Formatting"));
 	setup_items_for_quickbar(hbw, toolbar);
 
-	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLTablesToolbar");
-	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
-	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" Tables ")));
-	setup_items_for_quickbar(hbw, toolbar);
-
-	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLFramesToolbar");
-	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
-	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" Frames ")));
-	setup_items_for_quickbar(hbw, toolbar);
-
-	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLFormsToolbar");
-	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
-	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" Forms ")));
-	setup_items_for_quickbar(hbw, toolbar);
-
-	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLListToolbar");
-	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
-	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" List ")));
-	setup_items_for_quickbar(hbw, toolbar);
-
-	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLCSSToolbar");
-	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
-	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" CSS ")));
-	setup_items_for_quickbar(hbw, toolbar);
-
-	toolbar = gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLHTML5Toolbar");
-	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
-	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, gtk_label_new(_(" HTML 5 ")));
+	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLTablesToolbar"), _("Tables"));
 	setup_items_for_quickbar(hbw, toolbar);
 	
-	gtk_widget_show_all(hbw->handlebox);
+	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLListToolbar"), _("List"));
+	setup_items_for_quickbar(hbw, toolbar);
+	
+	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLCSSToolbar"), _("CSS"));
+	setup_items_for_quickbar(hbw, toolbar);
+
+	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLFormsToolbar"), _("Forms"));
+	setup_items_for_quickbar(hbw, toolbar);
+
+	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLFontsToolbar"), _("Fonts"));
+	setup_items_for_quickbar(hbw, toolbar);
+
+	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLFramesToolbar"), _("Frames"));
+	setup_items_for_quickbar(hbw, toolbar);
 	
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(html_notebook), hbs->notebooktab);
 	DEBUG_MSG("htmlbar htmlbar_toolbar_create, make page %d active\n", hbs->notebooktab);
 	g_signal_connect(G_OBJECT(html_notebook), "switch-page", G_CALLBACK(notebook_switch_page_lcb), hbw);
-
 	
+	return html_notebook;
 }
 
 void htmlbar_load_ui(Thtmlbarwin * hbw)
@@ -2113,6 +1523,7 @@ htmlbar_register_stock_icons(void)
 			{pixmap_cssnewstyle, BF_STOCK_CSSNEWSTYLE},
 			{pixmap_cssspan, BF_STOCK_CSSSPAN},
 			{pixmap_cssstyle, BF_STOCK_CSSSTYLE},
+			{pixmap_css_columns, BF_STOCK_CSS_COLUMNS},
 			{pixmap_del, BF_STOCK_DEL},
 			{pixmap_dfn, BF_STOCK_DFN},
 			{pixmap_edit_tag, BF_STOCK_EDIT_TAG},
