@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * image.c - the thumbnail/multi-thumbnail dialogs
  *
- * Copyright (C) 2003-2011 Olivier Sessink
+ * Copyright (C) 2003-2012 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -113,6 +113,9 @@ async_thumbsave_lcb(TcheckNsave_status status, GError * gerror, gpointer callbac
 {
 	DEBUG_MSG("async_thumbsave_lcb, status=%d\n", status);
 	/* TODO: handle error */
+	if (gerror) {
+		g_print("failed to save thumbnail: %s\n",gerror->message);
+	}
 	return CHECKNSAVE_CONT;
 }
 
@@ -149,6 +152,7 @@ image_insert_dialogok_lcb(GtkWidget * widget, Timage_diag * imdg)
 		w = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(imdg->dg->spin[0]));
 		h = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(imdg->dg->spin[1]));
 		tmp_im = gdk_pixbuf_scale_simple(imdg->pb, w, h, GDK_INTERP_BILINEAR);
+		
 		if (strcmp(main_v->props.image_thumbnailtype, "jpeg") == 0) {
 			gdk_pixbuf_save_to_buffer(tmp_im, &buffer, &buflen, main_v->props.image_thumbnailtype, &error,
 									  "quality", "50", NULL);
@@ -233,7 +237,11 @@ image_diag_finish(Timage_diag * imdg, GCallback ok_func)
 {
 	GtkWidget *align, *hbox;
 
+#if GTK_CHECK_VERSION(3,0,0)
+	hbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
+#else
 	hbox = gtk_hbutton_box_new();
+#endif
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
 	gtk_box_set_spacing(GTK_BOX(hbox), 6);
 
@@ -254,20 +262,33 @@ image_diag_finish(Timage_diag * imdg, GCallback ok_func)
 static void
 image_dialog_set_pixbuf(Timage_diag * imdg)
 {
-	gint pb_width, pd_height, toobig;
+	gfloat toobig, pb_width, pd_height;
 	GdkPixbuf *tmp_pb;
 	if (!imdg->pb) {
 		return;
 	}
-	toobig = 1;
-	pb_width = gdk_pixbuf_get_width(imdg->pb);
-	if ((pb_width / 250) > toobig) {
-		toobig = pb_width / 250;
-	}
-	pd_height = gdk_pixbuf_get_height(imdg->pb);
-	if ((pd_height / 300) > toobig) {
-		toobig = pd_height / 300;
+	
+	pb_width = (gfloat)gdk_pixbuf_get_width(imdg->pb);
+	pd_height = (gfloat)gdk_pixbuf_get_height(imdg->pb);
+	if (imdg->dg->bfwin) {
+		Thtmlbarsession *hbs;
+		hbs = g_hash_table_lookup(htmlbar_v.lookup, imdg->dg->bfwin->session);
+		toobig = pb_width / ((gfloat)hbs->thumbnailwidth);
+		DEBUG_MSG("initialize toobig as %f, using session thumbnailwidth %d and pixbuf width %f\n",toobig,hbs->thumbnailwidth,pb_width);
+	} else {
+		toobig = 1.0;
+		if ((pb_width / 250.0) > toobig) {
+			toobig = pb_width / 250.0;
+		}
+		
+		if ((pd_height / 300.0) > toobig) {
+			toobig = pd_height / 300.0;
+		}
 	}
+	/* because the spin buttons in the html dialogs are designed that they can have an empty string value ""
+	they will not accept a number as long as there is an empty string in them */
+	gtk_entry_set_text(GTK_ENTRY(imdg->dg->spin[0]), "1");
+	gtk_entry_set_text(GTK_ENTRY(imdg->dg->spin[1]), "1");
 
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(imdg->dg->spin[0]), (pb_width / toobig));
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(imdg->dg->spin[1]), (pd_height / toobig));
@@ -276,7 +297,7 @@ image_dialog_set_pixbuf(Timage_diag * imdg)
 	g_signal_handler_unblock(G_OBJECT(imdg->adjustment), imdg->adj_changed_id);
 
 	tmp_pb =
-		gdk_pixbuf_scale_simple(imdg->pb, (pb_width / toobig), (pd_height / toobig),
+		gdk_pixbuf_scale_simple(imdg->pb, (gint)(pb_width / toobig), (gint)(pd_height / toobig),
 								main_v->globses.image_thumbnail_refresh_quality ? GDK_INTERP_BILINEAR :
 								GDK_INTERP_NEAREST);
 
@@ -418,7 +439,7 @@ image_adjust_changed(GtkAdjustment * adj, Timage_diag * imdg)
 
 	tn_width = gtk_adjustment_get_value(imdg->adjustment) * gdk_pixbuf_get_width(imdg->pb);
 	tn_height = gtk_adjustment_get_value(imdg->adjustment) * gdk_pixbuf_get_height(imdg->pb);
-
+	DEBUG_MSG("image_adjust_changed, width=%d, height=%d\n",tn_width,tn_height);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(imdg->dg->spin[0]), tn_width);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(imdg->dg->spin[1]), tn_height);
 
@@ -447,13 +468,17 @@ image_insert_dialog_backend(gchar * filename, Tbfwin * bfwin, Ttagpopup * data)
 	static gchar *tagitems[] =
 		{ "width", "height", "alt", "border", "src", "hspace", "vspace", "align", "name", "usemap", NULL };
 	gchar *tagvalues[11];
-	gchar *custom = NULL;
+	gchar *custom = NULL, *tmp;
 	Timage_diag *imdg;
 	GList *popuplist = NULL;
 	GtkWidget *dgtable, *scale;
 
 	imdg = g_new0(Timage_diag, 1);
 
+	tmp = main_v->props.image_thumbnailtype; /* it is often uppercase, we need it lowercase */
+	main_v->props.image_thumbnailtype = g_ascii_strdown(tmp, -1);
+	g_free(tmp);
+
 	imdg->dg = html_diag_new(bfwin, _("Insert thumbnail"));
 
 	fill_dialogvalues(tagitems, tagvalues, &custom, (Ttagpopup *) data, imdg->dg);
@@ -464,7 +489,11 @@ image_insert_dialog_backend(gchar * filename, Tbfwin * bfwin, Ttagpopup * data)
 	gtk_box_pack_start(GTK_BOX(imdg->dg->vbox), imdg->frame, TRUE, TRUE, 0);
 
 	imdg->adjustment = GTK_ADJUSTMENT(gtk_adjustment_new(0.5, 0.0001, 1.1, 0.001, 0.1, 0.1));
+#if GTK_CHECK_VERSION(3,0,0)
+	scale = gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, imdg->adjustment);
+#else
 	scale = gtk_hscale_new(imdg->adjustment);
+#endif
 	imdg->adj_changed_id =
 		g_signal_connect(G_OBJECT(imdg->adjustment), "value_changed", G_CALLBACK(image_adjust_changed), imdg);
 	gtk_scale_set_digits(GTK_SCALE(scale), 3);
@@ -521,7 +550,7 @@ image_insert_dialog_backend(gchar * filename, Tbfwin * bfwin, Ttagpopup * data)
 	popuplist = g_list_append(popuplist, "top");
 	popuplist = g_list_append(popuplist, "left");
 	popuplist = g_list_append(popuplist, "right");
-	imdg->dg->combo[0] = combobox_with_popdown_sized(tagvalues[7], popuplist, 1, 90);
+	imdg->dg->combo[0] = html_diag_combobox_with_popdown_sized(tagvalues[7], popuplist, 1, 90);
 	g_list_free(popuplist);
 	dialog_mnemonic_label_in_table(_("_Align:"), imdg->dg->combo[0], dgtable, 3, 4, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->combo[0], 4, 6, 1, 2);
@@ -1007,11 +1036,16 @@ multi_thumbnail_dialog(Tbfwin * bfwin)
 	Tmuthudia *mtd;
 	GtkWidget *vbox, *hbox, *but, *table, *label, *scrolwin, *textview;
 	gint tb;
+	gchar *tmp;
 
 	if (!bfwin->current_document) {
 		return;
 	}
 
+	tmp = main_v->props.image_thumbnailtype; /* it is often uppercase, we need it lowercase */
+	main_v->props.image_thumbnailtype = g_ascii_strdown(tmp, -1);
+	g_free(tmp);
+
 	mtd = g_new0(Tmuthudia, 1);
 	mtd->bfwin = bfwin;
 	mtd->document = bfwin->current_document;
@@ -1065,8 +1099,11 @@ multi_thumbnail_dialog(Tbfwin * bfwin)
 									GTK_WRAP_CHAR);
 	mtd->tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
 	gtk_box_pack_start(GTK_BOX(vbox), scrolwin, TRUE, TRUE, 0);
-
+#if GTK_CHECK_VERSION(3,0,0)
+	hbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
+#else
 	hbox = gtk_hbutton_box_new();
+#endif
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
 	gtk_box_set_spacing(GTK_BOX(hbox), 1);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
