@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * quickstart.c - quickstart dialog
  *
- * Copyright (C) 2005-2011 James Hayward and Olivier Sessink
+ * Copyright (C) 2005-2012 James Hayward and Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 
 #include "htmlbar.h"
 #include "cap.h"
+#include "html_diag.h"
 #include "../config.h"
 #include "../bfwin.h"
 #include "../dialog_utils.h"
@@ -474,7 +475,11 @@ quickstart_meta_page_create(TQuickStart * qstart)
 	gtk_tree_view_set_model(GTK_TREE_VIEW(qstart->metaView), GTK_TREE_MODEL(metaStore));
 	g_object_unref(metaStore);
 
+#if GTK_CHECK_VERSION(3,0,0)
+	bbox = gtk_button_box_new(GTK_ORIENTATION_VERTICAL);
+#else
 	bbox = gtk_vbutton_box_new();
+#endif
 	gtk_box_set_spacing(GTK_BOX(bbox), 6);
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_START);
 	button = gtk_button_new_from_stock(GTK_STOCK_ADD);
@@ -538,7 +543,7 @@ quickstart_style_page_create(TQuickStart * qstart)
 	}
 	qstart->stylehref = gtk_combo_box_entry_new_with_model(GTK_TREE_MODEL(history), 0);
 	g_object_unref(history);*/
-	qstart->stylehref = combobox_with_popdown(NULL, qstart->bfwin->session->urllist, TRUE);
+	qstart->stylehref = html_diag_combobox_with_popdown("", qstart->bfwin->session->urllist, TRUE);
 	dialog_mnemonic_label_in_table(_("HRE_F:"), qstart->stylehref, table, 0, 1, 0, 1);
 	gtk_table_attach(GTK_TABLE(table), qstart->stylehref, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0,
 					 0);
@@ -593,7 +598,7 @@ quickstart_script_page_create(TQuickStart * qstart)
 	}
 	qstart->scriptsrc = gtk_combo_box_entry_new_with_model(GTK_TREE_MODEL(history), 0);
 	g_object_unref(history);*/
-	qstart->scriptsrc = combobox_with_popdown(NULL,qstart->bfwin->session->urllist , TRUE);
+	qstart->scriptsrc = html_diag_combobox_with_popdown("",qstart->bfwin->session->urllist , TRUE);
 	label = gtk_label_new_with_mnemonic(_("_Src:"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label), qstart->scriptsrc);
