@@ -22,6 +22,7 @@
 #define _GNU_SOURCE
 
 #include <string.h>
+#include <gdk/gdkkeysyms.h>
 
 #include "snippets.h"
 #include "snippets_gui.h"
@@ -181,7 +182,7 @@ snippets_connect_accelerators_from_doc(Tsnippetswin * snw, xmlNodePtr cur, GtkAc
 				guint key;
 				GdkModifierType mod;
 				gtk_accelerator_parse((const gchar *) accelerator, &key, &mod);
-				if (key != 0 && mod != 0 && gtk_accelerator_valid(key, mod)) {
+				if (key != 0 && (mod != 0 || (key >= GDK_KEY_F1 && key <= GDK_KEY_F12)) && gtk_accelerator_valid(key, mod)) {
 					GClosure *closure;
 					Taccelerator_cbdata *hcbdata;
 					hcbdata = g_slice_new(Taccelerator_cbdata);
@@ -327,6 +328,7 @@ snippetview_button_press_lcb(GtkWidget * widget, GdkEventButton * event, Tsnippe
 				if (event->button == 1 && event->type == GDK_2BUTTON_PRESS) {	/* left mouse button double-clicked */
 					if (xmlStrEqual(cur->name, (const xmlChar *) "leaf")) {
 						snippet_activate_leaf(snw, cur);
+						return TRUE;
 					}
 				} else if (event->button == 3) {	/* right mouse button clicked */
 					popup_menu_create(snw, cur, event);
