@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * rcfile.c - loading and parsing of the configfiles
  *
- * Copyright (C) 2000-2011 Olivier Sessink
+ * Copyright (C) 2000-2012 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -404,6 +404,7 @@ parse_config_file(GHashTable * config_list, GFile * file)
 static GHashTable *
 props_init_main(GHashTable * config_rc)
 {
+	init_prop_string(&config_rc, &main_v->props.config_version, "config_version:", "2.2.2");
 	init_prop_integer(&config_rc, &main_v->props.do_periodic_check, "do_periodic_check:", 1, TRUE);
 	init_prop_string(&config_rc, &main_v->props.editor_font_string, "editor_font_string:", "monospace 10");
 	init_prop_integer(&config_rc, &main_v->props.editor_smart_cursor, "editor_smart_cursor:", 1, TRUE);
@@ -416,19 +417,21 @@ props_init_main(GHashTable * config_rc)
 	init_prop_string(&config_rc, &main_v->props.tab_color_error, "tab_color_error:", "#FF0000");
 	init_prop_integer(&config_rc, &main_v->props.visible_ws_mode, "visible_ws_mode:", 3, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.right_margin_pos, "right_margin_pos:", 80, TRUE);
-	init_prop_arraylist(&config_rc, &main_v->props.external_command, "external_command:", 3, TRUE);
-	init_prop_arraylist(&config_rc, &main_v->props.external_filter, "external_filter:", 2, TRUE);
-	init_prop_arraylist(&config_rc, &main_v->props.external_outputbox, "external_outputbox:", 6, TRUE);
+	init_prop_arraylist(&config_rc, &main_v->props.external_command, "external_command:", 0, TRUE);
+	init_prop_arraylist(&config_rc, &main_v->props.external_filter, "external_filter:", 0, TRUE);
+	init_prop_arraylist(&config_rc, &main_v->props.external_outputbox, "external_outputbox:", 0, TRUE);
 
 	/*init_prop_integer   (&config_rc, &main_v->props.defaulthighlight, "defaulthighlight:", 1, TRUE); */
 	/* old type filetypes have a different count, they are converted below */
-	init_prop_integer(&config_rc, &main_v->props.transient_htdialogs, "transient_htdialogs:", 1, TRUE);
 
 	init_prop_integer(&config_rc, &main_v->props.leave_to_window_manager, "leave_to_window_manager:", 0,
 					  TRUE);
 	init_prop_integer(&config_rc, &main_v->props.restore_dimensions, "restore_dimensions:", 1, TRUE);
 
 	init_prop_integer(&config_rc, &main_v->props.left_panel_left, "left_panel_left:", 1, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.hide_bars_on_fullscreen, "hide_bars_on_fullscreen:", 1, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.cursor_size, "cursor_size:", 4, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.highlight_cursor, "highlight_cursor:", 0, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.save_accelmap, "save_accelmap:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.max_recent_files, "max_recent_files:", 15, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.max_dir_history, "max_dir_history:", 10, TRUE);
@@ -454,7 +457,7 @@ props_init_main(GHashTable * config_rc)
 					  TRUE);
 	init_prop_integer(&config_rc, &main_v->props.encoding_search_Nbytes, "encoding_search_Nbytes:", 500,
 					  TRUE);
-	init_prop_integer(&config_rc, &main_v->props.encoding_search_Nbytes, "max_window_title:", 0,TRUE);
+	init_prop_integer(&config_rc, &main_v->props.max_window_title, "max_window_title:", 0,TRUE);
 	init_prop_integer(&config_rc, &main_v->props.document_tabposition, "document_tabposition:",
 					  (gint) GTK_POS_BOTTOM, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.leftpanel_tabposition, "leftpanel_tabposition:",
@@ -463,21 +466,12 @@ props_init_main(GHashTable * config_rc)
 /*	init_prop_string    (&config_rc, &main_v->props.default_basedir,"default_basedir:",g_get_home_dir());*/
 	init_prop_string(&config_rc, &main_v->props.project_suffix, "project_suffix:", ".bfproject");
 	/* not yet in use */
-	init_prop_string(&config_rc, &main_v->props.image_editor_cline, "image_editor_command:",
-					 "gimp-remote -n \"%s\"&");
 	init_prop_integer(&config_rc, &main_v->props.allow_dep, "allow_the_use_of_font:", 0, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.format_by_context, "format_by_context:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.xhtml, "use_xhtml:", 1, TRUE);
 /*	init_prop_integer(&config_rc, &main_v->props.insert_close_tag, "insert_close_tag:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.close_tag_newline, "close_tag_newline:", 1, TRUE);*/
-	init_prop_integer(&config_rc, &main_v->props.allow_ruby, "allow_the_use_of_ruby:", 0, TRUE);
-	init_prop_integer(&config_rc, &main_v->props.force_dtd, "force_an_dtd:", 0, TRUE);
-	init_prop_integer(&config_rc, &main_v->props.dtd_url, "url_in_dtd:", 0, TRUE);
-	init_prop_integer(&config_rc, &main_v->props.xml_start, "xml_starting_line:", 0, TRUE);
-	init_prop_integer(&config_rc, &main_v->props.lowercase_tags, "lowercase_tags:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.smartindent, "smartindent:", 1, TRUE);
-	init_prop_integer(&config_rc, &main_v->props.drop_at_drop_pos, "drop_at_drop_position:", 0, TRUE);
-	init_prop_integer(&config_rc, &main_v->props.link_management, "link_management:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.open_in_running_bluefish, "open_in_running_bluefish:", 1,
 					  TRUE);
 	init_prop_integer(&config_rc, &main_v->props.open_in_new_window, "open_in_new_window:", 1, TRUE);
@@ -493,6 +487,8 @@ props_init_main(GHashTable * config_rc)
 	init_prop_string(&config_rc, &main_v->props.btv_color_str[BTV_COLOR_RIGHT_MARGIN], "right_margin:",
 					 "#990000");
 	init_prop_string(&config_rc, &main_v->props.btv_color_str[BTV_COLOR_CURSOR], "cursor_color:", "");
+	init_prop_string(&config_rc, &main_v->props.btv_color_str[BTV_COLOR_SELECTION], "selection_color:", "");
+	init_prop_string(&config_rc, &main_v->props.btv_color_str[BTV_COLOR_CURSOR_HIGHLIGHT], "cursor_highlight_color:", "#ffff33");
 	init_prop_arraylist(&config_rc, &main_v->props.textstyles, "textstyles3:", 6, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.block_folding_mode, "block_folding_mode:", 1, TRUE);
 	init_prop_arraylist(&config_rc, &main_v->props.highlight_styles, "highlight_styles:", 3, TRUE);
@@ -584,6 +580,262 @@ migrate_config_files(GHashTable * main_configlist, GFile * newrc)
 	}
 }
 
+
+static gchar *
+convert_old_placeholders(gchar *orig, gchar old1, gchar new1, gchar old2, gchar new2)
+{
+	gint i;
+	gboolean prev_was_percent=FALSE;
+	if (!orig)
+		return NULL;
+
+	for (i=0;orig[i]!='\0';i++) {
+		if (prev_was_percent && orig[i] == old1) {
+			orig[i] = new1;
+		} else if (prev_was_percent && orig[i] == old2) {
+			orig[i] = new2;
+		}
+		prev_was_percent=FALSE;
+		if (orig[i]=='%') {
+			prev_was_percent=TRUE;
+		}
+	}
+	return orig;
+}
+
+/* on array length = newlen-1 we update to the new version
+
+if (overwrite) we free the list and replace it with the default values 
+
+if (!overwrite) we overwrite all bluefish_defined options, and we append any new options
+*/
+static GList *update_externals(GList *current, GList *defaults, gboolean overwrite, gint newlen, gint commandindex)
+{
+	GList *tmplist;
+	GHashTable *ht=NULL;
+	
+	if (current == NULL)
+		overwrite = TRUE;
+	
+	if (overwrite) {
+		free_arraylist(current);
+		current = NULL;
+	} else {
+		ht = g_hash_table_new(g_str_hash, g_str_equal);
+		tmplist = g_list_first(current);
+		while (tmplist) {
+			gchar **arr = tmplist->data;
+			gint len;
+			GList *cur;
+			len = g_strv_length(arr);
+	
+			cur = tmplist;
+			tmplist = g_list_next(tmplist);
+			
+			if (len == (newlen-1)) { /* convert the old (2.2.2 or older) format into the new format */
+				gchar **oldarr = arr;
+				/*g_print("prepend %s in front of %s\n",USER_DEFINED_ENABLED,arr[0]);*/
+				cur->data = arr = prepend_array(USER_DEFINED_ENABLED,arr);
+				g_strfreev(oldarr);
+				/* arr[2] contains the commandstring. now replace %I and %O (previously fifo in and 
+				fifo out) to temporary file in %i and out %o */
+				arr[commandindex] = convert_old_placeholders(arr[commandindex], 'O', 'o', 'I', 'i');
+				len = newlen;
+			}
+			if (len == newlen && arr[0][0]!='0' && arr[0][0]!='1') {
+				/* keep all the user defined options */
+				/*g_print("update_externals, insert into hash: %s\n",arr[1]);*/
+				g_hash_table_insert(ht, arr[1], arr);
+			} else {
+				/* remove all others */
+				current = g_list_delete_link(current, cur);
+				DEBUG_MSG("removed list item %p from list, current =%p\n",cur, current);
+				g_strfreev(arr);
+			}
+		}
+	}
+	for (tmplist=g_list_first(defaults);tmplist;tmplist = g_list_next(tmplist)) {
+		gchar **arr=tmplist->data;
+		if (overwrite || g_hash_table_lookup(ht, arr[0])==NULL) {
+			/*g_print("update_externals, prepend a missing or removed-default value %s\n",arr[0]);*/
+			current = g_list_prepend(current, prepend_array(BLUEFISH_DEFINED_ENABLED, arr));
+		}
+	}
+	if (ht)
+		g_hash_table_destroy(ht);
+	return current;
+}
+
+GList * update_outputbox(GList *current, gboolean overwrite) {
+	GList *defaults = NULL, *retlist;
+	DEBUG_MSG("update_outputbox, started\n");
+#ifdef WIN32
+	defaults =
+			g_list_append(defaults,
+						  array_from_arglist(_("php"),
+											 "(.*) in (.*:\\\\[a-zA-Z0-9\\\\_.-]+) on line ([0-9]+)", "2",
+											 "3", "1", "php '%f'|", NULL));
+	}
+#else
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("make"), "([a-zA-Z0-9/_.-]+):([0-9]+):(.*)", "1", "2", "3",
+										 "cd %c && make|", NULL));
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("weblint HTML checker"),
+										 "([a-zA-Z0-9/_.-]+) \\(([0-9:]+)\\) (.*)", "1", "2", "3",
+										 "weblint '%f'|", NULL));
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("tidy HTML validator"), "line ([0-9]+) column [0-9]+ - (.*)",
+										 "-1", "1", "2", "tidy -qe '%i'|", NULL));
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("javac"), "([a-zA-Z0-9/_.-]+):([0-9]+):(.*)", "1", "2", "3",
+										 "javac '%f'|", NULL));
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("xmllint XML checker"),
+										 "([a-zA-Z0-9/_.-]+)\\:([0-9]+)\\: (.*)", "1", "2", "3",
+										 "xmllint --noout --valid '%f'|", NULL));
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("php"), "(.*) in (/[a-zA-Z0-9/_.-]+) on line ([0-9]+)", "2",
+										 "3", "1", "php '%f'|", NULL));
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("php syntax check"), "(.*) in (/[a-zA-Z0-9/_.-]+) on line ([0-9]+)", "2",
+										 "3", "1", "php -l -q -f '%f'|", NULL));
+	defaults =
+		g_list_append(defaults,
+						array_from_arglist(_("perl syntax check"), "(.*) at (/[a-zA-Z0-9/_.-]+) line ([0-9]+)", "2",
+										 "3", "1", "perl -c '%f'|", NULL));
+	defaults =
+		g_list_append(defaults,
+						array_from_arglist(_("php codesniffer"), ":([0-9:]+):(.*)", "-1",
+										 "1", "2", "phpcs --report=emacs '%f'|", NULL));
+#endif
+	retlist = update_externals(current, defaults, overwrite, 7, 6);
+	free_arraylist(defaults);
+	return retlist;
+}
+
+GList *update_filters(GList *current, gboolean overwrite)
+{
+	GList *defaults=NULL, *retlist;
+
+	defaults =
+		g_list_append(defaults, array_from_arglist(_("Sort"), "|sort|", NULL));
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("Sort / Uniq"), "|sort|uniq|", NULL));
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("Reverse lines"), "|tac|", NULL));
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("Dos2unix"), "|dos2unix|", NULL));
+	defaults =
+		g_list_append(defaults, array_from_arglist(_("Tidy HTML"), "|tidy|", NULL));
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("Tidy HTML (perltidy)"), "|perltidy -b|", NULL));
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("Strip empty lines"), "|egrep -v '^[ \t]*$'|", NULL));
+	defaults =
+		g_list_append(defaults,
+					  array_from_arglist(_("Render HTML to text"), "lynx -force_html -dump %i |", NULL));
+
+	retlist = update_externals(current, defaults, overwrite, 3, 2);
+	free_arraylist(defaults);
+	return retlist;
+}
+
+GList *update_commands(GList *current, gboolean overwrite)
+{
+	GList *defaults=NULL, *retlist;
+#ifdef WIN32	
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Windows XP Firefox"),
+										 "\"C:\\Program Files\\Mozilla Firefox\\firefox.exe\" '%p'",
+										 "1", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Windows XP Internet Explorer"),
+										 "\"C:\\Program Files\\Internet Explorer\\iexplore.exe\" '%p'",
+										 "0", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Windows XP Opera"),
+										 "\"C:\\Program Files\\Opera\\opera.exe\" '%p'", "0", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Windows XP Safari"),
+										 "\"C:\\Program Files\\Safari\\safari.exe\" '%p'", "0", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Windows 7 Firefox 32-bit"),
+										 "\"C:\\Program Files (x86)\\Mozilla Firefox\\firefox.exe\" '%p'",
+										 "1", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Windows 7 Internet Explorer 32-bit"),
+										 "\"C:\\Program Files (x86)\\Internet Explorer\\iexplore.exe\" '%p'",
+										 "0", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Windows 7 Internet Explorer 64-bit"),
+										 "\"C:\\Program Files\\Internet Explorer\\iexplore.exe\" '%p'",
+										 "0", NULL));
+#elif PLATFORM_DARWIN
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Safari"), "open -a Safari '%p'", "0", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Firefox"), "open -a Firefox '%p'", "1", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Opera"), "open -a Opera '%p'", "0", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("chmod a+x"), "chmod a+x %f", "0", NULL));
+#else
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Firefox"), "firefox -remote 'openURL(%p)' || firefox '%p'&",
+										 "1", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Opera"), "opera -remote 'openURL(%p)' || opera '%p'&", "0",
+										 NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Chromium"), "chromium-browser '%p'&", "0", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Konqueror"), "konqueror '%p'&", "0", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Epiphany"), "epiphany-browser -n '%p'&", "0", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Kazehakase"), "kazehakase '%p'&", "0", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Links2 (graphics)"), "links2 -g '%p'&", "0", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("chmod a+x"), "chmod a+x %f", "0", NULL));
+#endif
+	retlist = update_externals(current, defaults, overwrite, 4, 2);
+	free_arraylist(defaults);
+	return retlist;
+}
+
 void
 rcfile_parse_main(void)
 {
@@ -608,220 +860,14 @@ rcfile_parse_main(void)
 	g_object_unref(file);
 	if (main_v->props.encoding_search_Nbytes < 1000)
 		main_v->props.encoding_search_Nbytes = 2048;
+
 	/* do some default configuration for the lists */
-#ifdef WIN32
-	if (main_v->props.external_outputbox == NULL) {
-		/* if the user does not have outputbox settings --> set them to defaults values */
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-						  array_from_arglist(_("php"),
-											 "(.*) in (.*:\\\\[a-zA-Z0-9\\\\_.-]+) on line ([0-9]+)", "2",
-											 "3", "1", "php '%f'|", NULL));
+	if (!main_v->props.config_version || strlen(main_v->props.config_version)<5 ||main_v->props.config_version[0] < '2' || main_v->props.config_version[2] < '2' || main_v->props.config_version[4] < '3') {
+		main_v->props.external_command = update_commands(main_v->props.external_command, FALSE);
+		main_v->props.external_filter = update_filters(main_v->props.external_filter, FALSE);
+		main_v->props.external_outputbox = update_outputbox(main_v->props.external_outputbox, FALSE);
 	}
-	if (main_v->props.external_filter == NULL) {
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter, array_from_arglist(_("Sort"), "|sort|", NULL));
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter,
-						  array_from_arglist(_("Sort / Uniq"), "|sort|uniq|", NULL));
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter,
-						  array_from_arglist(_("Reverse lines"), "|tac|", NULL));
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter,
-						  array_from_arglist(_("Dos2unix"), "|dos2unix|", NULL));
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter, array_from_arglist(_("Tidy HTML"), "|tidy|", NULL));
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter,
-						  array_from_arglist(_("Strip empty lines"), "|egrep -v '^[ \t]*$'|", NULL));
-	}
-	if (main_v->props.external_command == NULL) {
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Windows XP Firefox"),
-											 "\"C:\\Program Files\\Mozilla Firefox\\firefox.exe\" '%p'",
-											 "1", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Windows XP Internet Explorer"),
-											 "\"C:\\Program Files\\Internet Explorer\\iexplore.exe\" '%p'",
-											 "0", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Windows XP Opera"),
-											 "\"C:\\Program Files\\Opera\\opera.exe\" '%p'", "0", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Windows XP Safari"),
-											 "\"C:\\Program Files\\Safari\\safari.exe\" '%p'", "0", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Windows 7 Firefox 32-bit"),
-											 "\"C:\\Program Files (x86)\\Mozilla Firefox\\firefox.exe\" '%p'",
-											 "1", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Windows 7 Internet Explorer 32-bit"),
-											 "\"C:\\Program Files (x86)\\Internet Explorer\\iexplore.exe\" '%p'",
-											 "0", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Windows 7 Internet Explorer 64-bit"),
-											 "\"C:\\Program Files\\Internet Explorer\\iexplore.exe\" '%p'",
-											 "0", NULL));
-	}
-#elif PLATFORM_DARWIN
-	if (main_v->props.external_outputbox == NULL) {
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-						  array_from_arglist(_("make"), "([a-zA-Z0-9/_.-]+):([0-9]+):(.*)", "1", "2", "3",
-											 "cd %c && make|", NULL));
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-						  array_from_arglist(_("weblint HTML checker"),
-											 "([a-zA-Z0-9/_.-]+) \\(([0-9:]+)\\) (.*)", "1", "2", "3",
-											 "weblint '%f'|", NULL));
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-						  array_from_arglist(_("tidy HTML validator"), "line ([0-9]+) column [0-9]+ - (.*)",
-											 "-1", "1", "2", "perltidy -qe '%I'|", NULL));
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-						  array_from_arglist(_("javac"), "([a-zA-Z0-9/_.-]+):([0-9]+):(.*)", "1", "2", "3",
-											 "javac '%f'|", NULL));
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-						  array_from_arglist(_("xmllint XML checker"),
-											 "([a-zA-Z0-9/_.-]+)\\:([0-9]+)\\: (.*)", "1", "2", "3",
-											 "xmllint --noout --valid '%f'|", NULL));
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-						  array_from_arglist(_("php"), "(.*) in (/[a-zA-Z0-9/_.-]+) on line ([0-9]+)", "2",
-											 "3", "1", "php '%f'|", NULL));
-	}
-	if (main_v->props.external_filter == NULL) {
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter, array_from_arglist(_("Sort"), "|sort|", NULL));
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter,
-						  array_from_arglist(_("Sort / Uniq"), "|sort|uniq|", NULL));
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter,
-						  array_from_arglist(_("Tidy HTML"), "|perltidy -b|", NULL));
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter,
-						  array_from_arglist(_("Strip empty lines"), "|egrep -v '^[ \t]*$'|", NULL));
-	}
-	if (main_v->props.external_command == NULL) {
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Safari"), "open -a Safari '%p'", "0", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Firefox"), "open -a Firefox '%p'", "1", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Opera"), "open -a Opera '%p'", "0", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("chmod a+x"), "chmod a+x %f", "0", NULL));
-	}
-#else
-	if (main_v->props.external_outputbox == NULL) {
-		/* if the user does not have outputbox settings --> set them to defaults values */
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-						  array_from_arglist(_("make"), "([a-zA-Z0-9/_.-]+):([0-9]+):(.*)", "1", "2", "3",
-											 "cd %c && make|", NULL));
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-						  array_from_arglist(_("weblint HTML checker"),
-											 "([a-zA-Z0-9/_.-]+) \\(([0-9:]+)\\) (.*)", "1", "2", "3",
-											 "weblint '%f'|", NULL));
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-						  array_from_arglist(_("tidy HTML validator"), "line ([0-9]+) column [0-9]+ - (.*)",
-											 "-1", "1", "2", "tidy -qe '%I'|", NULL));
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-						  array_from_arglist(_("javac"), "([a-zA-Z0-9/_.-]+):([0-9]+):(.*)", "1", "2", "3",
-											 "javac '%f'|", NULL));
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-						  array_from_arglist(_("xmllint XML checker"),
-											 "([a-zA-Z0-9/_.-]+)\\:([0-9]+)\\: (.*)", "1", "2", "3",
-											 "xmllint --noout --valid '%f'|", NULL));
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-						  array_from_arglist(_("php"), "(.*) in (/[a-zA-Z0-9/_.-]+) on line ([0-9]+)", "2",
-											 "3", "1", "php '%f'|", NULL));
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-						  array_from_arglist(_("php syntax check"), "(.*) in (/[a-zA-Z0-9/_.-]+) on line ([0-9]+)", "2",
-											 "3", "1", "php -l -q -f '%f'|", NULL));
-		main_v->props.external_outputbox =
-			g_list_append(main_v->props.external_outputbox,
-							array_from_arglist(_("perl syntax check"), "(.*) at (/[a-zA-Z0-9/_.-]+) line ([0-9]+)", "2",
-											 "3", "1", "perl -c '%f'|", NULL));
-	}
-	if (main_v->props.external_filter == NULL) {
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter, array_from_arglist(_("Sort"), "|sort|", NULL));
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter,
-						  array_from_arglist(_("Sort / Uniq"), "|sort|uniq|", NULL));
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter,
-						  array_from_arglist(_("Reverse lines"), "|tac|", NULL));
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter,
-						  array_from_arglist(_("Dos2unix"), "|dos2unix|", NULL));
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter, array_from_arglist(_("Tidy HTML"), "|tidy|", NULL));
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter,
-						  array_from_arglist(_("Strip empty lines"), "|egrep -v '^[ \t]*$'|", NULL));
-		main_v->props.external_filter =
-			g_list_append(main_v->props.external_filter,
-						  array_from_arglist(_("Render HTML to text"), "lynx -force_html -dump %i |", NULL));
-	}
-	if (main_v->props.external_command == NULL) {
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Firefox"), "firefox -remote 'openURL(%p)' || firefox '%p'&",
-											 "1", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Opera"), "opera -remote 'openURL(%p)' || opera '%p'&", "0",
-											 NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Chrome"), "chromium-browser '%p'&", "0", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Mozilla"), "mozilla -remote 'openURL(%p)' || mozilla '%p'&",
-											 "0", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Konqueror"), "konqueror '%p'&", "0", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Galeon"), "galeon -n  '%p'&", "0", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Epiphany"), "epiphany-browser -n '%p'&", "0", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Kazehakase"), "kazehakase '%p'&", "0", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("Links2 (graphics)"), "links2 -g '%p'&", "0", NULL));
-		main_v->props.external_command =
-			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("chmod a+x"), "chmod a+x %f", "0", NULL));
-	}
-#endif							/* ifdef WIN32 elif PLATFORM_DARWIN */
+
 	if (main_v->props.templates == NULL) {
 		main_v->props.templates =
 			g_list_append(main_v->props.templates,
@@ -847,6 +893,10 @@ rcfile_save_main(void)
 {
 	gint ret;
 	GFile *filename = user_bfdir(CURCONFIG);
+	
+	g_free(main_v->props.config_version);
+	main_v->props.config_version = g_strdup(VERSION);
+	
 	ret = save_config_file(main_configlist, filename);
 	g_object_unref(filename);
 	return ret;
@@ -856,22 +906,29 @@ void
 rcfile_check_directory(void)
 {
 	GFile *file, *rcpath;
+	GFileInfo *finfo;
 	GError *gerror = NULL;
 
 	file = g_file_new_for_path(g_get_home_dir());
 	rcpath = g_file_get_child(file, ".bluefish");
 
 	DEBUG_PATH("test if rcfile directory %s exists\n", rcpath);
-	if (g_file_query_file_type(rcpath, G_FILE_QUERY_INFO_NONE, NULL) == G_FILE_TYPE_UNKNOWN) {
+	finfo = g_file_query_info(rcpath, G_FILE_ATTRIBUTE_STANDARD_TYPE, G_FILE_QUERY_INFO_NONE, NULL, &gerror);
+	if (gerror && gerror->code == G_IO_ERROR_NOT_FOUND) {
+		g_clear_error(&gerror);
+
 		g_file_make_directory(rcpath, NULL, &gerror);
 		if (gerror) {
-			/* Do something with error */
+			g_warning("Error creating directory: .bluefish\n");
+			g_warning("%s", gerror->message);
 			g_error_free(gerror);
 		}
 	}
 
 	g_object_unref(rcpath);
 	g_object_unref(file);
+	if (finfo)
+		g_object_unref(finfo);
 }
 
 static gchar *
@@ -883,14 +940,17 @@ get_user_accelmap(void)
 void
 rcfile_save_accelerators(void)
 {
+#ifndef MAC_INTEGRATION
 	gchar *shortcutfilename = get_user_accelmap();
 	gtk_accel_map_save(shortcutfilename);
 	g_free(shortcutfilename);
+#endif
 }
 
 void
 rcfile_load_accelerators(gboolean defaultmap)
 {
+#ifndef MAC_INTEGRATION
 	gchar *shortcutfilename;
 	if (defaultmap) {
 		shortcutfilename = g_strdup(PKGDATADIR"/default_accelmap");
@@ -899,6 +959,7 @@ rcfile_load_accelerators(gboolean defaultmap)
 	}
 	gtk_accel_map_load(shortcutfilename);
 	g_free(shortcutfilename);
+#endif
 }
 
 static GHashTable *
@@ -1104,10 +1165,6 @@ rcfile_save_global_session(void)
 	configlist = return_globalsession_configlist(FALSE);
 	configlist = return_session_configlist(configlist, main_v->session);
 	DEBUG_MSG("rcfile_save_global_session, saving global session to %s\n", g_file_get_parse_name(filename));
-	DEBUG_MSG("rcfile_save_global_session, length session recent_files=%d\n",
-			  g_list_length(main_v->session->recent_files));
-	DEBUG_MSG("rcfile_save_global_session, length session recent_projects=%d\n",
-			  g_list_length(main_v->globses.recent_projects));
 	retval = save_config_file(configlist, filename);
 	free_configlist(configlist);
 	g_object_unref(filename);
@@ -1141,11 +1198,11 @@ rcfile_parse_global_session(void)
 		/*arr = array_from_arglist(_("Java programming"),"1", "java:image:jsp", NULL);
 		   main_v->globses.filefilters = g_list_append(main_v->globses.filefilters, arr); */
 		arr = array_from_arglist(_("Images"), "1", "image/png:image/jpeg:image/gif:image/tiff", "", NULL);
-		main_v->globses.filefilters = g_list_append(main_v->globses.filefilters, arr);
+		main_v->globses.filefilters = g_list_prepend(main_v->globses.filefilters, arr);
 		arr =
 			array_from_arglist(_("Hide objectfiles"), "0", "application/octet-stream:application/x-object",
 							   "", NULL);
-		main_v->globses.filefilters = g_list_append(main_v->globses.filefilters, arr);
+		main_v->globses.filefilters = g_list_prepend(main_v->globses.filefilters, arr);
 	}
 	if (main_v->globses.encodings == NULL) {
 		GFile *defaultfile =
